/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
// $Id: chatsndwin.cpp,v 1.3 2000/06/17 17:15:26 denis Exp $
// $Log: chatsndwin.cpp,v $
// Revision 1.3  2000/06/17 17:15:26  denis
// Cleanups.
//
// Revision 1.2  2000/06/15 01:55:56  bills
// fixed to use renamed chat functions
//
// Revision 1.1  2000/05/24 16:56:50  denis
// Sources have been splitted to the whole bunch of appropriate
// directories. Every directory is end up with static library. All
// of them are linked together to the final kicq binary.
//
// Revision 1.11  2000/01/10 16:08:04  denis
// Headers reorganized for simplify dependencies and speedup compilation.
//
// Revision 1.10  1999/12/02 13:58:02  denis
// Cleanups.
// IcqWindow random segfaults fixed.
//
// Revision 1.9  1999/10/06 19:10:51  denis
// Cleanups.
//
// Revision 1.8  1999/10/04 16:48:26  bills
// ICQ_NOTIFY_CHATINIT -> ICQ_NOTIFY_CHAT
//
// Revision 1.7  1999/10/01 21:53:03  guruz
// *** empty log message ***
//
// Revision 1.6  1999/07/16 15:47:41  denis
// Cleaned up.
//
// Revision 1.5  1999/07/16 12:59:36  denis
// Internalization support added.
// Smart loading of spinning face animation implemented.
//
// Revision 1.4  1999/07/12 15:23:17  cproch
// - changed icq_* function calls and callback function to support
//   icqlib with ICQLINK identifier.
// - look for FIXME in mainwindow.cpp
//
// Revision 1.3  1999/04/17 20:22:11  bills
// modified for new icqwindow base class
//
// Revision 1.2  1999/04/14 15:35:05  denis
// Commented out unused variable in sendChatReq()
//
// Revision 1.1  1999/04/04 21:57:11  bills
// added for chat support
//
// Revision 1.8  1999/03/31 01:27:08  bills
// added tcpflag check to sendURL
//
// Revision 1.7  1999/03/28 02:02:37  bills
// fixed number of args to icq_SendURL
//
// Revision 1.6  1998/12/08 16:33:51  denis
// Cleaned up a little before releasing
//

#include <icq.h>
#include <qmsgbox.h>
#include <qmovie.h>
#include <qregexp.h>
#include <qkeycode.h>
#include <kapp.h>
#include <kiconloader.h>

#include "chatwindow.h"
#include "chatsndwin.h"
#include "mainwindow.h"
#include "userlist.h"
#include "icqmessage.h"

#include "chatsndwin.moc"

ChatSndWindow::ChatSndWindow(const char *n, unsigned long u, const char *em) : IcqWindow()
{
  QString text;
  QPixmap infopix;
  KIconLoader *il = kapp->getIconLoader();
  setCaption(i18n("Send Chat Request"));
  tobox = new QGroupBox(i18n("To"),this);
  sendbutton = new QPushButton(i18n("Send"),this);
  sendbutton->setAccel(Key_Return+CTRL) ;
  connect(sendbutton, SIGNAL(clicked()),SLOT(sendChatReq()));
  cancelbutton = new QPushButton(i18n("Cancel"),this);
  connect(cancelbutton, SIGNAL(clicked()),SLOT(cancelChatReq()));
  uinline = new QLineEdit(this);
  uinline->setFocusPolicy(QWidget::NoFocus);
  uinlabel = new QLabel(uinline,i18n("ICQ#:"),this);
  handleline = new QLineEdit(this);
  handleline->setFocusPolicy(QWidget::NoFocus);
  handlelabel = new QLabel(handleline,i18n("Handle:"),this);
  emailline = new QLineEdit(this);
  emailline->setFocusPolicy(QWidget::NoFocus);
  emaillabel = new QLabel(emailline,i18n("EMail:"),this);
  infobutton = new QPushButton(this);
  infopix = il->loadIcon("info.xpm");
  infobutton->setPixmap(infopix);
  infobutton->resize(infopix.width()+6, infopix.height()+6);
  connect(infobutton, SIGNAL(clicked()), SLOT(infoRequest()));
  editbox = new QMultiLineEdit(this);
  editlabel = new QLabel(editbox,i18n("Enter Chat Description:"),this);
  updateanim = new QLabel(this);
  QString filename = kapp->localkdedir()+"/share/apps/"+kapp->appName()+"/pics/search.gif";
  QFile testfile(filename);
  if(!testfile.exists())
    filename = kapp->kde_datadir()+"/"+kapp->appName()+"/pics/search.gif";
  updateanim->setMovie(QMovie((const char*)filename));
  updateanim->setFrameStyle(QFrame::WinPanel | QFrame::Sunken);
  updateanim->setAlignment(AlignCenter);
  updateanim->movie()->pause();
  setName(n);
  setUIN(u);
  setEMail(em);
  editbox->setFocus();
  resize(350,250);
  setMinimumSize(350,250);
}

ChatSndWindow::~ChatSndWindow()
{
  if(sendbutton)
    delete sendbutton;
  if(cancelbutton)
    delete cancelbutton;
  if(tobox)
    delete tobox;
  if(uinlabel)
    delete uinlabel;
  if(handlelabel)
    delete handlelabel;
  if(emaillabel)
    delete emaillabel;
  if(uinline)
    delete uinline;
  if(handleline)
    delete handleline;
  if(emailline)
    delete emailline;
  if(infobutton)
    delete infobutton;
  if(editbox)
    delete editbox;
  if(editlabel)
    delete editlabel;
  if(updateanim)
    delete updateanim;
}

void ChatSndWindow::resizeEvent(QResizeEvent *re)
{
  IcqWindow::resizeEvent(re);
  QSize uinlblsize,handlelblsize,emaillblsize,toboxsize,editlblsize,btn1size,btn2size;
  toboxsize.setWidth(width()-6);
  uinlblsize = uinlabel->sizeHint();
  handlelblsize = handlelabel->sizeHint();
  emaillblsize = emaillabel->sizeHint();
  int wl = QMAX(uinlblsize.width(),emaillblsize.width());
  int w = (toboxsize.width()-uinlblsize.width()-handlelblsize.width()-36)/2;
  uinline->resize(w,uinline->sizeHint().height());
  handleline->resize(w,uinline->sizeHint().height());
  emailline->resize(toboxsize.width()-infobutton->width()-62,uinline->sizeHint().height());
  uinlabel->resize(wl,uinline->height());
  handlelabel->resize(handlelblsize.width(),handleline->height());
  emaillabel->resize(wl,emailline->height());
  uinlabel->move(12,15);
  uinline->move(3+uinlabel->x()+uinlabel->width(),15);
  handlelabel->move(12+uinline->x()+uinline->width(),15);
  handleline->move(3+handlelabel->x()+handlelabel->width(),15);
  emaillabel->move(12,20+uinline->height());
  emailline->move(3+emaillabel->x()+emaillabel->width(),20+uinline->height());
  tobox->setGeometry(3,3,toboxsize.width(),uinline->height()+emailline->height()+23);
  infobutton->move(7+emailline->x()+emailline->width(),
                   emailline->y()+(emailline->height()-infobutton->height())/2);
  btn1size = sendbutton->sizeHint();
  btn2size = cancelbutton->sizeHint();
  int h = QMAX(btn1size.height(),btn2size.height());
  w = QMAX(QMAX(btn1size.width(),btn2size.width()),75);
  sendbutton->setGeometry(width()/2-w-6,height()-h-12,w,h);
  cancelbutton->setGeometry(width()/2+6,height()-h-12,w,h);

  editlblsize = editlabel->sizeHint();
  editlabel->setGeometry(6,8+tobox->height(),editlblsize.width(),editlblsize.height());
  editbox->setGeometry(3,editlabel->y()+editlabel->height()+4,width()-6,height()-editlabel->y()-
                       editlabel->height()-h-28);
  updateanim->setGeometry(width()-40,height()-40,32,32);
  updateanim->movie()->restart();
}

void ChatSndWindow::setUIN(unsigned long u)
{
  uin = u;
  QString text;
  text.setNum(uin);
  uinline->setText(text);
}

void ChatSndWindow::setName(const char *name)
{
  handleline->setText(name);
}

void ChatSndWindow::setEMail(const char *email)
{
  emailline->setText(email);
}

void ChatSndWindow::sendChatReq()
{
  QString tmp;
  if(MW->isLogged())
  {
    tmp = editbox->text();
    if(MW->isConvertCRLF())
      tmp.replace(QRegExp("\n"),"\r\n");
    sequence = icq_SendChatRequest(&MW->icqlink, uin, (const char*)editbox->text());
    updateanim->movie()->unpause();
    sendbutton->setEnabled(FALSE);
  }
  else
    QMessageBox::warning(this, i18n("Error"), i18n("You are not connected to the icq network!"));
}

void ChatSndWindow::cancelChatReq()
{
  if(sequence==0)
    close();
  else
    stopCommand();
}

void ChatSndWindow::infoRequest()
{
  MW->infoWindow(uin);
}

void ChatSndWindow::stopCommand()
{
  updateanim->movie()->pause();
  sendbutton->setEnabled(TRUE);
  sequence = 0;
}

void ChatSndWindow::saveHistory()
{
  time_t curtm = time(0L);
  struct tm *t = localtime(&curtm);
  TimeStr time(t->tm_hour,t->tm_min,t->tm_mday,t->tm_mon+1,t->tm_year+1900) ;

  icqMessage msg(uin,time,editbox->text(),MSG_URL_SENT) ;
  QString historypath = MW->contactsDir() ;
  msg.save(historypath) ;
  MW->updateHistoryWindows(msg.uin()) ;
}

void ChatSndWindow::requestNotify(int type, int length, void *data)
{
  if(type==ICQ_NOTIFY_CHAT)
  {
    ChatWindow *p = new ChatWindow(seq(), uin);
    p->show();
    close();
  }
}
