/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
// $Id: chatwindow.cpp,v 1.1 2000/05/24 16:56:50 denis Exp $
// $Log: chatwindow.cpp,v $
// Revision 1.1  2000/05/24 16:56:50  denis
// Sources have been splitted to the whole bunch of appropriate
// directories. Every directory is end up with static library. All
// of them are linked together to the final kicq binary.
//
// Revision 1.12  2000/02/07 03:01:21  bills
// added code to understand icq chat escape sequences
//
// Revision 1.11  2000/01/10 16:08:06  denis
// Headers reorganized for simplify dependencies and speedup compilation.
//
// Revision 1.10  1999/12/02 13:58:35  denis
// Cleanups.
// IcqWindow random segfaults fixed.
//
// Revision 1.9  1999/10/14 12:46:18  denis
// Cleanups.
//
// Revision 1.8  1999/09/29 17:30:51  denis
// Cleanups.
//
// Revision 1.7  1999/07/16 15:47:43  denis
// Cleaned up.
//
// Revision 1.6  1999/07/16 13:01:00  denis
// Internalization support added.
//
// Revision 1.5  1999/07/12 15:23:18  cproch
// - changed icq_* function calls and callback function to support
//   icqlib with ICQLINK identifier.
// - look for FIXME in mainwindow.cpp
//
// Revision 1.4  1999/06/30 13:58:52  bills
// cleanups, added resize functionality
//
// Revision 1.3  1999/04/17 20:22:13  bills
// modified for new icqwindow base class
//
// Revision 1.2  1999/04/14 15:36:04  denis
// Cleanups.
//
// Revision 1.1  1999/04/04 21:57:12  bills
// added for chat support
//

#include <qmsgbox.h>
#include <qmovie.h>
#include <qregexp.h>
#include <qkeycode.h>
#include <kapp.h>
#include <kiconloader.h>

#include "chatwindow.h"
#include "mainwindow.h"
#include "chatedit.h"
#include "wrapedit.h"
#include "chatedit.h"

#include "chatwindow.moc"

ChatWindow::ChatWindow(unsigned long id, unsigned long uin) : IcqWindow()
{
  sequence=id;
  remote_uin=uin;

  splitter=new QSplitter(QSplitter::Vertical, this);
  splitter->setOpaqueResize(true);

  localbox=new ChatEdit(splitter);
  localbox->remote_uin=remote_uin;
  localbox->setLimit(0);

  remotebox=new WrapEdit(splitter);

  localbox->setFocus();

  resize(500,250);
  setMinimumSize(400,250);

  splitter->resize(500,250);
}

ChatWindow::~ChatWindow()
{
  //
}

void ChatWindow::requestNotify(int type, int length, void *data)
{
  if(type==ICQ_NOTIFY_CHATDATA)
    textReceived(length, data);
  if(type==ICQ_NOTIFY_SUCCESS)
  {
    sequence=0;
    QMessageBox::information(this, i18n("ICQ Chat"), i18n("Last user has left chat"));
  }

  IcqWindow::requestNotify(type, length, data);
}

void ChatWindow::textReceived(int length, void *data)
{
  int row,col,i;

  for(i=0; i < length;)
  {
    QString qs;
    char ch, *p;

    remotebox->cursorPosition(&row, &col);
    p = ((char *)data) + i++;
    ch = *p++;
    switch(ch)
    {
      case '\x00': /* Set text color - RR GG BB 00 */
           {
             QPalette mypalette = (remotebox->palette()).copy();
             QColorGroup cgrp = mypalette.normal();
             QColor c (*(unsigned char *)p, *(unsigned char *)(p+1), *(unsigned char *)(p+2));
             QColorGroup ncgrp (c, cgrp.background(), cgrp.light(), cgrp.dark(), cgrp.mid(), c, cgrp.base());
             mypalette.setNormal(ncgrp);
             mypalette.setDisabled(ncgrp);
             mypalette.setActive(ncgrp);
             remotebox->setPalette(mypalette);
             remotebox->setBackgroundColor(cgrp.base());
           }
           i += 4;
           break;
      case '\x01': /* Set background color - RR GG BB 00 */
           {
             QPalette mypalette = (remotebox->palette()).copy();
             QColorGroup cgrp = mypalette.normal();
             QColor c (*(unsigned char *)p, *(unsigned char *)(p+1), *(unsigned char *)(p+2));
             QColorGroup ncgrp (cgrp.foreground(), cgrp.background(), cgrp.light(), cgrp.dark(), cgrp.mid(), cgrp.text(), c);
             mypalette.setNormal(ncgrp);
             mypalette.setDisabled(ncgrp);
             mypalette.setActive(ncgrp);
             remotebox->setPalette(mypalette);
             remotebox->setBackgroundColor(c);
           }
           i += 4;
           break;
      case '\x11': /* Font style (Bold=1, Italic=2, Underline=4) */
           {
             QFont f = remotebox->font();
             int curattr = 0;
             int newattr = *(int *)p;
             int chgattr;
             if(f.bold())
               curattr |= 1;
             if(f.italic())
               curattr |= 2;
             if(f.underline())
               curattr |= 4;
             if((chgattr = curattr ^ newattr) != 0)
             {
               if(chgattr & 1)
                 f.setBold(newattr & 1);
               if(chgattr & 2)
                 f.setItalic(newattr & 2);
               if(chgattr & 4)
                 f.setUnderline(newattr & 4);
               remotebox->setFont(f);
             }
           }
           i += 4;
           break;
      case '\x12': /* Font point size */
           {
             QFont f = remotebox->font();
             if(f.pointSize() != *(int *)p)
             {
               f.setPointSize(*(unsigned int *)p);
               remotebox->setFont(f);
             }
           }
           i += 4;
           break;
      case '\x10': /* Font family and encoding */
           {
             QFont f = remotebox->font();
             
             if(strcasecmp(f.family(), p + 2) != 0)
             {
               f.setFamily(p + 2);
               remotebox->setFont(f);
             }
           }
           i += *p + 2 + 2;
           break;
      case '\x03': /* Chat is active */
      case '\x04': /* Chat is inactive */
      case '\b':   /* Backspace */
           if(col > 0)
           {
             /* Very ugly but works */
             bool _autoUpdate = remotebox->autoUpdate();
             char *line = strdup(remotebox->textLine(row));
             strcpy(&line[col - 1], &line[col]);
             remotebox->setAutoUpdate(FALSE);
             remotebox->removeLine(row);
             remotebox->insertLine(line, row);
             remotebox->setCursorPosition(row, col - 1);
             remotebox->setAutoUpdate(_autoUpdate);
           }
           break;
      case '\x07': /* Bell */
           kapp->beep();
           break;
      default:
           qs.fill((ch == '\x0D' ? '\x0A' : ch), 1);
           remotebox->insertAt(qs, row, col);
    }
  }
}

void ChatWindow::closeEvent(QCloseEvent *ce)
{
  IcqWindow::closeEvent(ce);
  icq_TCPCloseChat(&MW->icqlink, remote_uin);
//  delete this;
}

void ChatWindow::resizeEvent(QResizeEvent *re)
{
  IcqWindow::resizeEvent(re);
  splitter->setGeometry(0, 0, width(), height());
}

/*
void MsgWindow::setUIN(unsigned long u)
{
  uin = u;
  QString text;
  text.setNum(uin);
  uinline->setText(text);
}

void MsgWindow::setName(const char *name)
{
  handleline->setText(name);
}

void MsgWindow::setEMail(const char *email)
{
  emailline->setText(email);
}

void MsgWindow::setMessage(const char *msg)
{
  message = msg;
}

void MsgWindow::sendMessage()
{
  ContactItem *item = MW->listWidget()->getInfoByUIN(uin);
  QString tmp;
  if(MW->isLogged())
  {
    tmp = editbox->text();
    if(MW->isConvertCRLF())
      tmp.replace(QRegExp("\n"),"\r\n");
    sendSeq = icq_SendMessage(uin,(const char*)tmp,(item->tcpflag!=0x04 ? 1 : 0));
    updateanim->movie()->unpause();
    sendbutton->setEnabled(FALSE);
  }
  else
    QMessageBox::warning(this, i18n("Error"), i18n("You are not connected to the icq network!"));
}

void MsgWindow::cancelMessage()
{
  if(sendSeq==-1)
  {
    hide();
    MW->delSendWin(this);
  }
  else
    stopCommand();
}

void MsgWindow::infoRequest()
{
  ContactItem tmpitem, *item = MW->listWidget()->getInfoByUIN(uin);
  if(item)
    if(MW->listWidget()->isUINNotInList(uin))
      MW->infoShow(*item, FALSE);
    else
      MW->infoWindow(uin);
  else
  {
    tmpitem.uin = uin;
    MW->infoShow(tmpitem, FALSE);
  }
}

long MsgWindow::seq()
{
  return sendSeq;
}

void MsgWindow::stopCommand()
{
  updateanim->movie()->pause();
  sendbutton->setEnabled(TRUE);
  sendSeq = -1;
}

void MsgWindow::saveHistory()
{
  time_t curtm = time(0L);
  struct tm *t = localtime(&curtm);
  TimeStr time(t->tm_hour,t->tm_min,t->tm_mday,t->tm_mon+1,t->tm_year+1900) ;

  icqMessage msg(uin,time,editbox->text(),MSG_SENT) ;
  QString historypath = MW->contactsDir() ;
  msg.save(historypath) ;
  MW->updateHistoryWindows(msg.uin()) ;
}

void MsgWindow::moreOptions()
{
  int mwHeight = height() ;
  QSize multiSize = multibutton->sizeHint() ;

  if(morebutton->isOn())
  {
    mwHeight += multiSize.height() * 2 ;
    multibutton->show() ;
  }
  else
  {   
    mwHeight -= multiSize.height() * 2 ;
    multibutton->hide() ;
  }
   
  resize(width(),mwHeight) ;
}

void MsgWindow::multiRecpts()
{
  int mwWidth = width() ;
  QSize listSize = multilist->sizeHint() ;
  QSize boxSize = multibox->sizeHint() ;

  if(multibutton->isOn())
  {
    mwWidth += QMAX(listSize.width(),boxSize.width()) + 16 ;
    multilist->show() ;
    multibox->show() ;
  }
  else
  {
    mwWidth -= QMAX(listSize.width(),boxSize.width()) + 16 ;
    multilist->hide() ;
    multibox->hide() ;
  }

  resize(mwWidth,height()) ;
}
*/
