/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
// $Id: blinklistbox.cpp,v 1.2 2000/06/17 17:38:03 denis Exp $
// $Log: blinklistbox.cpp,v $
// Revision 1.2  2000/06/17 17:38:03  denis
// Contact List code was rewritten from scratch! Although we are trying
// to not add new features in anticipation of the next stable release,
// but this rewrite solves many things, removes many hacks and stabilizes
// whole program.
// One of bug, needs to mention, is an absence of IP address in Info
// window. The root of problem was in Contact List, because in any time
// there were several ContactItems in memory for single user - they were
// "smeared" on the whole code. Now it's clean and strict - all Contact
// List operations can be done only by ContactList class.
//
// Revision 1.1  2000/05/24 16:57:03  denis
// Sources have been splitted to the whole bunch of appropriate
// directories. Every directory is end up with static library. All
// of them are linked together to the final kicq binary.
//
// Revision 1.10  2000/05/04 15:35:26  denis
// "Update nonexistent icon in the contact list" bug fixed.
//
// Revision 1.9  2000/03/30 15:09:49  denis
// ANSI fixes.
//
// Revision 1.8  2000/01/10 16:08:01  denis
// Headers reorganized for simplify dependencies and speedup compilation.
//
// Revision 1.7  1999/12/02 13:56:51  denis
// Cleanups.
//
// Revision 1.6  1999/07/16 15:47:37  denis
// Cleaned up.
//
// Revision 1.5  1998/12/08 16:08:45  denis
// setToolTip moved to MainWindow
// Cleaned up a little before releasing
//

#include "mainwindow.h"
#include "blinklistbox.h"

/*********************/
/* WBlinkListBoxItem */
/*********************/

WBlinkListBoxItem::WBlinkListBoxItem(const char *s) : VirtUserItem()
{
  activePixmapNo = 0;
  isBlinkingFlag = 0;
  setText(s);
}

WBlinkListBoxItem::WBlinkListBoxItem() : VirtUserItem()
{
  activePixmapNo = 0;
  isBlinkingFlag = 0;
}

void WBlinkListBoxItem::setPixmap(int n, const QPixmap &p)
{
  if(n == 0)
    pmap1 = p;
  if(n == 1)
    pmap2 = p;
}

void WBlinkListBoxItem::setPixmapXY(int x, int y)
{
  pmapX = x;
  pmapY = y;
}

void WBlinkListBoxItem::setPaintMode(blPaintMode m)
{
  paintMode = m;
}

const QPixmap* WBlinkListBoxItem::activePixmap() const
{
  const QPixmap *pm;
  if(activePixmapNo == 0)
    pm = &pmap1;
  else
    pm = &pmap2;
  return pm;
}

void WBlinkListBoxItem::paint(QPainter *p)
{
  const QPixmap *pm = activePixmap();
  if(isBlinkingFlag || paintMode == FREEZE)
    p->drawPixmap(pmapX, pmapY, *pm);
  QFontMetrics fm = p->fontMetrics();
  int yPos;
  if(pixmapMaxHeight() < fm.height())
    yPos = fm.ascent() + fm.leading()/2;
  else
    yPos = pixmapMaxHeight()/2 - fm.height()/2 + fm.ascent();
  p->drawText(pixmapMaxWidth() + 6, yPos, text() );
}

int WBlinkListBoxItem::togglePixmap()
{
  if(isBlinkingFlag && activePixmapNo == 0)
    activePixmapNo = 1;
  else 
    activePixmapNo = 0;
  return activePixmapNo;
}

void WBlinkListBoxItem::resetPixmap()
{
  activePixmapNo = 0;
}

const QPixmap* WBlinkListBoxItem::pixmap()
{
  return activePixmap();
}

int WBlinkListBoxItem::pixmapMaxWidth() const
{
  return QMAX(pmap1.width(), pmap2.width());
}

int WBlinkListBoxItem::pixmapMaxHeight() const
{
  return QMAX(pmap1.height(), pmap2.height());
}

void WBlinkListBoxItem::setBlinking(int b)
{
  isBlinkingFlag = b;
}

int WBlinkListBoxItem::isBlinking()
{
  return isBlinkingFlag;
}

int WBlinkListBoxItem::height(const QListBox *lb) const
{
  return QMAX(pixmapMaxHeight(), lb->fontMetrics().lineSpacing() + 1);
}

int WBlinkListBoxItem::width(const QListBox *lb) const
{
  return pixmapMaxWidth() + lb->fontMetrics().width(text()) + 6;
}
