/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * $Id: contactitem.cpp,v 1.2 2000/07/28 18:34:41 denis Exp $
 * $Log: contactitem.cpp,v $
 * Revision 1.2  2000/07/28 18:34:41  denis
 * Code for setting per-user "Send Thru Server" status added.
 *
 * Revision 1.1  2000/06/17 17:38:03  denis
 * Contact List code was rewritten from scratch! Although we are trying
 * to not add new features in anticipation of the next stable release,
 * but this rewrite solves many things, removes many hacks and stabilizes
 * whole program.
 * One of bug, needs to mention, is an absence of IP address in Info
 * window. The root of problem was in Contact List, because in any time
 * there were several ContactItems in memory for single user - they were
 * "smeared" on the whole code. Now it's clean and strict - all Contact
 * List operations can be done only by ContactList class.
 *
 */

#include <icq.h>

#include "mainwindow.h"
#include "contactitem.h"

ContactItem::ContactItem(unsigned long puin, QString contdir) :
  uin(puin), nick("N/A"), alias(""), firstName("N/A"), lastName("N/A"), email("N/A"),
  lastIp(0), realIp(0), port(0), tcpflag(0), city("N/A"), state("N/A"), zip("N/A"),
  country(0), countryStat(0), homePage("N/A"), age(0), phone("N/A"), gender(0),
  about("N/A"), notes(""), status((unsigned long)STATUS_OFFLINE), auth(0),
  historyDir(""), msglast(LAST_NONE), onlineAlert(0), visibleUser(0), thruServer(0)
{
  load(uin, contdir);
}

ContactItem::ContactItem(unsigned long puin) :
  uin(puin), nick("N/A"), alias(""), firstName("N/A"), lastName("N/A"), email("N/A"),
  lastIp(0), realIp(0), port(0), tcpflag(0), city("N/A"), state("N/A"), zip("N/A"),
  country(0), countryStat(0), homePage("N/A"), age(0), phone("N/A"), gender(0),
  about("N/A"), notes(""), status((unsigned long)STATUS_OFFLINE), auth(0),
  historyDir(""), msglast(LAST_NONE), onlineAlert(0), visibleUser(0), thruServer(0)
{
}

ContactItem::ContactItem(unsigned long puin, QString pnick, QString pfirst, QString plast,
                         QString pemail, unsigned char pauth) :
  uin(puin), nick(pnick), alias(""), firstName(pfirst), lastName(plast), email(pemail),
  lastIp(0), realIp(0), port(0), tcpflag(0), city("N/A"), state("N/A"), zip("N/A"),
  country(0), countryStat(0), homePage("N/A"), age(0), phone("N/A"), gender(0),
  about("N/A"), notes(""), status((unsigned long)STATUS_OFFLINE), auth(pauth),
  historyDir(""), msglast(LAST_NONE), onlineAlert(0), visibleUser(0), thruServer(0)
{
}

ContactItem::~ContactItem()
{
  messageList.clear();
}

void ContactItem::load(unsigned long puin, QString contdir)
{
  uin = puin;

  QString uinstr;
  uinstr.setNum(uin);

  QString uinfname = contdir+"/"+uinstr+".uin";

  KSimpleConfig usrcfg(uinfname);
  usrcfg.setGroup("Info");
  nick = usrcfg.readEntry("Nick", "N/A");
  email = usrcfg.readEntry("EMail", "N/A");
  firstName = usrcfg.readEntry("FirstName", "N/A");
  lastName = usrcfg.readEntry("LastName", "N/A");
  alias = usrcfg.readEntry("Alias","");
  if(alias.isEmpty())
    alias = calcAlias();
  city = usrcfg.readEntry("City", "N/A");
  state = usrcfg.readEntry("State", "N/A");
  zip = usrcfg.readEntry("Zip", "N/A");
  country = usrcfg.readNumEntry("Country", 0);
  countryStat = usrcfg.readNumEntry("CountryStatus", 0);
  homePage = usrcfg.readEntry("HomePage", "N/A");
  age = usrcfg.readUnsignedNumEntry("Age", 0);
  phone = usrcfg.readEntry("Phone", "N/A");
  gender = usrcfg.readUnsignedNumEntry("Gender", 0);
  onlineAlert = usrcfg.readNumEntry("OnlineAlert", 0);
  visibleUser = usrcfg.readNumEntry("VisibleUser", 0);
  status = (unsigned long)STATUS_OFFLINE;
  about = readAboutNotes(contdir+"/"+uinstr+".about");
  notes = readAboutNotes(contdir+"/"+uinstr+".notes");
}

void ContactItem::save(QString contdir)
{
  QString uinstr;
  uinstr.setNum(uin);

  KSimpleConfig usrcfg(contdir+"/"+uinstr+".uin");
  usrcfg.setGroup("Info");
  usrcfg.writeEntry("Nick", nick);
  usrcfg.writeEntry("Alias", alias);
  usrcfg.writeEntry("FirstName", firstName);
  usrcfg.writeEntry("LastName", lastName);
  usrcfg.writeEntry("EMail", email);
  usrcfg.writeEntry("City", city);
  usrcfg.writeEntry("State", state);
  usrcfg.writeEntry("Zip", zip);
  usrcfg.writeEntry("Country", (unsigned int)country);
  usrcfg.writeEntry("CountryStatus", countryStat);
  usrcfg.writeEntry("HomePage", homePage);
  usrcfg.writeEntry("Age", (unsigned int)age);
  usrcfg.writeEntry("Phone", phone);
  usrcfg.writeEntry("Gender", (int)gender);
  usrcfg.writeEntry("OnlineAlert", onlineAlert);
  usrcfg.writeEntry("VisibleUser", visibleUser);
  writeAboutNotes(contdir+"/"+uinstr+".about", about);
  writeAboutNotes(contdir+"/"+uinstr+".notes", notes);
  usrcfg.sync();
}

void ContactItem::remove(QString contdir)
{
  QString uinstr;
  uinstr.setNum(uin);

  unlink(contdir+"/"+uinstr+".uin");
  unlink(contdir+"/"+uinstr+".about");
  unlink(contdir+"/"+uinstr+".notes");
  unlink(contdir+"/"+uinstr+".history");
}

QString ContactItem::readAboutNotes(QString anfname)
{
  QString ret = "";
  QFile anfile(anfname);
  if(anfile.exists())
  {
    char *buf = new char[anfile.size()+1];
    anfile.open(IO_ReadOnly);
    anfile.readBlock(buf, anfile.size());
    buf[anfile.size()] = 0;
    ret = (const char*)buf;
    anfile.close();
    delete buf;
  }
  return ret;
}

void ContactItem::writeAboutNotes(QString anfname, QString txt)
{
  QFile anfile(anfname);
  if(anfile.exists())
    unlink(anfname);
  anfile.open(IO_WriteOnly | IO_Raw);
  anfile.writeBlock(txt, txt.length());
}

icqMessage *ContactItem::getMsg(int n)
{
  return messageList.at(n);
}

int ContactItem::getNumMsgs()
{
  return messageList.count();
}

void ContactItem::setMsgLast(lastEvent l)
{
  msglast = l;
}

void ContactItem::addMsg(icqMessage *msg)
{
  messageList.append(msg);
  if(msg->type() == MSG_RECV)
    setMsgLast(LAST_MSG);
  if(msg->type() == MSG_URL_RECV)
    setMsgLast(LAST_URL);
  if(msg->type() == MSG_AUTH_RECV)
    setMsgLast(LAST_AUTH);
  if(msg->type() == MSG_CHAT_REQ_RECV)
    setMsgLast(LAST_MSG);
  if(msg->type() == MSG_ADDED)
    setMsgLast(LAST_EXCLAM);
  if(msg->type() == MSG_FILE_REQ_RECV)
    setMsgLast(LAST_MSG);

  msg->save(MW->contactsDir());
  MW->updateHistoryWindows(msg->uin());
}

icqMsgType ContactItem::getFirstUnreadMsgType()
{
  icqMessage *msg;
  for(msg = messageList.first(); msg != 0L; msg = messageList.next())
    if(msg->isNew())
    {
      return msg->type();
    }
  return MSG_UNDEF;
}

int ContactItem::getNumUnreadMsgs()
{
  icqMessage *msg;
  int msgCount = 0;
  for(msg = messageList.first(); msg != 0L; msg = messageList.next())
    if(msg->isNew())
      msgCount++;
  return msgCount;
}

void ContactItem::updateInfo(QString pnick, QString pfirst, QString plast, QString pemail, char pauth)
{
  nick = pnick;
  firstName = pfirst;
  lastName = plast;
  email = pemail;
  auth = pauth;
  if(alias.isEmpty())
    alias = calcAlias();
}

void ContactItem::updateExtInfo(QString pcity, unsigned short pcountry_code,
                                char pcountry_stat, QString pstate, unsigned short page,
                                char pgender, QString pphone, QString php, QString pabout)
{
  city = pcity;
  state = pstate;
  country = pcountry_code;
  countryStat = pcountry_stat;
  homePage = php;
  age = page;
  phone = pphone;
  gender = pgender;
  about = pabout;
}

QString ContactItem::calcAlias() const
{
  if(!nick.isEmpty() && nick!="N/A")
    return nick;
  else if(!firstName.isEmpty() && firstName!="N/A")
    return firstName;
  else if(!lastName.isEmpty() && lastName!="N/A")
    return lastName;
  else if(!email.isEmpty() && email!="N/A")
    return email;
  else
  {
    QString ret;
    ret.setNum(uin);
    return ret;
  }
}

void ContactItem::setStatus(unsigned long pstatus)
{
  status = pstatus;
}

QString ContactItem::getName() const
{
  if(alias.isEmpty())
    return calcAlias();
  return alias;
}

void ContactItem::setLastIP(unsigned long ip)
{
  lastIp = ip;
}

void ContactItem::setRealIP(unsigned long ip)
{
  realIp = ip;
}

void ContactItem::setPort(unsigned short p)
{
  port = p;
}

void ContactItem::setTCPFlag(unsigned short t)
{
  tcpflag = t;
}

void ContactItem::setAlias(QString palias)
{
  alias = palias;
}

void ContactItem::setNotes(QString pnotes)
{
  notes = pnotes;
}

void ContactItem::setOnlineAlert(bool oa)
{
  onlineAlert = oa;
}

void ContactItem::setVisible(bool vu)
{
  visibleUser = vu;
}

void ContactItem::setThruServer(bool ts)
{
  thruServer = ts;
}
