/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * $Id: contactitem.h,v 1.2 2000/07/28 18:34:41 denis Exp $
 */

#ifndef _CONTACTITEM_
#define _CONTACTITEM_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <qlist.h>
#include <ksimpleconfig.h>

#include "icqmessage.h"

enum lastEvent {LAST_NONE, LAST_MSG, LAST_URL, LAST_AUTH, LAST_EXCLAM};

class ContactItem
{
protected:
  unsigned long uin;
  QString nick;
  QString alias;
  QString firstName;
  QString lastName;
  QString email;
  unsigned long lastIp; /* Current/Last IP address */
  unsigned long realIp; /* Real IP address */
  unsigned short port;
  unsigned char tcpflag;
  QString city;
  QString state;
  QString zip; /* Where can we get zip code? ;) */
  unsigned short country;
  char countryStat;
  QString homePage;
  unsigned char age;
  QString phone;
  unsigned char gender;
  QString about;
  QString notes;
  unsigned long status;
  unsigned char auth;
  QString historyDir;
  lastEvent msglast;
  bool onlineAlert;
  bool visibleUser;
  bool thruServer;
  QList<icqMessage> messageList;

  QString readAboutNotes(QString);
  void writeAboutNotes(QString, QString);
public:
  // empty contact info for the given UIN
  ContactItem(unsigned long puin);
  // load contact info from the given contact list path and for given UIN
  ContactItem(unsigned long puin, QString contdir);
  // construct contact info from the given details
  ContactItem(unsigned long puin, QString pnick, QString pfirst, QString plast,
              QString pemail, unsigned char pauth);
  ~ContactItem();

  void load(unsigned long, QString);
  void save(QString);
  void remove(QString);

  QString calcAlias() const;
  void addMsg(icqMessage*);

  unsigned long getUIN() const { return uin; }
  QString getNick() const { return nick; }
  QString getAlias() const { return alias; }
  QString getFirstName() const { return firstName; }
  QString getLastName() const { return lastName; }
  QString getEMail() const { return email; }
  unsigned long getLastIP() const { return lastIp; }
  unsigned long getRealIP() const { return realIp; }
  unsigned short getPort() const { return port; }
  unsigned char getTCPFlag() const { return tcpflag; }
  QString getCity() const { return city; }
  QString getState() const { return state; }
  QString getZip() const { return zip; }
  unsigned short getCountry() const { return country; }
  char getCountryStat() const { return countryStat; }
  QString getHomePage() const { return homePage; }
  unsigned char getAge() const { return age; }
  QString getPhone() const { return phone; }
  unsigned char getGender() const { return gender; }
  QString getAbout() const { return about; }
  QString getNotes() const { return notes; }
  unsigned long getStatus() const { return status; }
  unsigned char getAuth() const { return auth; }
  QString getHistoryDir() const { return historyDir; }
  lastEvent getMsgLast() const { return msglast; }
  bool isOnlineAlert() const { return onlineAlert; }
  bool isVisible() const { return visibleUser; }
  bool isThruServer() const { return thruServer; }
  QList<icqMessage> *getMsgList() { return &messageList; }

  QString getName() const;

  void setStatus(unsigned long);

  void setLastIP(unsigned long);
  void setRealIP(unsigned long);
  void setPort(unsigned short);
  void setTCPFlag(unsigned short);
  void setAlias(QString);
  void setNotes(QString);
  void setOnlineAlert(bool);
  void setVisible(bool);
  void setThruServer(bool);

  void updateInfo(QString, QString, QString, QString, char);
  void updateExtInfo(QString, unsigned short, char, QString,
                     unsigned short, char, QString, QString, QString);

  icqMessage *getMsg(int);
  int getNumMsgs();
  void setMsgLast(lastEvent);
  icqMsgType getFirstUnreadMsgType();
  int getNumUnreadMsgs();
};

#endif
