/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * $Id: contactlist.cpp,v 1.2 2000/07/28 18:34:41 denis Exp $
 * $Log: contactlist.cpp,v $
 * Revision 1.2  2000/07/28 18:34:41  denis
 * Code for setting per-user "Send Thru Server" status added.
 *
 * Revision 1.1  2000/06/17 17:38:03  denis
 * Contact List code was rewritten from scratch! Although we are trying
 * to not add new features in anticipation of the next stable release,
 * but this rewrite solves many things, removes many hacks and stabilizes
 * whole program.
 * One of bug, needs to mention, is an absence of IP address in Info
 * window. The root of problem was in Contact List, because in any time
 * there were several ContactItems in memory for single user - they were
 * "smeared" on the whole code. Now it's clean and strict - all Contact
 * List operations can be done only by ContactList class.
 *
 */

#include <stdlib.h>
#include <qdir.h>
#include <kapp.h>
#include <icq.h>

#include "contactlist.h"
#include "mainwindow.h"

ContactList::ContactList() :
  onlineListIterator(onlineList), offlineListIterator(offlineList),
  notInListIterator(notInList)
{
  onlineList.setAutoDelete(TRUE);
  offlineList.setAutoDelete(TRUE);
  notInList.setAutoDelete(TRUE);
}

ContactList::ContactList(QString dirname) :
  onlineListIterator(onlineList), offlineListIterator(offlineList),
  notInListIterator(notInList)
{
  onlineList.setAutoDelete(TRUE);
  offlineList.setAutoDelete(TRUE);
  notInList.setAutoDelete(TRUE);
  load(dirname);
}

ContactList::~ContactList()
{
  onlineList.clear();
  offlineList.clear();
  notInList.clear();
}

QStrList ContactList::getUINDirList()
{
  QDir dir(contdir);
  return *dir.entryList("*.uin", QDir::Files | QDir::Readable);
}

void ContactList::load(QString dirname)
{
  contdir = dirname;

  icq_ContactClear(&MW->icqlink);
  onlineList.clear();
  offlineList.clear();
  notInList.clear();

  QStrList usrlist = getUINDirList();
  for(unsigned int i = 0; i < usrlist.count(); i++)
  {
    QString uinstr = usrlist.at(i);
    uinstr.resize(uinstr.find(".")+1);
    unsigned long uin = strtoul(uinstr, 0, 10);

    ContactItem *item = new ContactItem(uin, contdir);

    offlineList.append(item);
    icq_ContactAdd(&MW->icqlink, item->getUIN());
    icq_ContactSetVis(&MW->icqlink, item->getUIN(), item->isVisible());
  }
  MW->updateVisibleList();
}

void ContactList::save()
{
}

void ContactList::setStatusByUIN(unsigned long uin, unsigned long status)
{
  ContactItem *ptr, *usritem;
  ptr = offlineList.first();
  while(ptr != 0L)
  {
    if(ptr->getUIN() == uin)
    {
      ptr->setStatus(status);
      if(status != (unsigned long)STATUS_OFFLINE)
      {
        usritem = offlineList.take();
        onlineList.insert(0, usritem);
      }
    }
    ptr = offlineList.next();
  }

  ptr = onlineList.first();
  while(ptr != 0L)
  {
    if(ptr->getUIN() == uin)
    {
      ptr->setStatus(status);
      if(status == (unsigned long)STATUS_OFFLINE)
      {
        usritem = onlineList.take();
        offlineList.insert(0, usritem);
      }
    }
    ptr = onlineList.next();
  }

  MW->updateVisibleList();
}

void ContactList::appendItem(ContactItem *item)
{
  item->setStatus((unsigned long)STATUS_OFFLINE);
  offlineList.append(item);
  item->save(contdir);
  MW->updateVisibleList();
}

void ContactList::appendNILItem(ContactItem *item)
{
  item->setStatus((unsigned long)STATUS_OFFLINE);
  notInList.append(item);
  MW->updateVisibleList();
}

void ContactList::deleteByUIN(unsigned long uin)
{
  ContactItem *ptr;
  ptr = offlineList.first();
  while(ptr != 0L)
  {
    if(ptr->getUIN() == uin)
    {
      ptr->remove(contdir);
      offlineList.removeRef(ptr);
    }
    ptr = offlineList.next();
  }

  ptr = onlineList.first();
  while(ptr != 0L)
  {
    if(ptr->getUIN() == uin)
    {
      ptr->remove(contdir);
      onlineList.removeRef(ptr);
    }
    ptr = onlineList.next();
  }

  MW->updateVisibleList();
}

void ContactList::deleteNILByUIN(unsigned long uin)
{
  ContactItem *ptr = notInList.first();
  while(ptr != 0L)
  {
    if(ptr->getUIN() == uin)
    {
      ptr->remove(contdir);
      notInList.removeRef(ptr);
    }
    ptr = notInList.next();
  }
  MW->updateVisibleList();
}

void ContactList::addToContactListByUIN(unsigned long uin)
{
  ContactItem *ptr, *usritem;
  ptr = notInList.first();
  while(ptr != 0L)
  {
    if(ptr->getUIN() == uin)
    {
      ptr->save(contdir);
      usritem = notInList.take();
      offlineList.insert(0, usritem);
    }
    ptr = notInList.next();
  }
  MW->updateVisibleList();
}

void ContactList::addNewToContactListByUIN(unsigned long uin)
{
  ContactItem *ptr = getInfoPtrByUIN(uin);
  if(ptr)
  {
    addToContactListByUIN(uin);
    if(MW->isLogged())
      icq_SendNewUser(&MW->icqlink, uin);
    MW->updateVisibleList();
  }
}

void ContactList::addMsg(unsigned long uin, icqMessage *msg)
{
  if(!msg)
    printf("UserList::addMsg(uin=%lu, msg=0L)\n", uin);
  ContactItem *ptr;
  for(ptr = offlineList.first(); ptr != 0L; ptr = offlineList.next())
    if(ptr->getUIN() == uin)
    {
      ptr->addMsg(msg);
      MW->updateVisibleList();
      return;
    }
  for(ptr = onlineList.first(); ptr != 0L; ptr = onlineList.next())
    if(ptr->getUIN() == uin)
    {
      ptr->addMsg(msg);
      MW->updateVisibleList();
      return;
    }

  /* User is not in the list */
  for(ptr = notInList.first(); ptr != 0L; ptr = notInList.next())
    if(ptr->getUIN() == uin)
    {
      ptr->addMsg(msg);
      MW->updateVisibleList();
      return;
    }

  /* User is not even in the 'Not in list' list :) */
  ContactItem *ci = new ContactItem(uin);
  notInList.append(ci);

  if(MW->isLogged())
    icq_SendInfoReq(&MW->icqlink, uin);

  ci->addMsg(msg);
  MW->updateVisibleList();
}

void ContactList::addMsg(unsigned long uin, TimeStr time, const char *body)
{
  icqMessage *msg = new icqMessage(uin,time,body,MSG_RECV);
  addMsg(uin, msg);
}

void ContactList::addURL(unsigned long uin, TimeStr time, const char *url, const char *desc)
{
  icqMessage *msg = new icqMessage(uin,time,url,desc,MSG_URL_RECV);
  addMsg(uin, msg);
}

void ContactList::addAuth(unsigned long uin, TimeStr time, const char *nick, const char *first, 
                       const char *last,const char *email,const char *reason)
{
  icqMessage *msg = new icqMessage(uin,time,nick,first,last,email,reason,MSG_AUTH_RECV) ;
  addMsg(uin, msg);
}

void ContactList::addAdded(unsigned long uin, TimeStr time, const char *nick, 
                        const char *first, const char *last,const char *email)
{
  icqMessage *msg = new icqMessage(uin,time,nick,first,last,email,MSG_ADDED);
  addMsg(uin, msg);
}

void ContactList::addPager(unsigned long uin, TimeStr time, const char *nick, const char *email,
                        const char *body)
{
  icqMessage *msg = new icqMessage(uin,time,nick,email,body,MSG_PAGER_RECV);
  addMsg(uin, msg);
}

void ContactList::addExpress(unsigned long uin, TimeStr time, const char *nick, const char *email,
                          const char *body)
{
  icqMessage *msg = new icqMessage(uin,time,nick,email,body,MSG_EXPRESS_RECV);
  addMsg(uin, msg);
}

void ContactList::addChatReq(unsigned long uin, TimeStr time, const char *desc, unsigned long seq)
{
  icqMessage *msg = new icqMessage(uin,time,desc,seq,MSG_CHAT_REQ_RECV);
  addMsg(uin, msg);
}

void ContactList::addFileReq(unsigned long uin, TimeStr time, const char *text,
                          const char *file, unsigned long size, unsigned long seq)
{
  icqMessage *msg = new icqMessage(uin,time,text,file,size,seq,MSG_FILE_REQ_RECV);
  addMsg(uin, msg);
}

void ContactList::updateInfoByUIN(unsigned long uin, QString nick, QString first,
                                  QString last, QString email, char auth)
{
  ContactItem *ptr = getInfoPtrByUIN(uin);
  if(ptr)
    ptr->updateInfo(nick, first, last, email, auth);
  if(isUINInContactList(uin))
    ptr->save(contdir);
  MW->updateVisibleList();
}

void ContactList::updateExtInfoByUIN(unsigned long uin, QString city, unsigned short country_code,
                                     char country_stat, QString state, unsigned short age,
                                     char gender, QString phone, QString hp, QString about)
{
  ContactItem *ptr = getInfoPtrByUIN(uin);
  if(ptr)
    ptr->updateExtInfo(city, country_code, country_stat, state, age, gender, phone, hp, about);
  if(isUINInContactList(uin))
    ptr->save(contdir);
  MW->updateVisibleList();
}

void ContactList::userOnlineByUIN(unsigned long uin, unsigned long status, unsigned long ip,
                                  unsigned short port, unsigned long r_ip, unsigned char tcp_flag)
{
  ContactItem *ptr = getInfoPtrByUIN(uin);
  if(ptr)
  {
    setStatusByUIN(uin, status);
    ptr->setLastIP(ip);
    ptr->setRealIP(r_ip);
    ptr->setPort(port);
    ptr->setTCPFlag(tcp_flag);
//     printf("ContactList: IP = %lu\n", ptr->getLastIP());
//     printf("ContactList - 2: IP = %lu\n", getLastIPByUIN(uin));
  }
  MW->updateVisibleList();
}

// void ContactList::saveInfoByUIN(unsigned long uin)
// {
//   ContactItem *ptr = getInfoPtrByUIN(uin);
//   if(ptr)
//     ptr->save(contdir);
// }

void ContactList::setAllOffline()
{
  ContactItem *ptr, *usritem;
  ptr = onlineList.first();
  while(ptr)
  {
    ptr->setStatus((unsigned long)STATUS_OFFLINE);
    usritem = onlineList.take();
    offlineList.insert(0, usritem);
    ptr = onlineList.first();
  }
  MW->updateVisibleList();
}

QString ContactList::getNickByUIN(unsigned long uin)
{
  ContactItem *ptr = getInfoPtrByUIN(uin);
  if(ptr)
    return ptr->getNick();
  return i18n("Unknown");
}

QString ContactList::getEMailByUIN(unsigned long uin)
{
  ContactItem *ptr = getInfoPtrByUIN(uin);
  if(ptr)
    return ptr->getEMail();
  return i18n("Unknown");
}

QString ContactList::getNameByUIN(unsigned long uin)
{
  ContactItem *ptr = getInfoPtrByUIN(uin);
  if(ptr)
    return ptr->getName();
  return i18n("Unknown");
}

lastEvent ContactList::getMsgLastByUIN(unsigned long uin)
{
  ContactItem *ptr = getInfoPtrByUIN(uin);
  if(ptr)
    return ptr->getMsgLast();
  return LAST_NONE;
}

unsigned long ContactList::getStatusByUIN(unsigned long uin)
{
  ContactItem *ptr = getInfoPtrByUIN(uin);
  if(ptr)
    return ptr->getStatus();
  return (unsigned long)STATUS_OFFLINE;
}

unsigned long ContactList::getLastIPByUIN(unsigned long uin)
{
  ContactItem *ptr = getInfoPtrByUIN(uin);
  if(ptr)
    return ptr->getLastIP();
  return 0L;
}

bool ContactList::getOnlineAlertByUIN(unsigned long uin)
{
  ContactItem *ptr = getInfoPtrByUIN(uin);
  if(ptr)
    return ptr->isOnlineAlert();
  return FALSE;
}

bool ContactList::getVisibleUserByUIN(unsigned long uin)
{
  ContactItem *ptr = getInfoPtrByUIN(uin);
  if(ptr)
    return ptr->isVisible();
  return FALSE;
}

icqMsgType ContactList::getFirstUnreadMsgTypeByUIN(unsigned long uin)
{
  ContactItem *ptr = getInfoPtrByUIN(uin);
  if(ptr)
    return ptr->getFirstUnreadMsgType();
  return MSG_UNDEF;
}

int ContactList::getNumUnreadMsgsByUIN(unsigned long uin)
{
  ContactItem *ptr = getInfoPtrByUIN(uin);
  if(ptr)
    return ptr->getNumUnreadMsgs();
  return 0;
}

QString ContactList::getAboutByUIN(unsigned long uin)
{
  ContactItem *ptr = getInfoPtrByUIN(uin);
  if(ptr)
    return ptr->getAbout();
  return 0;
}

QString ContactList::getNotesByUIN(unsigned long uin)
{
  ContactItem *ptr = getInfoPtrByUIN(uin);
  if(ptr)
    return ptr->getNotes();
  return 0;
}

QList<icqMessage> *ContactList::getMsgListByUIN(unsigned long uin)
{
  ContactItem *ptr = getInfoPtrByUIN(uin);
  if(ptr)
    return ptr->getMsgList();
  return 0;
}

bool ContactList::isUINOffline(unsigned long uin)
{
  ContactItem *ptr;
  for(ptr = offlineList.first(); ptr != 0L; ptr = offlineList.next())
    if(ptr->getUIN() == uin)
      return TRUE;
  return FALSE;
}

bool ContactList::isUINOnline(unsigned long uin)
{
  ContactItem *ptr;
  for(ptr = onlineList.first(); ptr != 0L; ptr = onlineList.next())
    if(ptr->getUIN() == uin)
      return TRUE;
  return FALSE;
}

bool ContactList::isUINNotInList(unsigned long uin)
{
  ContactItem *ptr;
  for(ptr = notInList.first(); ptr != 0L; ptr = notInList.next())
    if(ptr->getUIN() == uin)
      return TRUE;
  return FALSE;
}

bool ContactList::isUINInContactList(unsigned long uin)
{
  if(!isUINOnline(uin))
    return isUINOffline(uin);
  else
    return TRUE;
}

const ContactItem *ContactList::getInfoByUIN(unsigned long uin)
{
  return getInfoPtrByUIN(uin);
}

ContactItem *ContactList::getInfoPtrByUIN(unsigned long uin)
{
  ContactItem *ptr;
  for(ptr = offlineList.first(); ptr != 0L; ptr = offlineList.next())
    if(ptr->getUIN() == uin)
      return ptr;
  for(ptr = onlineList.first(); ptr != 0L; ptr = onlineList.next())
    if(ptr->getUIN() == uin)
      return ptr;
  for(ptr = notInList.first(); ptr != 0L; ptr = notInList.next())
    if(ptr->getUIN() == uin)
      return ptr;
  return 0L;
}

void ContactList::setAliasByUIN(unsigned long uin, QString alias)
{
  ContactItem *ptr = getInfoPtrByUIN(uin);
  if(ptr)
    ptr->setAlias(alias);
  if(isUINInContactList(uin))
    ptr->save(contdir);
  MW->updateVisibleList();
}

void ContactList::setNotesByUIN(unsigned long uin, QString notes)
{
  ContactItem *ptr = getInfoPtrByUIN(uin);
  if(ptr)
    ptr->setNotes(notes);
  if(isUINInContactList(uin))
    ptr->save(contdir);
}

void ContactList::toggleOnlineAlertByUIN(unsigned long uin)
{
  ContactItem *ptr = getInfoPtrByUIN(uin);
  if(ptr)
    ptr->setOnlineAlert(!ptr->isOnlineAlert());
  if(isUINInContactList(uin))
    ptr->save(contdir);
  MW->updateVisibleList();
}

void ContactList::toggleVisibleUserByUIN(unsigned long uin)
{
  ContactItem *ptr = getInfoPtrByUIN(uin);
  if(!ptr)
    return;
  ptr->setVisible(!ptr->isVisible());
  icq_ContactSetVis(&MW->icqlink, uin, ptr->isVisible());

  if(isUINInContactList(uin))
    ptr->save(contdir);
  MW->updateVisibleList();
}

void ContactList::setThruServerByUIN(unsigned long uin, bool ts)
{
  ContactItem *ptr = getInfoPtrByUIN(uin);
  if(ptr)
    ptr->setThruServer(ts);
}

bool ContactList::getThruServerByUIN(unsigned long uin)
{
  ContactItem *ptr = getInfoPtrByUIN(uin);
  if(ptr)
    return ptr->isThruServer();
  return FALSE;
}
