/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * $Id: contactlist.h,v 1.2 2000/07/28 18:34:41 denis Exp $
 * $Log: contactlist.h,v $
 * Revision 1.2  2000/07/28 18:34:41  denis
 * Code for setting per-user "Send Thru Server" status added.
 *
 * Revision 1.1  2000/06/17 17:38:03  denis
 * Contact List code was rewritten from scratch! Although we are trying
 * to not add new features in anticipation of the next stable release,
 * but this rewrite solves many things, removes many hacks and stabilizes
 * whole program.
 * One of bug, needs to mention, is an absence of IP address in Info
 * window. The root of problem was in Contact List, because in any time
 * there were several ContactItems in memory for single user - they were
 * "smeared" on the whole code. Now it's clean and strict - all Contact
 * List operations can be done only by ContactList class.
 *
 */

#ifndef _CONTACTLIST_
#define _CONTACTLIST_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "contactitem.h"

class ContactList
{
public:
  ContactList();
  ContactList(QString);
  ~ContactList();

  void load(QString);
  void save(); // TODO: finish

  void appendItem(ContactItem*);
  void appendNILItem(ContactItem*);

  void deleteByUIN(unsigned long);
  void deleteNILByUIN(unsigned long);

  void addToContactListByUIN(unsigned long); // TODO: rename it
  void addNewToContactListByUIN(unsigned long);

  QListIterator<ContactItem> &getOnlineListIterator() { return onlineListIterator; }
  QListIterator<ContactItem> &getOfflineListIterator() { return offlineListIterator; }
  QListIterator<ContactItem> &getNotInListIterator() { return notInListIterator; }

  void setAllOffline();

  QString getNickByUIN(unsigned long);
  QString getEMailByUIN(unsigned long);
  lastEvent getMsgLastByUIN(unsigned long);
  unsigned long getStatusByUIN(unsigned long);
  bool getOnlineAlertByUIN(unsigned long);
  bool getVisibleUserByUIN(unsigned long);
  QString getNameByUIN(unsigned long);
  icqMsgType getFirstUnreadMsgTypeByUIN(unsigned long);
  int getNumUnreadMsgsByUIN(unsigned long);
  QString getAboutByUIN(unsigned long);
  QString getNotesByUIN(unsigned long);
  QList<icqMessage> *getMsgListByUIN(unsigned long); // TODO: hide it
  const ContactItem *getInfoByUIN(unsigned long);
  unsigned long getLastIPByUIN(unsigned long);
  bool getThruServerByUIN(unsigned long);

  void setStatusByUIN(unsigned long, unsigned long);
  void setAliasByUIN(unsigned long, QString);
  void setNotesByUIN(unsigned long, QString);
  void setThruServerByUIN(unsigned long, bool);

  void toggleOnlineAlertByUIN(unsigned long);
  void toggleVisibleUserByUIN(unsigned long);

  bool isUINOnline(unsigned long);
  bool isUINOffline(unsigned long);
  bool isUINNotInList(unsigned long);
  bool isUINInContactList(unsigned long);

  void updateInfoByUIN(unsigned long, QString, QString, QString, QString, char);
  void updateExtInfoByUIN(unsigned long, QString, unsigned short, char, QString,
                          unsigned short, char, QString, QString, QString);
  void userOnlineByUIN(unsigned long, unsigned long, unsigned long, unsigned short,
                       unsigned long, unsigned char);

  void addMsg(unsigned long, icqMessage*);
  void addMsg(unsigned long, TimeStr, const char*);
  void addURL(unsigned long, TimeStr, const char*, const char*);
  void addAuth(unsigned long, TimeStr, const char*, const char*, const char*, const char*, const char*);
  void addAdded(unsigned long, TimeStr, const char*, const char*, const char*, const char*);
  void addPager(unsigned long, TimeStr, const char*, const char*, const char*);
  void addExpress(unsigned long, TimeStr, const char*, const char*, const char*);
  void addChatReq(unsigned long, TimeStr, const char*, unsigned long);
  void addFileReq(unsigned long, TimeStr, const char*, const char*, unsigned long, unsigned long);

  int onlineListCount() const { return onlineList.count(); }
  int offlineListCount() const { return offlineList.count(); }
  int notInListCount() const { return notInList.count(); }

protected:
  QString contdir;
  QList<ContactItem> onlineList;
  QList<ContactItem> offlineList;
  QList<ContactItem> notInList;
  QListIterator<ContactItem> onlineListIterator;
  QListIterator<ContactItem> offlineListIterator;
  QListIterator<ContactItem> notInListIterator;

  QStrList getUINDirList();
  ContactItem *getInfoPtrByUIN(unsigned long);
//  void saveInfoByUIN(unsigned long);
};

#endif
