/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
// $Id: systembar.cpp,v 1.1 2000/06/17 17:38:03 denis Exp $
// $Log: systembar.cpp,v $
// Revision 1.1  2000/06/17 17:38:03  denis
// Contact List code was rewritten from scratch! Although we are trying
// to not add new features in anticipation of the next stable release,
// but this rewrite solves many things, removes many hacks and stabilizes
// whole program.
// One of bug, needs to mention, is an absence of IP address in Info
// window. The root of problem was in Contact List, because in any time
// there were several ContactItems in memory for single user - they were
// "smeared" on the whole code. Now it's clean and strict - all Contact
// List operations can be done only by ContactList class.
//
// Revision 1.1  2000/05/24 16:59:50  denis
// Sources have been splitted to the whole bunch of appropriate
// directories. Every directory is end up with static library. All
// of them are linked together to the final kicq binary.
//
// Revision 1.9  2000/01/10 16:09:23  denis
// Headers reorganized for simplify dependencies and speedup compilation.
//
// Revision 1.8  1999/09/29 17:35:37  denis
// Cleanups.
//
// Revision 1.7  1999/07/16 15:48:58  denis
// Cleaned up.
//
// Revision 1.6  1999/07/16 13:12:26  denis
// Internalization support added.
//
// Revision 1.5  1999/07/12 15:23:32  cproch
// - changed icq_* function calls and callback function to support
//   icqlib with ICQLINK identifier.
// - look for FIXME in mainwindow.cpp
//
// Revision 1.4  1998/12/11 18:29:29  whale
// Changed inheritance to WBlinkListBox to make system bar able to blink
//
// Revision 1.3  1998/12/08 16:31:00  denis
// Focus code removed
// Cleaned up a little before releasing
//

#include <qpopmenu.h>
#include <qpainter.h>
#include <kiconloader.h>
#include <kapp.h>

#include "systemitem.h"
#include "systembar.h"
#include "icqmessage.h"

#include "systembar.moc"

SystemBar::SystemBar(QWidget *parent, const char *name, WFlags f) : QListBox(parent, name, f)
{
  QPalette pal = palette();
  QColorGroup colgrp = pal.normal();
  pal.setNormal(QColorGroup(colgrp.foreground(),colgrp.background(),colgrp.light(),colgrp.dark(),
                            colgrp.mid(),colgrp.text(),colgrp.background()));
  colgrp = pal.active();
  pal.setActive(QColorGroup(colgrp.foreground(),colgrp.background(),colgrp.light(),colgrp.dark(),
                            colgrp.mid(),colgrp.text(),colgrp.background()));
  setPalette(pal);

//  setBlinkInterval(1000) ;

  SystemItem *useritem = new SystemItem();
  insertItem(useritem);

  KIconLoader *il = kapp->getIconLoader();
  popup = new QPopupMenu();
  popup->insertItem(il->loadIcon("sendicq.xpm"),i18n("Send ICQ Program To (N/A)"), 1);
  popup->insertSeparator();
  popup->insertItem(il->loadIcon("wwpager.xpm"),i18n("Send WWPager Address To (N/A)"), 2);
  popup->setItemEnabled(1,FALSE);
  popup->setItemEnabled(2,FALSE);
}

SystemBar::~SystemBar()
{
  clear();
  if(popup)
  {
    popup->clear();
    delete popup;
  }
}

void SystemBar::paintCell(QPainter *p, int row, int)
{
  SystemItem *lbi = (SystemItem*)item(row);
  if(!lbi)
    return;

  QColorGroup g = colorGroup();
  {
    p->setBackgroundColor(g.base());
    p->setPen(g.text());
  }
  lbi->paint(p);
  p->setBackgroundColor(g.base());
  p->setPen(g.text());
}

void SystemBar::mousePressEvent(QMouseEvent *e)
{
  QListBox::mousePressEvent(e);
  if(e->button()==RightButton)
  {
    popup->popup(mapToGlobal(QPoint(e->x(),e->y())));
  }
}
