/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
// $Id: useritem.cpp,v 1.2 2000/06/17 17:38:03 denis Exp $
// $Log: useritem.cpp,v $
// Revision 1.2  2000/06/17 17:38:03  denis
// Contact List code was rewritten from scratch! Although we are trying
// to not add new features in anticipation of the next stable release,
// but this rewrite solves many things, removes many hacks and stabilizes
// whole program.
// One of bug, needs to mention, is an absence of IP address in Info
// window. The root of problem was in Contact List, because in any time
// there were several ContactItems in memory for single user - they were
// "smeared" on the whole code. Now it's clean and strict - all Contact
// List operations can be done only by ContactList class.
//
// Revision 1.1  2000/05/24 16:57:03  denis
// Sources have been splitted to the whole bunch of appropriate
// directories. Every directory is end up with static library. All
// of them are linked together to the final kicq binary.
//
// Revision 1.26  2000/01/10 16:09:27  denis
// Headers reorganized for simplify dependencies and speedup compilation.
//
// Revision 1.25  1999/12/28 01:51:23  bills
// added new code to handle file transfers
//
// Revision 1.24  1999/12/12 18:09:10  denis
// Existing kapp->topWidget()'s chnaged to MW macros.
//
// Revision 1.23  1999/12/02 14:03:49  denis
// Cleanups.
//
// Revision 1.22  1999/10/18 15:35:53  tim
// Display entry with italic if the user is on visible or online alert list.
//
// Revision 1.21  1999/09/29 17:36:01  denis
// Cleanups.
//
// Revision 1.20  1999/07/23 12:36:10  denis
// Cleaned up.
//
// Revision 1.19  1999/07/19 13:15:03  tim
// - added support for MSG_ADDED_RECV ("you were added to contact list")
//
// Revision 1.18  1999/07/16 15:49:04  denis
// Cleaned up.
//
// Revision 1.17  1999/07/16 13:15:02  denis
// Internalization support added.
// Support for unsupported early statuses added.
//
// Revision 1.16  1999/07/12 15:23:34  cproch
// - changed icq_* function calls and callback function to support
//   icqlib with ICQLINK identifier.
// - look for FIXME in mainwindow.cpp
//
// Revision 1.15  1999/04/04 22:04:05  bills
// added support for MSG_CHAT_REQ_RECV and chat window handling
//
// Revision 1.14  1998/12/11 18:25:13  whale
// Added support for MSG_AUTH_RECV (incoming auth requests)
//
// Revision 1.13  1998/12/08 16:34:58  denis
// getFirstUnreadMsgType method added
// Cleaned up a little before releasing
//
// Revision 1.12  1998/11/25 12:04:18  denis
// Trying to catch QGDict::look: Attempt to insert null item
//

#include <kapp.h>
#include <kiconloader.h>

#include "useritem.h"
#include "mainwindow.h"
#include "userlist.h"
#include "contactitem.h"

UserItem::UserItem(unsigned long puin) : WBlinkListBoxItem()
{
  uin = puin;
  KIconLoader *loader = kapp->getIconLoader();

  pmap_null = loader->loadIcon("null.xpm");
  pmap_msg = loader->loadIcon("message.xpm");
  pmap_url = loader->loadIcon("url.xpm");
  pmap_auth = loader->loadIcon("auth.xpm");
  pmap_chat = loader->loadIcon("chat.xpm");
  pmap_file = loader->loadIcon("file.xpm");
  pmap_exclam = loader->loadIcon("exclam.xpm");
 
  setPaintMode(VANISH);
  setPixmap(1, pmap_null);
  setPixmap(0, pmap_msg);

  if(MW->contactList().getMsgLastByUIN(uin) == LAST_NONE)
    pm_last = loader->loadIcon("gray.xpm");
  else if(MW->contactList().getMsgLastByUIN(uin) == LAST_MSG)
    pm_last = loader->loadIcon("msglast.xpm");
  else if(MW->contactList().getMsgLastByUIN(uin) == LAST_URL)
    pm_last = loader->loadIcon("urllast.xpm");
  else if(MW->contactList().getMsgLastByUIN(uin) == LAST_AUTH)
    pm_last = loader->loadIcon("authlast.xpm");
  else if(MW->contactList().getMsgLastByUIN(uin) == LAST_EXCLAM)
    pm_last = loader->loadIcon("exclamlast.xpm");

  if(MW->contactList().getStatusByUIN(uin) == (unsigned long)STATUS_OFFLINE)
    pm_stat = loader->loadIcon("null.xpm");
  else if((MW->contactList().getStatusByUIN(uin) & STATUS_INVISIBLE) == STATUS_INVISIBLE)
    pm_stat = loader->loadIcon("uprivacy.xpm");
  else if((MW->contactList().getStatusByUIN(uin) & STATUS_FREE_CHAT) == STATUS_FREE_CHAT)
    pm_stat = loader->loadIcon("ufreechat.xpm");
  else if((MW->contactList().getStatusByUIN(uin) & STATUS_DND) == STATUS_DND)
    pm_stat = loader->loadIcon("udnd.xpm");
  else if((MW->contactList().getStatusByUIN(uin) & STATUS_OCCUPIED) == STATUS_OCCUPIED)
    pm_stat = loader->loadIcon("uoccupied.xpm");
  else if((MW->contactList().getStatusByUIN(uin) & STATUS_NA) == STATUS_NA)
    pm_stat = loader->loadIcon("una.xpm");
  else if((MW->contactList().getStatusByUIN(uin) & STATUS_AWAY) == STATUS_AWAY)
    pm_stat = loader->loadIcon("uaway.xpm");
  else if(!(MW->contactList().getStatusByUIN(uin) & 0x01FF))
    pm_stat = loader->loadIcon("null.xpm");
  else
  {
    QString buf;
    buf.sprintf(i18n("User %s has unhandled status value %lu. Please report it to the authors.\n"),
                (const char*)MW->contactList().getNickByUIN(uin), MW->contactList().getStatusByUIN(uin));
    MW->logEvent(&MW->icqlink, time(0L), ICQ_LOG_ERROR, buf);
  }
}

UserItem::~UserItem()
{
}

const QPixmap *UserItem::pixmap_last()
{
  return &pm_last;
}

const QPixmap *UserItem::pixmap_stat()
{
  return &pm_stat;
}

void UserItem::paint(QPainter *p)
{
  QFont f = p->font();
  QFontMetrics fm = p->fontMetrics();

  if(isBlinkingFlag)
  {
    const QPixmap *pm_active = activePixmap();
    setPixmapXY(3, (QMAX(pm_active->height()+6, fm.lineSpacing())-pm_active->height())/2);
    WBlinkListBoxItem::paint(p);
  }
  else
  {
    p->drawPixmap(3, (QMAX(pm_last.height()+6, fm.lineSpacing())-pm_last.height())/2, pm_last);
    p->drawPixmap(3, (QMAX(pm_stat.height()+6, fm.lineSpacing())-pm_stat.height())/2, pm_stat);
  }

  int yPos;
  if (pm_last.height() < fm.height())
    yPos = fm.ascent() + fm.leading()/2;
  else
    yPos = (pm_last.height()+6)/2 - fm.height()/2 + fm.ascent();

  if(MW->contactList().getOnlineAlertByUIN(uin) ||
     MW->contactList().getVisibleUserByUIN(uin))
  {
    f.setItalic(TRUE); 
    p->setFont(f);
  }
  p->drawText(pm_last.width() + 7, yPos, MW->contactList().getNameByUIN(uin));

  if(MW->contactList().getOnlineAlertByUIN(uin) ||
     MW->contactList().getVisibleUserByUIN(uin))
  {
    f.setItalic(FALSE);
    p->setFont(f);
  }
}

int UserItem::height(const QListBox *lb ) const
{
  return QMAX(pm_last.height()+6, lb->fontMetrics().lineSpacing());
}

int UserItem::width(const QListBox *lb ) const
{
  return pm_last.width() + lb->fontMetrics().width(MW->contactList().getNameByUIN(uin)) + 6;
}

void UserItem::updatePixmap()
{
  icqMsgType t = MW->contactList().getFirstUnreadMsgTypeByUIN(uin); // Last
  if(t == MSG_RECV)
    setPixmap(0,pmap_msg);
  if(t == MSG_URL_RECV)
    setPixmap(0,pmap_url);
  if(t == MSG_AUTH_RECV)
    setPixmap(0,pmap_auth);
  if(t == MSG_CHAT_REQ_RECV)
    setPixmap(0,pmap_chat);
  if(t == MSG_FILE_REQ_RECV)
    setPixmap(0,pmap_file);
  if(t == MSG_ADDED)
    setPixmap(0,pmap_exclam);
}
