/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
// $Id: userlist.cpp,v 1.3 2000/07/28 18:32:56 denis Exp $
// $Log: userlist.cpp,v $
// Revision 1.3  2000/07/28 18:32:56  denis
// Cleanups.
//
// Revision 1.2  2000/06/17 17:38:03  denis
// Contact List code was rewritten from scratch! Although we are trying
// to not add new features in anticipation of the next stable release,
// but this rewrite solves many things, removes many hacks and stabilizes
// whole program.
// One of bug, needs to mention, is an absence of IP address in Info
// window. The root of problem was in Contact List, because in any time
// there were several ContactItems in memory for single user - they were
// "smeared" on the whole code. Now it's clean and strict - all Contact
// List operations can be done only by ContactList class.
//
// Revision 1.1  2000/05/24 16:57:03  denis
// Sources have been splitted to the whole bunch of appropriate
// directories. Every directory is end up with static library. All
// of them are linked together to the final kicq binary.
//
// Revision 1.34  2000/04/10 09:15:47  denis
// Fixed bug with new message arrived from offline user when only online
// users are shown.
//
// Revision 1.33  2000/04/04 17:19:24  denis
// Applied patch from Maxim Dmitrichenko <dmax@nix.com.ua> for All/Online
// toggle button with some modifications like transparency in icons and
// save/restore its state in config file.
//
// Revision 1.32  2000/03/01 20:48:29  guruz
// -Added WGet as browser when recieving URLs
// -Modified server prefs
// -fixed some things with special ops
//
// Revision 1.31  2000/01/20 20:03:03  bills
// initial implementation for sending file requests
//
// Revision 1.30  2000/01/10 16:09:29  denis
// Headers reorganized for simplify dependencies and speedup compilation.
//
// Revision 1.29  1999/12/28 01:51:25  bills
// added new code to handle file transfers
//
// Revision 1.28  1999/12/12 18:09:31  denis
// Existing kapp->topWidget()'s chnaged to MW macros.
//
// Revision 1.27  1999/12/02 14:03:07  denis
// addMsg() functions renamed to corresponding names like addURL(),
// addAuth(), etc.
// Cleanups.
//
// Revision 1.26  1999/11/29 21:24:41  guruz
// Some new features in the user list context menu
// I hope it all REALLY works because i cannot test it
// properly because of the damn online users bug
//
// Revision 1.25  1999/11/21 18:55:37  guruz
// Added "Send E-Mail" in User List Context Menu
//
// Revision 1.24  1999/09/29 17:36:21  denis
// Cleanups.
//
// Revision 1.23  1999/07/23 12:36:23  denis
// Cleaned up.
//
// Revision 1.22  1999/07/19 13:15:05  tim
// - added support for MSG_ADDED_RECV ("you were added to contact list")
//
// Revision 1.21  1999/07/16 15:49:06  denis
// Cleaned up.
//
// Revision 1.20  1999/07/16 13:15:33  denis
// Internalization support added.
// Support for unsupported early statuses added.
//
// Revision 1.19  1999/07/12 15:23:35  cproch
// - changed icq_* function calls and callback function to support
//   icqlib with ICQLINK identifier.
// - look for FIXME in mainwindow.cpp
//
// Revision 1.18  1999/05/07 00:04:20  billb
// 'receive message' menu item now dynamically loads if a message is waiting, or is removed if there isn't.
//
// Revision 1.17  1999/04/04 22:04:07  bills
// added support for MSG_CHAT_REQ_RECV and chat window handling
//
// Revision 1.16  1999/03/05 13:04:43  denis
// Applied patch to handle authorization request messages by Bogdan Surdu
// <tim@ima-info.ro>
//
// Revision 1.15  1998/12/11 18:26:15  whale
// Added support for MSG_AUTH_RECV (incoming auth requests)
//
// Revision 1.14  1998/12/08 16:35:50  denis
// Cleaned up a little before releasing
//
// Revision 1.13  1998/11/25 12:04:34  denis
// Trying to catch QGDict::look: Attempt to insert null item
//
// Revision 1.12  1998/11/23 10:11:36  denis
// recvRequest support added
//

#include <icq.h>
#include <qpopmenu.h>
#include <qpainter.h>
#include <kiconloader.h>
#include <kapp.h>

#include "mainwindow.h"
#include "userlist.h"
#include "specialops.h"

#include "virtuseritem.h"
#include "useritem.h"
#include "separatoritem.h"

#include "userlist.moc"

UserList::UserList(QWidget *parent, const char *name, WFlags f) : QListBox(parent, name, f)
{
  timerId = -1;
  oldCurrent = -1;
  allSetting = TRUE;
  doRemove = FALSE;
  recvRemove = FALSE;
  QPalette pal = palette();
  QColorGroup colgrp = pal.normal();
  pal.setNormal(QColorGroup(colgrp.foreground(),colgrp.background(),colgrp.light(),
                            colgrp.dark(),colgrp.mid(),colgrp.text(),colgrp.background()));
  colgrp = pal.active();
  pal.setActive(QColorGroup(colgrp.foreground(),colgrp.background(),colgrp.light(),
                            colgrp.dark(),colgrp.mid(),colgrp.text(),colgrp.background()));
  setPalette(pal);

  connect(this, SIGNAL(selected(int)), MW ,SLOT(recvRequest(int)));

  KIconLoader *il = kapp->getIconLoader();
  alertpopup = new QPopupMenu();
  onlineID = alertpopup->insertItem(i18n("Online Alert"), MW, SLOT(onlineAlert()));
  alertpopup->insertSeparator();
  visibleID = alertpopup->insertItem(i18n("Visible To User"), MW, SLOT(visibleUser()));
  alertpopup->setCheckable(TRUE);

  specialoppopup = new QPopupMenu();
  connect(specialoppopup, SIGNAL(activated(int)), MW, SLOT(specialOpRequest(int)));
  specialoppopup->insertItem(il->loadMiniIcon("ftp.xpm"),i18n("Connect to FTP Server of this user"),
                             KICQ_SPECIALOPS_CONNECT_TO_FTP_SERVER);
  specialoppopup->insertItem(il->loadMiniIcon("www.xpm"),i18n("Connect to HTTP Server of this user"),
                             KICQ_SPECIALOPS_CONNECT_TO_HTTP_SERVER);
  specialoppopup->insertItem(i18n("Connect to Tetrinet Server of this user"),
                             KICQ_SPECIALOPS_CONNECT_TO_TETRINET_SERVER);
  specialoppopup->insertSeparator();
  specialoppopup->insertItem(i18n("Telnet this user"), KICQ_SPECIALOPS_TELNET);
  specialoppopup->insertSeparator();
  specialoppopup->insertItem(i18n("Ping this user"), KICQ_SPECIALOPS_PING);
  specialoppopup->insertItem(i18n("Traceroute this user"), KICQ_SPECIALOPS_TRACEROUTE);

  popup = new QPopupMenu();
  popup->insertItem(il->loadIcon("message.xpm"), i18n("Send Message"), MW, SLOT(messageRequest()));
  popup->insertItem(il->loadIcon("url.xpm"), i18n("Send URL"), MW, SLOT(urlRequest()));
  popup->insertItem(il->loadMiniIcon("kmail.xpm"), i18n("Send E-Mail"), MW, SLOT(mailRequest()));
  popup->insertItem(il->loadIcon("file.xpm"), i18n("Send File"), MW, SLOT(fileRequest()));
  popup->insertItem(il->loadIcon("chat.xpm"), i18n("ICQ Chat"), MW, SLOT(chatRequest()));
  popup->insertSeparator();
  popup->insertItem(il->loadIcon("history.xpm"), i18n("View Message History"), MW, SLOT(historyRequest()));
  popup->insertSeparator();
  popup->insertItem(il->loadIcon("alert.xpm"), i18n("Alert/Accept Modes"), alertpopup);
  popup->insertItem(il->loadIcon("info.xpm"), i18n("Info"), MW, SLOT(infoRequest()));
  popup->insertItem(il->loadIcon("rename.xpm"), i18n("Rename"), MW, SLOT(renameRequest()));
  popup->insertItem(il->loadIcon("delete.xpm"), i18n("Delete"), MW, SLOT(delUserRequest()));
  popup->insertSeparator();
  popup->insertItem(i18n("Special Operations"), specialoppopup);
  popup->setItemEnabled(1,FALSE);

  setBlinkInterval(1000);
}

UserList::~UserList()
{
  clear();
  if(alertpopup)
  {
    alertpopup->clear();
    delete alertpopup;
  }
  if(popup)
  {
    popup->clear();
    delete popup;
  }
}

void UserList::paintCell(QPainter *p, int row, int col)
{
  UserItem *lbi = (UserItem*)item(row);
  if(!lbi)
    return;

  QColorGroup g = colorGroup();
  if(lbi->canBeSelected())
  {
    if(isSelected(row))
    {
      QColor fc(blue);
      p->fillRect(0, 0, cellWidth(col), cellHeight(row), fc);
      p->setPen(QColor(white));
      p->setBackgroundColor(fc);
    }
    else
    {
      p->setBackgroundColor(g.base());
      if(MW->contactList().isUINNotInList(lbi->getUIN()))
      {
        p->setPen(QColor(darkGreen));
      }
      else
      {
        if(MW->contactList().isUINOffline(lbi->getUIN()))
          p->setPen(QColor(red));
        else if(MW->contactList().isUINOnline(lbi->getUIN()))
          p->setPen(QColor(blue));
        else
          p->setPen(g.text());
      }
    }
  }
  lbi->paint(p);
  p->setBackgroundColor(g.base());
  p->setPen(g.text());
}

void UserList::mousePressEvent(QMouseEvent *e)
{
  QListBox::mousePressEvent(e);
  if(currentItem() < 0)
    return;

  UserItem *ui = getCurrent();
  if(!ui->canBeSelected())
    return;

  KIconLoader *l = kapp->getIconLoader();
  if(e->button()==RightButton)
  {
    if(MW->contactList().getNumUnreadMsgsByUIN(ui->getUIN()))
    {
      if(!recvRemove)
      {
        popup->insertItem(l->loadIcon("receive.xpm"), i18n("Receive Message"), -1, 0);
        popup->connectItem(0, MW, SLOT(recvRequest()));
        recvRemove = TRUE;
      }
    }
    else
      if(recvRemove)
      {
        recvRemove = FALSE;
        popup->removeItem(0);
      }
    alertpopup->setItemChecked(onlineID, MW->contactList().getOnlineAlertByUIN(ui->getUIN()));
    alertpopup->setItemChecked(visibleID, MW->contactList().getVisibleUserByUIN(ui->getUIN()));
    if(MW->contactList().isUINNotInList(ui->getUIN()))
    {
      if(!doRemove)
      {
        popup->insertSeparator();
        popup->insertItem(l->loadIcon("add.xpm"),i18n("Add To Contact List"),this,SLOT(addRequest()));
        doRemove = TRUE;
      }
    }
    else
      if(doRemove)
      {
        popup->removeItemAt(popup->count() - 1);
        popup->removeItemAt(popup->count() - 1);
        doRemove = FALSE;
      }
    popup->popup(mapToGlobal(QPoint(e->x(),e->y())));
  }
}

void UserList::addRequest()
{
  unsigned long uin = getCurrent()->getUIN();
  MW->contactList().addToContactListByUIN(uin);
  if(MW->isLogged())
    icq_SendNewUser(&MW->icqlink, uin);
}

UserItem *UserList::getItem(int i)
{
  return (UserItem*)item(i);
}

UserItem *UserList::getCurrent()
{
  if(currentItem() < 0)
    return 0L;
  return (UserItem*)item(currentItem());
}

void UserList::changeList()
{
  unsigned long cur_uin = 0;
  unsigned int i;
  setAutoUpdate(FALSE);

  if(currentItem()>=0)
    cur_uin = getCurrent()->getUIN();

  clear();

  if(MW->contactList().onlineListCount() > 0)
  {
    SeparatorItem *sitem = new SeparatorItem(SeparatorItem::tpOnSeparator);
    insertItem(sitem);
    QListIterator<ContactItem> it = MW->contactList().getOnlineListIterator();
    for(it.toFirst(); it.current(); ++it)
    {
      UserItem *item = new UserItem(it.current()->getUIN());
      insertItem(item);
      if(it.current()->getNumUnreadMsgs() > 0)
      {
        item->updatePixmap();
        setBlinking(item, TRUE);
      }
    }
  }

  if(MW->contactList().notInListCount() > 0)
  {
    SeparatorItem *sitem = new SeparatorItem(SeparatorItem::tpNILSeparator);
    insertItem(sitem);
    QListIterator<ContactItem> it = MW->contactList().getNotInListIterator();
    for(it.toFirst(); it.current(); ++it)
    {
      UserItem *item = new UserItem(it.current()->getUIN());
      insertItem(item);
      if(it.current()->getNumUnreadMsgs() > 0)
      {
        item->updatePixmap();
        setBlinking(item, TRUE);
      }
    }
  }

  if(MW->contactList().offlineListCount() > 0)
  {
    if(getAllOnline())
    {
      SeparatorItem *sitem = new SeparatorItem(SeparatorItem::tpOffSeparator);
      insertItem(sitem);
      QListIterator<ContactItem> it = MW->contactList().getOfflineListIterator();
      for(it.toFirst(); it.current(); ++it)
      {
        UserItem *item = new UserItem(it.current()->getUIN());
        insertItem(item);
        if(it.current()->getNumUnreadMsgs() > 0)
        {
          item->updatePixmap();
          setBlinking(item, TRUE);
        }
      }
    }
    else
    {
      bool sepIns = false;
      QListIterator<ContactItem> it = MW->contactList().getOfflineListIterator();
      for(it.toFirst(); it.current(); ++it)
      {
        if(it.current()->getNumUnreadMsgs() > 0)
        {
          if(!sepIns)
          {
            SeparatorItem *sitem = new SeparatorItem(SeparatorItem::tpOffSeparator);
            insertItem(sitem);
            sepIns = true;
          }
          UserItem *item = new UserItem(it.current()->getUIN());
          insertItem(item);
          item->updatePixmap();
          setBlinking(item, TRUE);
        }
      }
    }
  }

  if(cur_uin)
    for(i=0; i<count(); i++)
      if(getItem(i)->getUIN()==cur_uin)
        setCurrentItem(i);

  setAutoUpdate(TRUE);
  update();
}

void UserList::setBlinkingByUIN(unsigned long uin, int b)
{
  for(unsigned int i=0; i<count(); i++)
  {
    UserItem *item = getItem(i);
    if(item->canBeSelected())
      if(item->getUIN() == uin)
        setBlinking(item, b);
  }
}

void UserList::allChange()
{
  allSetting = !allSetting;
}

void UserList::setAllOnline(bool allSet)
{
  allSetting = allSet;
}

bool UserList::getAllOnline() const
{
  return allSetting;
}

void UserList::timerEvent(QTimerEvent *te)
{
  int n = 0;
  if(te->timerId() == timerId)
  {
    for(unsigned int i = 0; i < count(); i ++)
    {
      WBlinkListBoxItem* bitem;
      bitem = (WBlinkListBoxItem*)item(i);
      if(bitem->canBeSelected())
        if(bitem->isBlinking())
        {
          n = bitem->togglePixmap();
          int yPos;
          if(itemYPos(i,&yPos))
            update(bitem->pixmapX(), yPos + bitem->pixmapY(),
                   bitem->pixmapMaxWidth() + 4, bitem->pixmapMaxHeight() + 4);
        }
    }
    MW->togglePixmap(n);
  }
}

void UserList::setBlinkInterval(int interval)
{
  blinkInterval = interval;
}

int UserList::getIndex(UserItem* bitem)
{
  for(unsigned long i=0; i<count(); i++)
    if(getItem(i) == bitem)
      return i;
  return -1;
}

void UserList::setBlinking(UserItem* bitem, int b)
{
  if(!bitem->canBeSelected())
    return;

  if(bitem)
    bitem->setBlinking(b);

  /* If we are requested to stop blinking, reset to the first pixmap */
  if(bitem)
  {
    if(!b)
      bitem->resetPixmap();
    int yPos;
    if(itemYPos(getIndex(bitem),&yPos))
      update(3,yPos,bitem->pixmapMaxWidth() + 2,bitem->pixmapMaxHeight() + 1);
  }

  if(blinkItemsCount() > 0)
    if(timerId == -1)
      timerId = startTimer(blinkInterval);
  if(blinkItemsCount() == 0) 
  {
    killTimer(timerId);
    timerId = -1;
  }
}

int UserList::blinkItemsCount()
{
  unsigned int i, icount = 0;
  for(i=0; i<count(); i++)
  {
    WBlinkListBoxItem *it = (WBlinkListBoxItem*)item(i);
    if(it->canBeSelected())
      if(it->isBlinking())
        icount++;
  }
  return icount;
}
