/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
// $Id: renwindow.cpp,v 1.2 2000/06/17 17:38:22 denis Exp $
// $Log: renwindow.cpp,v $
// Revision 1.2  2000/06/17 17:38:22  denis
// Contact List code was rewritten from scratch! Although we are trying
// to not add new features in anticipation of the next stable release,
// but this rewrite solves many things, removes many hacks and stabilizes
// whole program.
// One of bug, needs to mention, is an absence of IP address in Info
// window. The root of problem was in Contact List, because in any time
// there were several ContactItems in memory for single user - they were
// "smeared" on the whole code. Now it's clean and strict - all Contact
// List operations can be done only by ContactList class.
//
// Revision 1.1  2000/05/24 16:57:16  denis
// Sources have been splitted to the whole bunch of appropriate
// directories. Every directory is end up with static library. All
// of them are linked together to the final kicq binary.
//
// Revision 1.8  1999/12/02 14:26:02  denis
// Random IcqWindow segfaults fixed.
// Cleanups.
//
// Revision 1.7  1999/10/14 12:58:38  denis
// QLayout support implemented.
// Segfault on window close fixed.
//
// Revision 1.6  1999/07/16 15:48:56  denis
// Cleaned up.
//
// Revision 1.5  1999/07/16 13:12:25  denis
// Internalization support added.
//
// Revision 1.4  1999/04/17 20:22:23  bills
// modified for new icqwindow base class
//
// Revision 1.3  1998/12/08 16:29:48  denis
// Cleaned up a little before releasing
//

#include <qlayout.h>
#include <qlabel.h>
#include <qpushbt.h>

#include "renwindow.h"
#include "mainwindow.h"

#include "renwindow.moc"

RenameWindow::RenameWindow(QString name, unsigned long puin, QString nick, QString first,
                           QString last, QString email) : IcqWindow()
{
  QString txt;
  txt.sprintf(i18n("Rename user %s to:"), (const char*)name);
  uin = puin;

  QBoxLayout *topLayout = new QVBoxLayout(this, 7);

  QLabel *label = new QLabel(txt,this);
  label->setMinimumSize(label->sizeHint());
  topLayout->addWidget(label);

  aliasbox = new QComboBox(TRUE, this);
  aliasbox->setMinimumSize(aliasbox->sizeHint());
  aliasbox->setMaximumHeight(aliasbox->sizeHint().height());
  aliasbox->setInsertionPolicy(QComboBox::NoInsertion);
  if(!nick.isEmpty() && nick!=i18n("N/A"))
    aliasbox->insertItem(nick);
  if(!first.isEmpty() && first!=i18n("N/A") && first!=nick)
    aliasbox->insertItem(first);
  if(!last.isEmpty() && last!=i18n("N/A") && last!=nick)
    aliasbox->insertItem(last);
  if(!first.isEmpty() && !last.isEmpty() && first!=i18n("N/A") && last!=i18n("N/A"))
    aliasbox->insertItem(first+" "+last);
  if(!email.isEmpty() && email!=i18n("N/A"))
    aliasbox->insertItem(email);
  txt.setNum(uin);
  aliasbox->insertItem(txt);
  topLayout->addWidget(aliasbox);

  QBoxLayout *butLayout = new QHBoxLayout(7);
  topLayout->addLayout(butLayout);

  butLayout->addStretch(1);

  QPushButton *okbtn = new QPushButton(i18n("Ok"), this);
  okbtn->setMaximumHeight(okbtn->sizeHint().height());
  okbtn->setDefault(TRUE);
  connect(okbtn, SIGNAL(clicked()), SLOT(accept()));
  butLayout->addWidget(okbtn);

  QPushButton *cancelbtn = new QPushButton(i18n("Cancel"), this);
  cancelbtn->setMaximumHeight(cancelbtn->sizeHint().height());
  connect(cancelbtn, SIGNAL(clicked()), SLOT(reject()));
  butLayout->addWidget(cancelbtn);

  int w = QMAX(okbtn->sizeHint().width(), cancelbtn->sizeHint().width());
  okbtn->setMinimumSize(w, okbtn->sizeHint().height());
  cancelbtn->setMinimumSize(w, cancelbtn->sizeHint().height());

  butLayout->addStretch(1);

  topLayout->activate();
  aliasbox->setFocus();

  setCaption(i18n("Rename User"));

  resize(1,1);
}

unsigned long RenameWindow::UIN()
{
  return uin;
}

void RenameWindow::accept()
{
  MW->contactList().setAliasByUIN(uin, aliasbox->currentText());
  IcqWindow::accept();
}

void RenameWindow::done(int p)
{
  IcqWindow::done(p);
}
