/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
// $Id: filesndwin.cpp,v 1.3 2000/06/17 17:38:34 denis Exp $
// $Log: filesndwin.cpp,v $
// Revision 1.3  2000/06/17 17:38:34  denis
// Contact List code was rewritten from scratch! Although we are trying
// to not add new features in anticipation of the next stable release,
// but this rewrite solves many things, removes many hacks and stabilizes
// whole program.
// One of bug, needs to mention, is an absence of IP address in Info
// window. The root of problem was in Contact List, because in any time
// there were several ContactItems in memory for single user - they were
// "smeared" on the whole code. Now it's clean and strict - all Contact
// List operations can be done only by ContactList class.
//
// Revision 1.2  2000/06/15 01:58:00  bills
// fixed crash bug when cancelling a file request and then trying to send
// again, now calls icq_CancelFileRequest when request is cancelled to notify
// remote icq client
//
// Revision 1.1  2000/05/24 16:57:49  denis
// Sources have been splitted to the whole bunch of appropriate
// directories. Every directory is end up with static library. All
// of them are linked together to the final kicq binary.
//
// Revision 1.3  2000/05/05 03:38:06  bills
// removed unnecessary deletes in destructor, removed debug statements,
// disabled send button until files are selected.
//
// Revision 1.2  2000/05/04 16:20:28  bills
// updated to use new icqlib file transfer notification method, which enables
// support for sending files.
//
// Revision 1.1  2000/01/20 20:03:32  bills
// added
//
//

#include <icq.h>
#include <qmsgbox.h>
#include <qmovie.h>
#include <qregexp.h>
#include <qkeycode.h>
#include <kapp.h>
#include <kiconloader.h>
#include <kfiledialog.h>

#include "filewindow.h"
#include "filesndwin.h"
#include "mainwindow.h"
#include "userlist.h"
#include "icqmessage.h"

#include "filesndwin.moc"

FileSndWindow::FileSndWindow(const char *n, unsigned long u, const char *em) : IcqWindow()
{
  QString text;
  QPixmap infopix;
  KIconLoader *il = kapp->getIconLoader();
  setCaption(i18n("Send File Request"));
  tobox = new QGroupBox(i18n("To"),this);
  sendbutton = new QPushButton(i18n("Send"),this);
  sendbutton->setAccel(Key_Return+CTRL) ;
  sendbutton->setEnabled(false);
  connect(sendbutton, SIGNAL(clicked()),SLOT(sendFileReq()));
  cancelbutton = new QPushButton(i18n("Cancel"),this);
  connect(cancelbutton, SIGNAL(clicked()),SLOT(cancelFileReq()));
  selectbutton = new QPushButton(i18n("Select Files..."),this);
  connect(selectbutton, SIGNAL(clicked()),SLOT(selectFileReq()));
  uinline = new QLineEdit(this);
  uinline->setFocusPolicy(QWidget::NoFocus);
  uinlabel = new QLabel(uinline,i18n("ICQ#:"),this);
  handleline = new QLineEdit(this);
  handleline->setFocusPolicy(QWidget::NoFocus);
  handlelabel = new QLabel(handleline,i18n("Handle:"),this);
  emailline = new QLineEdit(this);
  emailline->setFocusPolicy(QWidget::NoFocus);
  emaillabel = new QLabel(emailline,i18n("EMail:"),this);
  infobutton = new QPushButton(this);
  infopix = il->loadIcon("info.xpm");
  infobutton->setPixmap(infopix);
  infobutton->resize(infopix.width()+6, infopix.height()+6);
  connect(infobutton, SIGNAL(clicked()), SLOT(infoRequest()));
  editbox = new QMultiLineEdit(this);
  editlabel = new QLabel(editbox,i18n("Enter File Description:"),this);
  updateanim = new QLabel(this);
  QString filename = kapp->localkdedir()+"/share/apps/"+kapp->appName()+"/pics/search.gif";
  QFile testfile(filename);
  if(!testfile.exists())
    filename = kapp->kde_datadir()+"/"+kapp->appName()+"/pics/search.gif";
  updateanim->setMovie(QMovie((const char*)filename));
  updateanim->setFrameStyle(QFrame::WinPanel | QFrame::Sunken);
  updateanim->setAlignment(AlignCenter);
  updateanim->movie()->pause();
  setName(n);
  setUIN(u);
  setEMail(em);
  files=0L;
  editbox->setFocus();
  resize(350,250);
  setMinimumSize(350,250);
}

FileSndWindow::~FileSndWindow()
{
}

void FileSndWindow::resizeEvent(QResizeEvent *re)
{
  IcqWindow::resizeEvent(re);
  QSize uinlblsize,handlelblsize,emaillblsize,toboxsize,editlblsize,btn1size,btn2size;
  toboxsize.setWidth(width()-6);
  uinlblsize = uinlabel->sizeHint();
  handlelblsize = handlelabel->sizeHint();
  emaillblsize = emaillabel->sizeHint();
  int wl = QMAX(uinlblsize.width(),emaillblsize.width());
  int w = (toboxsize.width()-uinlblsize.width()-handlelblsize.width()-36)/2;
  uinline->resize(w,uinline->sizeHint().height());
  handleline->resize(w,uinline->sizeHint().height());
  emailline->resize(toboxsize.width()-infobutton->width()-62,uinline->sizeHint().height());
  uinlabel->resize(wl,uinline->height());
  handlelabel->resize(handlelblsize.width(),handleline->height());
  emaillabel->resize(wl,emailline->height());
  uinlabel->move(12,15);
  uinline->move(3+uinlabel->x()+uinlabel->width(),15);
  handlelabel->move(12+uinline->x()+uinline->width(),15);
  handleline->move(3+handlelabel->x()+handlelabel->width(),15);
  emaillabel->move(12,20+uinline->height());
  emailline->move(3+emaillabel->x()+emaillabel->width(),20+uinline->height());
  tobox->setGeometry(3,3,toboxsize.width(),uinline->height()+emailline->height()+23);
  infobutton->move(7+emailline->x()+emailline->width(),
                   emailline->y()+(emailline->height()-infobutton->height())/2);
  btn1size = sendbutton->sizeHint();
  btn2size = cancelbutton->sizeHint();
  QSize btn3size = selectbutton->sizeHint();
  int h = QMAX(QMAX(btn1size.height(),btn2size.height()),btn3size.height());
  w = QMAX(QMAX(QMAX(btn1size.width(),btn2size.width()),btn3size.width()),
      75);
  sendbutton->setGeometry(width()/3-w-6,height()-h-12,w,h);
  selectbutton->setGeometry(width()/3+6, height()-h-12, w, h);
  cancelbutton->setGeometry(width()/3+w+6,height()-h-12,w,h);

  editlblsize = editlabel->sizeHint();
  editlabel->setGeometry(6,8+tobox->height(),editlblsize.width(),editlblsize.height());
  editbox->setGeometry(3,editlabel->y()+editlabel->height()+4,width()-6,height()-editlabel->y()-
                       editlabel->height()-h-28);
  updateanim->setGeometry(width()-40,height()-40,32,32);
  updateanim->movie()->restart();
}

void FileSndWindow::setUIN(unsigned long u)
{
  uin = u;
  QString text;
  text.setNum(uin);
  uinline->setText(text);
}

void FileSndWindow::setName(const char *name)
{
  handleline->setText(name);
}

void FileSndWindow::setEMail(const char *email)
{
  emailline->setText(email);
}

void FileSndWindow::selectFileReq()
{
  QString open_filename=KFileDialog::getOpenFileName();

  if(!open_filename.isEmpty())
  {
    if (files) {
      free(*files);
      free(files);
    }
    files=(char **)malloc(sizeof(char *)*2);
    *files=(char *)malloc(open_filename.length()+1);
    strcpy(*files, (const char *)open_filename);
    files++;
    *files=0L;
    files--;
    sendbutton->setEnabled(true);
  }

/*
  QStrList filenames(QFileDialog::getOpenFileNames(0,QDir::homeDirPath()));

    for (const char* f = filenames.first(); f; f = filenames.next())

      printf("%s\n", f);
*/

}

void FileSndWindow::sendFileReq()
{
  QString tmp;
  if(MW->isLogged())
  {
    tmp = editbox->text();
    if(MW->isConvertCRLF())
      tmp.replace(QRegExp("\n"),"\r\n");
    sequence = icq_SendFileRequest(&MW->icqlink, uin,(const char*)editbox->text(), files);
    files = 0L;
    updateanim->movie()->unpause();
    sendbutton->setEnabled(FALSE);
  }
  else
    QMessageBox::warning(this, i18n("Error"), i18n("You are not connected to the icq network!"));
}

void FileSndWindow::cancelFileReq()
{
  if(sequence==0)
    close();
  else
    stopCommand();
}

void FileSndWindow::infoRequest()
{
  MW->infoWindow(uin);
}

void FileSndWindow::stopCommand()
{
  if (sequence) {
    icq_CancelFileRequest(&MW->icqlink, uin, sequence);
    sequence = 0;
  }
  updateanim->movie()->pause();
}

void FileSndWindow::saveHistory()
{
  time_t curtm = time(0L);
  struct tm *t = localtime(&curtm);
  TimeStr time(t->tm_hour,t->tm_min,t->tm_mday,t->tm_mon+1,t->tm_year+1900) ;

  icqMessage msg(uin,time,editbox->text(),MSG_URL_SENT) ;
  QString historypath = MW->contactsDir() ;
  msg.save(historypath) ;
  MW->updateHistoryWindows(msg.uin()) ;
}

void FileSndWindow::requestNotify(int type, int length, void *data)
{
  if(type==ICQ_NOTIFY_FILESESSION)
  {
    FileWindow *p=new FileWindow((icq_FileSession *)data, seq());
    p->show();
    close();
  }
}
