/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
// $Id: filewindow.cpp,v 1.3 2000/07/30 18:53:25 denis Exp $
// $Log: filewindow.cpp,v $
// Revision 1.3  2000/07/30 18:53:25  denis
// Warnings were fixed.
//
// Revision 1.2  2000/06/17 17:04:19  denis
// Cleanups.
//
// Revision 1.1  2000/05/24 16:57:49  denis
// Sources have been splitted to the whole bunch of appropriate
// directories. Every directory is end up with static library. All
// of them are linked together to the final kicq binary.
//
// Revision 1.7  2000/05/04 16:15:00  bills
// updated to use new icqlib file transfer notification method, tweaked
// layout, implemented abort button and timers.
//
// Revision 1.6  2000/03/30 15:13:11  denis
// Float/integer warning fixed.
//
// Revision 1.5  2000/01/20 20:03:03  bills
// initial implementation for sending file requests
//
// Revision 1.4  2000/01/16 21:36:51  bills
// implemented save/save as action buttons, changed code to use new file
// session interface
//
// Revision 1.3  2000/01/10 16:08:14  denis
// Headers reorganized for simplify dependencies and speedup compilation.
//
// Revision 1.2  2000/01/07 04:14:56  bills
// fixed warning with reduceNumber
//
// Revision 1.1  1999/12/28 01:55:27  bills
// added
//

#include <qmsgbox.h>
#include <qmovie.h>
#include <qregexp.h>
#include <qkeycode.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <kapp.h>
#include <kiconloader.h>
#include <qpushbutton.h>

#include "filewindow.h"
#include "mainwindow.h"

#include "filewindow.moc"

QString reduceNumber(float num)
{
  const int kilobyte=1024;
  const int megabyte=kilobyte*1024;
  float value=num;
  QString result;

  if (value > 2*megabyte)
  {
    value=value/megabyte;
    result.sprintf("%.2f MB", value);
  } 
  else if (value > 10*kilobyte) 
  {
    value=value/kilobyte;
    result.sprintf("%.2f KB", value);
  }
  else
  {
    result.sprintf("%d bytes", (int)value);
  }

  return result;
}
  
FileWindow::FileWindow(icq_FileSession *session, unsigned long id) :
  IcqWindow(), icqFileSession( new IcqFileSession(session, this) )
{

  sequence=id;

  QGridLayout *mainLayout = new QGridLayout(this, 5, 1, 7);
  mainLayout->setRowStretch(0, 6);
  mainLayout->setRowStretch(1, 0);
  mainLayout->setRowStretch(2, 9);
  mainLayout->setRowStretch(3, 0);
  mainLayout->setRowStretch(4, 3);

  // Header

  QGridLayout *headerLayout = new QGridLayout(2, 4, 7);
  mainLayout->addLayout(headerLayout, 0, 0);
  headerLayout->setColStretch(0, 0);
  headerLayout->setColStretch(1, 1);
  headerLayout->setColStretch(2, 0);
  headerLayout->setColStretch(3, 1);

  QLabel *filelabel = new QLabel("Filename:", this);
  filelabel->setMinimumSize(filelabel->sizeHint());
  headerLayout->addWidget(filelabel, 0, 0);

  headerFileName = new QLineEdit(this);
  headerFileName->setFocusPolicy(QWidget::NoFocus);
  headerFileName->setMinimumSize(headerFileName->sizeHint());
  headerFileName->setMaximumHeight(headerFileName->sizeHint().height());
  headerLayout->addWidget(headerFileName, 0, 1);

  QLabel *filecountlabel = new QLabel("Files:", this);
  filecountlabel->setMinimumSize(filecountlabel->sizeHint());
  headerLayout->addWidget(filecountlabel, 0, 2);

  fileCount = new QLineEdit(this);
  fileCount->setFocusPolicy(QWidget::NoFocus);
  fileCount->setMinimumSize(fileCount->sizeHint());
  fileCount->setMaximumHeight(fileCount->sizeHint().height());
  headerLayout->addWidget(fileCount, 0, 3);

  QLabel *operationlabel;

  if (icqFileSession->isSending())
    operationlabel=new QLabel("Reading:", this);
  else
    operationlabel=new QLabel("Saving To:", this);

  operationlabel->setMinimumSize(operationlabel->sizeHint());
  headerLayout->addWidget(operationlabel, 1, 0);

  operationFileName = new QLineEdit(this);
  operationFileName->setFocusPolicy(QWidget::NoFocus);
  operationFileName->setMinimumSize(operationFileName->sizeHint());
  operationFileName->
    setMaximumHeight(operationFileName->sizeHint().height());
  headerLayout->addWidget(operationFileName, 1, 1);

  QLabel *statuslabel = new QLabel("Status:", this);
  statuslabel->setMinimumSize(statuslabel->sizeHint());
  headerLayout->addWidget(statuslabel, 1, 2);
  
  statusText = new QLineEdit(this);
  statusText->setFocusPolicy(QWidget::NoFocus);
  statusText->setMinimumSize(statusText->sizeHint());
  operationFileName->
    setMaximumHeight(statusText->sizeHint().height());
  headerLayout->addWidget(statusText, 1, 3);

  // Bar

  QFrame *line1 = new QFrame( this );
  line1->setFrameStyle( QFrame::HLine | QFrame::Sunken );
  line1->setLineWidth( 1 );

  mainLayout->addWidget(line1, 1, 0);

  // Progress section

  QGridLayout *midLayout = new QGridLayout(3, 6, 7);
  mainLayout->addLayout(midLayout, 2, 0);
  midLayout->setColStretch(0, 0);
  midLayout->setColStretch(1, 1);
  midLayout->setColStretch(2, 0);
  midLayout->setColStretch(3, 1);
  midLayout->setColStretch(4, 0);
  midLayout->setColStretch(5, 3);

  QLabel *progresslabel = new QLabel("File:", this);
  progresslabel->setMinimumSize(progresslabel->sizeHint());
  midLayout->addWidget(progresslabel, 0, 0);

  progressBar = new QProgressBar(this);
  //progressBar->setMinimumSize(progressBar->sizeHint());
  midLayout->addMultiCellWidget(progressBar, 0, 0, 1, 3);

  QLabel *sizelabel1 = new QLabel("Size:", this);
  sizelabel1->setMinimumSize(sizelabel1->sizeHint());
  midLayout->addWidget(sizelabel1, 0, 4);

  fileSizeText = new QLineEdit(this);
  fileSizeText->setFocusPolicy(QWidget::NoFocus);
  int width=fontMetrics().width("1000.00 MB of 1000.00 MB   ");
  fileSizeText->setMinimumSize(width,fileSizeText->sizeHint().height());
  fileSizeText->setMaximumHeight(fileSizeText->sizeHint().height());
  midLayout->addWidget(fileSizeText, 0, 5);

  QLabel *progresslabel2 = new QLabel("Batch:", this);
  progresslabel2->setMinimumSize(progresslabel2->sizeHint());
  midLayout->addWidget(progresslabel2, 1, 0);

  batchProgressBar = new QProgressBar(this);
  //batchProgressBar->setMinimumSize(batchProgressBar->sizeHint());
  midLayout->addMultiCellWidget(batchProgressBar, 1, 1, 1, 3);

  QLabel *sizelabel2 = new QLabel("Size:", this);
  sizelabel2->setMinimumSize(sizelabel2->sizeHint());
  midLayout->addWidget(sizelabel2, 1, 4);

  batchSizeText = new QLineEdit(this);
  batchSizeText->setFocusPolicy(QWidget::NoFocus);
  width=fontMetrics().width("1000.00 MB of 1000.00 MB   ");
  batchSizeText->setMinimumSize(width,batchSizeText->sizeHint().height());
  batchSizeText->setMaximumHeight(batchSizeText->sizeHint().height());
  midLayout->addWidget(batchSizeText, 1, 5);

  QLabel *elapsedlabel = new QLabel("Elapsed:", this);
  elapsedlabel->setMinimumSize(elapsedlabel->sizeHint());
  midLayout->addWidget(elapsedlabel, 2, 0);

  elapsedText = new QLineEdit(this);
  elapsedText->setFocusPolicy(QWidget::NoFocus);
  elapsedText->setMinimumSize(elapsedText->sizeHint());
  elapsedText->setMaximumHeight(elapsedText->sizeHint().height());
  midLayout->addWidget(elapsedText, 2, 1);

  QLabel *remaininglabel = new QLabel("Remaining:", this);
  remaininglabel->setMinimumSize(remaininglabel->sizeHint());
  midLayout->addWidget(remaininglabel, 2, 2);

  remainingText = new QLineEdit(this);
  remainingText->setFocusPolicy(QWidget::NoFocus);
  remainingText->setMinimumSize(remainingText->sizeHint());
  remainingText->setMaximumHeight(remainingText->sizeHint().height());
  midLayout->addWidget(remainingText, 2, 3);

  QLabel *cpslabel = new QLabel("CPS:", this);
  cpslabel->setMinimumSize(cpslabel->sizeHint());
  midLayout->addWidget(cpslabel, 2, 4);

  cpsText = new QLineEdit(this);
  cpsText->setFocusPolicy(QWidget::NoFocus);
  cpsText->setMinimumSize(cpsText->sizeHint());
  cpsText->setMaximumHeight(cpsText->sizeHint().height());
  midLayout->addWidget(cpsText, 2, 5);

  // Bar

  QFrame *line2 = new QFrame( this );
  line2->setFrameStyle( QFrame::HLine | QFrame::Sunken );
  line2->setLineWidth( 1 );

  mainLayout->addWidget(line2, 3, 0);

  // Bottom

  QGridLayout *bottomLayout = new QGridLayout(1, 4);
  mainLayout->addLayout(bottomLayout, 4, 0);
  bottomLayout->setColStretch(0, 0);
  bottomLayout->setColStretch(1, 1);
  bottomLayout->setColStretch(2, 0);
  bottomLayout->setColStretch(3, 0);
  
  QLabel *speedlabel = new QLabel("Speed:", this);
  speedlabel->setMinimumSize(speedlabel->sizeHint());
  bottomLayout->addWidget(speedlabel, 0, 0);

  speedSlider = new QSlider(0, 100, 10, 0, QSlider::Horizontal,
    this);
  speedSlider->setMinimumSize(speedSlider->sizeHint());
  speedSlider->setMaximumHeight(speedSlider->sizeHint().height());
  speedSlider->setTickInterval(10);
  speedSlider->setTickmarks(QSlider::Above);
  connect(speedSlider, SIGNAL(valueChanged(int)),
    SLOT(sliderChanged(int)));
  bottomLayout->addWidget(speedSlider, 0, 1);

  QPushButton *skipButton = new QPushButton(this);
  skipButton->setText("Skip File");
  skipButton->setMinimumSize(skipButton->sizeHint());
  bottomLayout->addWidget(skipButton, 0, 2);
  skipButton->setEnabled(false);

  QPushButton *abortButton = new QPushButton(this);
  abortButton->setText("Abort");
  abortButton->setMinimumSize(abortButton->sizeHint());
  bottomLayout->addWidget(abortButton, 0, 3);
  connect(abortButton, SIGNAL(clicked()), SLOT(abortClicked()));

  mainLayout->activate();

  resize(500,250);
  setMinimumSize(400,250);

  connect( icqFileSession, SIGNAL(progress(unsigned long, unsigned long)),
    SLOT(progress(unsigned long, unsigned long)) );
  connect( icqFileSession, SIGNAL(statusChanged(int)),
    SLOT(statusChanged(int)) );
}

FileWindow::~FileWindow()
{
  //
}

void FileWindow::progress(unsigned long current, unsigned long total)
{
  progressBar->setProgress(current*current_scale);
  batchProgressBar->setProgress(total*total_scale);

  fileSizeText->setText(reduceNumber(current) +
      " of " + reduceNumber(icqFileSession->currentFileSize()));
  batchSizeText->setText(reduceNumber(total) +
      " of " + reduceNumber(icqFileSession->totalSize()));
  if(!cps_start_time)
  {
    start_bytes=current-2048;
    cps_start_time=time(0L);
    cps_timer=time(0L);
  }   
  if(time(0L)>cps_timer)
  {
    int seconds=time(0L)-cps_start_time;
    cps=(current-start_bytes)/seconds;
    cpsText->setText(reduceNumber(cps) + "/sec");
    cps_timer=time(0L);
  }
}

void FileWindow::statusChanged(int status)
{
  QTimer *timer;

  switch(status) {
    case FILE_STATUS_LISTENING:
      statusText->setText("Waiting for connection...");
      break;
    case FILE_STATUS_CONNECTING:
      statusText->setText("Connecting...");
      break;
    case FILE_STATUS_CONNECTED:
      statusText->setText("Connected.");
      break;
    case FILE_STATUS_INITIALIZED:
      total_scale=PROGRESS_SCALE/icqFileSession->totalSize();
      batchProgressBar->setTotalSteps(PROGRESS_SCALE);
      fileCount->setText(
        QString().sprintf("0/%d", icqFileSession->totalFiles()));
      if (icqFileSession->isReceiving())
         setCaption(QString("kicq - Receiving files from ") + 
           icqFileSession->remoteHandle());
      else
         setCaption(QString("kicq - Sending files to ") +
           icqFileSession->remoteHandle());

      timer = new QTimer( this );
      connect( timer, SIGNAL(timeout()), SLOT(updateTimes()) );
      timer->start( 1000 ); 
      elapsed_time=0;

      statusText->setText("Waiting for next file...");
      break;

    case FILE_STATUS_NEXT_FILE:
      headerFileName->setText(icqFileSession->currentFile());
      operationFileName->setText(icqFileSession->currentFile());
      fileCount->setText(QString().sprintf("%d/%d",
        icqFileSession->currentFileNumber(),
        icqFileSession->totalFiles()));
      current_scale=PROGRESS_SCALE/icqFileSession->currentFileSize();
      progressBar->reset();
      progressBar->setTotalSteps(PROGRESS_SCALE);
      cps_start_time=0;
      speedSlider->setValue(icqFileSession->speed());
      statusText->setText("Waiting for data...");
      break;

    case FILE_STATUS_NEW_SPEED:
      speedSlider->setValue(icqFileSession->speed());
      break;

    case FILE_STATUS_SENDING:
      statusText->setText("Sending...");
      break;

    case FILE_STATUS_RECEIVING:
      statusText->setText("Receiving...");
      break;

    default:
      break;
  }
}

void FileWindow::requestNotify(int type, int length, void *data)
{
  icqFileSession->requestNotify(type, length, data);

  IcqWindow::requestNotify(type, length, data);
}

void FileWindow::closeEvent(QCloseEvent *ce)
{
  bool complete = requestComplete();
  IcqWindow::closeEvent(ce);
 
  if (!complete)
    icqFileSession->close();
}

void FileWindow::resizeEvent(QResizeEvent *re)
{
  IcqWindow::resizeEvent(re);
}

void FileWindow::sliderChanged(int new_value)
{
  if(icqFileSession && icqFileSession->isTransferring())
    icqFileSession->setSpeed(new_value);
}

void FileWindow::setWorkingDirectory(const QString &directory)
{
  icqFileSession->setWorkingDir(directory);
}

void FileWindow::abortClicked()
{
  int result=QMessageBox::warning(this, "Warning", 
    "This will stop the file transfer.  Are you sure?",
    QMessageBox::Yes | QMessageBox::Default,
    QMessageBox::No | QMessageBox::Escape);

  if(result==QMessageBox::Yes)
    close();
}

void FileWindow::updateTimes()
{
  elapsed_time++;

  elapsedText->setText(formatTime(elapsed_time));
  long remaining=icqFileSession->totalSize()-icqFileSession->totalProgress();
  if (cps) 
  {
    remainingText->setText(formatTime(remaining/cps));
  }
}

QString FileWindow::formatTime(long seconds)
{
  long minutes=seconds/60;
  long hours=minutes/60;

  return QString().sprintf("%02d:%02d:%02d", (int)hours, (int)minutes, (int)(seconds % 60));
}
