/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
#ifndef _FILEWINDOW_
#define _FILEWINDOW_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <qdatetime.h>
#include <time.h>
#include <qsplitter.h>
#include <qlineedit.h>
#include <qprogressbar.h>
#include <qslider.h>
#include <qbutton.h>

#include <icq.h>

#include "icqwindow.h"
#include "icqfilesession.h"

#define PROGRESS_SCALE       1024.0

class FileWindow : public IcqWindow {
  Q_OBJECT
public:
  FileWindow(icq_FileSession *, unsigned long);
  ~FileWindow();
  unsigned long id() { return m_id; }
  void filePacket(int);
  void requestNotify(int type, int length, void *data);
  void setWorkingDirectory( const QString &directory );
  void acceptFileRequest();

public slots:
  void statusChanged(int status);
  void progress(unsigned long current, unsigned long total);
  void updateTimes();

signals:
protected:
  unsigned long remote_uin;
  unsigned long m_id;
  IcqFileSession *icqFileSession;
  QString working_directory;

  void resizeEvent(QResizeEvent *);
  void closeEvent(QCloseEvent *);

  QString formatTime(long seconds);

protected slots:
  void abortClicked();

private:
  // GUI
  QLineEdit *headerFileName;
  QLineEdit *fileText;
  QLineEdit *operationFileName;
  QLineEdit *statusText;

  QProgressBar *progressBar;  
  QProgressBar *batchProgressBar;

  QLineEdit *fileCount;
  QLineEdit *fileSizeText;
  QLineEdit *batchSizeText;

  QLineEdit *status;
  QLineEdit *elapsedText, *remainingText;
  QLineEdit *cpsText;

  QSlider *speedSlider;

  QButton *skipButton, abortButton;

  // Status
  time_t cps_timer, cps_start_time;
  unsigned long start_bytes;

  double total_scale, current_scale, cps;

  long elapsed_time;

private slots:
  void sliderChanged(int new_value);

};
#endif /* _FILEWINDOW_ */
