//
// $Id: icqfilesession.cpp,v 1.3 2000/07/30 18:53:25 denis Exp $
//
// Wrapper class for icqlib's icq_FileSession struct.
//
// $Log: icqfilesession.cpp,v $
// Revision 1.3  2000/07/30 18:53:25  denis
// Warnings were fixed.
//
// Revision 1.2  2000/06/17 17:05:27  denis
// Cleanups.
//
// Revision 1.1  2000/05/24 16:57:49  denis
// Sources have been splitted to the whole bunch of appropriate
// directories. Every directory is end up with static library. All
// of them are linked together to the final kicq binary.
//
// Revision 1.1  2000/05/04 16:08:40  bills
// added
//
//



#include "icqfilesession.h"

#include "icqfilesession.moc"

IcqFileSession::IcqFileSession(icq_FileSession *psession, QObject *parent, 
  const char *name ) :
  QObject( parent, name ), m_session(psession)
{

}

IcqFileSession::~IcqFileSession()
{

}

void IcqFileSession::requestNotify(int type, int length, void *data)
{
  if (type==ICQ_NOTIFY_FILE)
    emit statusChanged(length);

  if (type==ICQ_NOTIFY_FILEDATA)
    emit progress(currentFileProgress(), totalProgress());
}

void IcqFileSession::setSpeed(int speed)
{
  icq_FileSessionSetSpeed(m_session, speed);
}

void IcqFileSession::setWorkingDir(const QString &directory)
{
  icq_FileSessionSetWorkingDir(m_session, directory);
}

void IcqFileSession::setFiles(const QStrList &files)
{
  // TODO
}

bool IcqFileSession::isTransferring() const
{
  return m_session->status==FILE_STATUS_SENDING ||
    m_session->status==FILE_STATUS_RECEIVING;
}

bool IcqFileSession::isSending() const
{
  return m_session->direction==FILE_STATUS_SENDING;
}

bool IcqFileSession::isReceiving() const
{
  return m_session->direction==FILE_STATUS_RECEIVING;
}

unsigned long IcqFileSession::remoteUIN() const
{
  return m_session->remote_uin;
}

QString IcqFileSession::remoteHandle() const
{
  return QString().sprintf("%s", m_session->remote_handle);
}

int IcqFileSession::totalFiles() const
{
  return m_session->total_files;
}

int IcqFileSession::currentFileNumber() const 
{
  return m_session->current_file_num;
}

unsigned long IcqFileSession::totalSize() const
{
  return m_session->total_bytes;
}

unsigned long IcqFileSession::totalProgress() const
{
  return m_session->total_transferred_bytes;
}

QString IcqFileSession::workingDir() const
{
  return QString().sprintf("%s", m_session->working_dir);
}

QString IcqFileSession::currentFile() const 
{
  return QString().sprintf("%s", m_session->current_file);
}

unsigned long IcqFileSession::currentFileSize() const 
{
  return m_session->current_file_size;
}

unsigned long IcqFileSession::currentFileProgress() const
{
  return m_session->current_file_progress;
}

int IcqFileSession::speed() const
{
  return m_session->current_speed;
}

void IcqFileSession::close()
{
  icq_FileSessionClose(m_session);
}
