/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
// $Id: historywin.cpp,v 1.2 2000/06/17 17:38:46 denis Exp $
// $Log: historywin.cpp,v $
// Revision 1.2  2000/06/17 17:38:46  denis
// Contact List code was rewritten from scratch! Although we are trying
// to not add new features in anticipation of the next stable release,
// but this rewrite solves many things, removes many hacks and stabilizes
// whole program.
// One of bug, needs to mention, is an absence of IP address in Info
// window. The root of problem was in Contact List, because in any time
// there were several ContactItems in memory for single user - they were
// "smeared" on the whole code. Now it's clean and strict - all Contact
// List operations can be done only by ContactList class.
//
// Revision 1.1  2000/05/24 16:58:07  denis
// Sources have been splitted to the whole bunch of appropriate
// directories. Every directory is end up with static library. All
// of them are linked together to the final kicq binary.
//
// Revision 1.24  2000/05/05 03:38:26  bills
// added code to restore filesize for MSG_FILE_REQ_RECV
//
// Revision 1.23  2000/05/04 16:10:55  bills
// icqMessages are now expired by calling expire() when read from history so
// action buttons are disabled when viewing history
//
// Revision 1.22  2000/02/03 15:54:06  bills
// converted to use new typeString method in icqmessage
//
// Revision 1.21  2000/01/10 16:08:20  denis
// Headers reorganized for simplify dependencies and speedup compilation.
//
// Revision 1.20  1999/12/28 01:40:33  bills
// added code to readHistory & refresh routines to handle chat requests and
// file transfers
//
// Revision 1.19  1999/12/02 14:20:33  denis
// Layout management implemented.
// Random IcqWindow segfaults fixed.
// WebPager and EMailExpress messages handlers added.
// Try to fix Authorization request problems.
// Old Qt-1.3x code removed.
// Cleanups.
//
// Revision 1.18  1999/10/14 14:18:37  denis
// Removed i18n from history file delimiters.
//
// Revision 1.17  1999/09/29 17:30:27  denis
// Cleanups.
//
// Revision 1.16  1999/07/16 15:47:59  denis
// Cleaned up.
//
// Revision 1.15  1999/07/16 13:05:22  denis
// Internalization support added.
//
// Revision 1.14  1999/07/12 15:23:22  cproch
// - changed icq_* function calls and callback function to support
//   icqlib with ICQLINK identifier.
// - look for FIXME in mainwindow.cpp
//
// Revision 1.13  1999/04/17 20:22:15  bills
// modified for new icqwindow base class
//
// Revision 1.12  1999/03/16 18:01:52  whale
//
//
// Quick hack to fix following kicq bug:
// when you open history window, double click on some event, then close history
// window and press 'Read Next' in receive window, kicq crashes.
// Now it closes all receive windows derived from history window on history
// window close.
//
// Revision 1.11  1999/03/05 13:04:35  denis
// Applied patch to handle authorization request messages by Bogdan Surdu
// <tim@ima-info.ro>
//
// Revision 1.10  1998/12/11 18:18:14  whale
// Added support for MSG_AUTH_RECV (incoming auth requests)
//
// Revision 1.9  1998/12/08 16:16:22  denis
// Cleaned up a little before releasing
//
// Revision 1.8  1998/11/25 12:03:53  denis
// Trying to catch QGDict::look: Attempt to insert null item
//

#include <unistd.h>
#include <qmsgbox.h>
#include <qmovie.h>
#include <qregexp.h>
#include <qkeycode.h>
#include <qfile.h>
#include <qfileinf.h>
#include <qmsgbox.h>
#include <qlayout.h>
#include <kapp.h>
#include <kiconloader.h>

#include "historywin.h"
#include "mainwindow.h"
#include "userlist.h"
#include "mlinelistbox.h"

#include "historywin.moc"

HistoryWindow::HistoryWindow(unsigned long u) : IcqWindow()
{
  uin = u;

  QString caption;
  caption.sprintf(i18n("History Events of: %s"), (const char*)MW->contactList().getNickByUIN(uin));
  setCaption((const char*)caption);

  msgList.setAutoDelete(TRUE);
  rwList.setAutoDelete(TRUE);

  QBoxLayout *topLayout = new QVBoxLayout(this, 7);

  KTabCtl *tabctl = new KTabCtl(this);
  allevents = new AllEventsWidget(tabctl);
  inevents = new InOutEventsWidget(tabctl, MSG_IN);
  outevents = new InOutEventsWidget(tabctl, MSG_OUT);
  tabctl->addTab(allevents,i18n("All Events"));
  tabctl->addTab(inevents,i18n("Incoming Events"));
  tabctl->addTab(outevents,i18n("Outgoing Events"));
  connect(tabctl,SIGNAL(tabSelected(int)), SLOT(changeTab(int)));
  curtab = 0;
  topLayout->addWidget(tabctl, 1);

  QBoxLayout *butLayout = new QHBoxLayout(7);
  topLayout->addLayout(butLayout);

  QPushButton *findbtn = new QPushButton(i18n("Find"),this);
  findbtn->setEnabled(FALSE);
  findbtn->setFixedSize(findbtn->sizeHint());
  // Reserved for future releases :)
  // connect(findbtn,SIGNAL(clicked()),SLOT(findCmd())); 
  butLayout->addWidget(findbtn);

  QPushButton *saveasbtn = new QPushButton(i18n("Save As"),this);
  saveasbtn->setEnabled(FALSE);
  saveasbtn->setFixedSize(saveasbtn->sizeHint());
  // Reserved for future releases :)
  // connect(saveasbtn,SIGNAL(clicked()),SLOT(saveAsCmd())); 
  butLayout->addWidget(saveasbtn);

  deletebtn = new QPushButton(i18n("Delete"),this);
  deletebtn->setEnabled(FALSE);
  deletebtn->setFixedSize(deletebtn->sizeHint());
  connect(deletebtn,SIGNAL(clicked()),SLOT(deleteCmd()));
  butLayout->addWidget(deletebtn);

  QPushButton *deleteallbtn = new QPushButton(i18n("Delete All"),this);
  deleteallbtn->setFixedSize(deleteallbtn->sizeHint());
  connect(deleteallbtn,SIGNAL(clicked()),SLOT(deleteAllCmd()));
  butLayout->addWidget(deleteallbtn);

  orderbtn = new QCheckBox(i18n("Show most recent event first"),this);
  orderbtn->setChecked(TRUE);
  orderbtn->setFixedHeight(orderbtn->sizeHint().height());
  orderbtn->setMinimumWidth(orderbtn->sizeHint().width());
  connect(orderbtn,SIGNAL(clicked()),SLOT(refresh()));
  butLayout->addWidget(orderbtn, 1);

  QPushButton *closebtn = new QPushButton(i18n("Close"),this);
  closebtn->setFixedSize(closebtn->sizeHint());
  connect(closebtn,SIGNAL(clicked()),SLOT(closeCmd()));
  butLayout->addWidget(closebtn);

  topLayout->activate();

  refresh();
  resize(1, 1);
}

HistoryWindow::~HistoryWindow()
{
  rwList.clear();
  msgList.clear();
}

void HistoryWindow::listSelected()
{
  deletebtn->setEnabled(TRUE);
}

void HistoryWindow::closeCmd()
{
  close();
}

void HistoryWindow::deleteCmd()
{
  int n;
  switch(curtab)
  {
    case 0:
      n = allevents->currentItem();
      break;
    case 1:
      n = inevents->currentItem();
      break;
    case 2:
      n = outevents->currentItem();
      break;
    default:
      n = -1;
  }
  if(n == -1)
    return;

  if(QMessageBox::warning(this, i18n("Confirm"),
     i18n("Are you sure you want to delete this event?"), QMessageBox::Yes, 
     QMessageBox::No | QMessageBox::Default | QMessageBox::Escape) != QMessageBox::Yes)
    return;

  if(orderbtn->isChecked())
    msgList.remove(n);
  else
    msgList.remove(msgList.count() - n - 1);

  QString strUin;
  strUin.setNum(uin);

  QString historypath = MW->contactsDir() 
    + "/" + strUin + ".history";

  QFile hfile(historypath);
  QFileInfo hinfo(hfile);

  if(!hinfo.isWritable())
  {
    QMessageBox::warning(this, i18n("Error"), i18n("Cannot write to history file"));
    return;
  }

  unlink(hfile.name());

  icqMessage *ptr;
  for(ptr = msgList.first(); ptr != 0L; ptr = msgList.next())
    ptr->save(MW->contactsDir());

  refresh();
}

void HistoryWindow::deleteAllCmd()
{
  if(QMessageBox::warning(this, i18n("Confirm"),
     i18n("Are you sure you want to delete ALL events?"), QMessageBox::Yes, 
     QMessageBox::No) == QMessageBox::No)
    return;

  QString strUin;
  strUin.setNum(uin);

  QString historypath = MW->contactsDir() 
    + "/" + strUin + ".history";

  QFile hfile(historypath);
  QFileInfo hinfo(hfile);

  if(!hinfo.isWritable())
  {
    QMessageBox::warning(this,i18n("Error"),i18n("Cannot write to history file"));
    return;
  }

  unlink(hfile.name());
  refresh();
}

void HistoryWindow::changeTab(int tab)
{
  curtab = tab;
  refresh();
}

void HistoryWindow::showDetail()
{
  int n;
  switch(curtab)
  {
    case 0:
      n = allevents->currentItem();
      break;
    case 1:
      n = inevents->currentItem();
      break;
    case 2:
      n = outevents->currentItem();
      break;
    default:
      n = -1;
  }
  if(n == -1)
    return;
  
  if(!orderbtn->isChecked())
    n = msgList.count() - n - 1;

  RecvWindow *rw = new RecvWindow(&msgList, n, UIN(), FALSE);
  rw->show();
  rwList.append(rw);
}

void HistoryWindow::refresh()
{
  readHistory();
  ascending = orderbtn->isChecked();  
  switch(curtab)
  {
    case 0:
      allevents->refresh(&msgList, ascending);
      break;
    case 1:
      inevents->refresh(&msgList, ascending);
      break;
    case 2:
      outevents->refresh(&msgList, ascending);
      break;
  }
}

void HistoryWindow::readHistory()
{

  msgList.clear();

  QString strUin;
  strUin.setNum(uin);

  QString historypath = MW->contactsDir() 
    + "/" + strUin + ".history";

  QFile hfile(historypath);

  if(!hfile.exists())
    return;

  hfile.open(IO_ReadOnly);

  QRegExp *r[13];
  QString re;

  for(int i = 0; i < 13; i++)
  {
    re = (QString)"\\[" + icqMessage::typeString((icqMsgType)i) + ".../../.......:..\\]";
    r[i] = new QRegExp(re);
  }

  int i, c, pos, type = -1;
  QString s, nextmsg;
  TimeStr nexttime;  

  do
  {
    char buffer[512];
    c = hfile.readLine((char*)&buffer, 512);

    s = buffer;

    for(i = 0; i < 13; i++)
    {
      if((pos = r[i]->match(s)) != -1 || c == -1)
      {
        if(type != -1)
        {
          icqMessage *msg;
          QString url, nick, firstname, lastname, email, file, filesize;
          switch(type)
          {
            case MSG_RECV:
            case MSG_SENT:
              msg = new icqMessage(uin,nexttime,(const char*)nextmsg,(icqMsgType)type);
              break;
            case MSG_URL_RECV:
            case MSG_URL_SENT:
              url = (const char*)nextmsg;
              url.remove(url.find('\n',0) + 1,url.length());
              
              nextmsg.remove(0,nextmsg.find('\n',0) + 1);
              msg = new icqMessage(uin, nexttime, (const char*)url, (const char*)nextmsg,
                                   (icqMsgType)type);
              break;
            case MSG_AUTH_RECV:
            case MSG_AUTH_REFUSE:
            case MSG_AUTH_GRANT:
            case MSG_ADDED:
              nick = (const char*)nextmsg;
              nick.remove(nick.find('\n',0),nick.length());
              nextmsg.remove(0,nextmsg.find('\n',0) + 1);
              firstname = (const char*)nextmsg;
              firstname.remove(firstname.find('\n',0),firstname.length());
              nextmsg.remove(0,nextmsg.find('\n',0) + 1);
              lastname = (const char*)nextmsg;
              lastname.remove(lastname.find('\n',0),lastname.length());
              nextmsg.remove(0,nextmsg.find('\n',0) + 1);
              email = (const char*)nextmsg;
              email.remove(email.find('\n',0),email.length());
              nextmsg.remove(0,nextmsg.find('\n',0) + 1);
              if(type == MSG_AUTH_RECV)
                msg = new icqMessage(uin,nexttime,(const char*)nick,
                                     (const char*)firstname,
                                     (const char*)lastname,
                                     (const char*)email,
                                     (const char*)nextmsg,(icqMsgType)type);
              else
                msg = new icqMessage(uin,nexttime,(const char*)nick,
                                     (const char*)firstname,
                                     (const char*)lastname,
                                     (const char*)email, (icqMsgType)type);
              break;
            case MSG_PAGER_RECV:
            case MSG_EXPRESS_RECV:
              nick = (const char*)nextmsg;
              nick.remove(nick.find('\n',0),nick.length());
              nextmsg.remove(0,nextmsg.find('\n',0) + 1);
              email = (const char*)nextmsg;
              email.remove(email.find('\n',0),email.length());
              nextmsg.remove(0,nextmsg.find('\n',0) + 1);
              msg = new icqMessage(uin,nexttime,(const char*)nick,
                                   (const char*)email,
                                   (const char*)nextmsg,(icqMsgType)type);
              break;
            case MSG_CHAT_REQ_RECV:
              msg = new icqMessage(uin, nexttime,(const char *)nextmsg,
                0UL, (icqMsgType)type);
              break;
            case MSG_FILE_REQ_RECV:
              file = (const char *)nextmsg;
              file.remove(file.find('\n',0),file.length());
              nextmsg.remove(0,nextmsg.find('\n',0) + 1);
              filesize = (const char *)nextmsg;
              filesize.remove(filesize.find('\n',0),filesize.length());
              if (nextmsg.find('\n',0)>-1)
                nextmsg.remove(0,nextmsg.find('\n',0) + 1);
              msg = new icqMessage(uin, nexttime, (const char *)nextmsg,
                (const char *)file, filesize.toULong(), 0UL, (icqMsgType)type);
              break;              
            default:
              msg = new icqMessage(0,nexttime,i18n("unknown\n"),MSG_UNDEF);
          }

          msg->expire();
          msgList.append(msg);
          if(c == -1)
            break;
        }
        if(c != -1)
        {
          type = i;
          nextmsg.resize(0);
          pos += strlen(icqMessage::typeString((icqMsgType)i)) + 2;

          QString timestr = s.right(s.length() - pos);
          int day,month,year,hour,minute;
          sscanf(timestr,"%02d/%02d/%04d %02d:%02d]",&day,&month,&year,
            &hour,&minute);
          nexttime.day = day;
          nexttime.month = month;
          nexttime.year = year;
          nexttime.minute = minute;
          nexttime.hour = hour;
          break;
        }
      }
    }
    if(i == 13)
    {
      nextmsg += s;
    }
  } while(c != -1);
    
  hfile.close();

  for(i = 0; i < 13; i ++)
    delete r[i];
}

unsigned long HistoryWindow::UIN() const 
{
  return uin;
}

AllEventsWidget::AllEventsWidget(QWidget *p) : QWidget(p)
{
  QBoxLayout *topLayout = new QVBoxLayout(this, 7);

  msglist = new QListView(this);
  msglist->addColumn(i18n("Event"),120);
  msglist->addColumn(i18n("Date"),110);
  msglist->addColumn(i18n("Message"),349);
  msglist->setAllColumnsShowFocus(TRUE);
  msglist->setFrameStyle(QFrame::WinPanel | QFrame::Sunken);
  msglist->setMinimumSize(300, 300);
  topLayout->addWidget(msglist, 1);

  connect(msglist,SIGNAL(doubleClicked(QListViewItem *)),p->parent(),SLOT(showDetail()));
  connect(msglist,SIGNAL(returnPressed(QListViewItem *)),p->parent(),SLOT(showDetail()));
  connect(msglist,SIGNAL(selectionChanged()),p->parent(),SLOT(listSelected()));

  topLayout->activate();
}

void AllEventsWidget::resizeEvent(QResizeEvent *ev)
{
  QWidget::resizeEvent(ev);

  QSize s = msglist->verticalScrollBar()->sizeHint();

  int w = msglist->width() - s.width() - 4;
  msglist->setColumnWidth(0,w < 480 ? w/4 : 120);
  msglist->setColumnWidth(1,w < 480 ? w/4 : 120 );
  msglist->setColumnWidth(2,w > 480 ? w - 240 : 240);

  msglist->update();
}

int AllEventsWidget::currentItem()
{
  WMlineListBoxItem *i = (WMlineListBoxItem *)msglist->currentItem(); 
  int n = atoi(i->text(3));
  return n;
}

void AllEventsWidget::refresh(QList<icqMessage> *l,int asc)
{
  msglist->clear();

  KIconLoader *il = new KIconLoader();

  icqMessage *ptr;

  int n = 0;

  for(ptr = (asc ? l->first() : l->last()); ptr != 0L; 
    ptr = (asc ? l->next() : l->prev()))
  {
    QString icon;
    QString body = ptr->body();
    body.replace(QRegExp("\r"),"");

    switch(ptr->type())
    {
      case MSG_SENT:
        icon = "message.xpm";
        break;
      case MSG_RECV:
        icon = "message.xpm";
        break;
      case MSG_URL_SENT:
        icon = "url.xpm";
        body = (QString)ptr->url() + "\n" + body;
        break;
      case MSG_URL_RECV:
        icon = "url.xpm";
        body = (QString)ptr->url() + "\n" + body;
        break;
      case MSG_AUTH_RECV:
        icon = "auth.xpm";
        body = (QString)ptr->nick() + ": " + (QString)ptr->first() + " " +
               (QString)ptr->last() + " <" + (QString)ptr->email() + ">\n" +
               body;
        break;
      case MSG_AUTH_GRANT:
        icon = "auth.xpm";
        body = (QString)ptr->nick() + ": " + (QString)ptr->first() + " " +
               (QString)ptr->last() + " <" + (QString)ptr->email() + ">\n" +
               body;
        break;
      case MSG_AUTH_REFUSE:
        icon = "auth.xpm";
        body = (QString)ptr->nick() + ": " + (QString)ptr->first() + " " +
               (QString)ptr->last() + " <" + (QString)ptr->email() + ">\n" +
               body;
        break;
      case MSG_ADDED:
        icon = "exclam.xpm";
        body = (QString)ptr->nick() + ": " + (QString)ptr->first() + " " +
               (QString)ptr->last() + " <" + (QString)ptr->email() + ">";
        break;
      case MSG_PAGER_RECV:
        icon = "exclam.xpm";
        body = (QString)ptr->nick() + ": " + " <" + (QString)ptr->email() + ">";
        break;
      case MSG_EXPRESS_RECV:
        icon = "exclam.xpm";
        body = (QString)ptr->nick() + ": " + " <" + (QString)ptr->email() + ">";
        break;
      case MSG_CHAT_REQ_RECV:
        icon = "chat.xpm";
        break;
      case MSG_FILE_REQ_RECV:
        icon = "file.xpm";
        break;
      default:
        icon = "null.xpm";
    }

    QPixmap pm = il->loadIcon(icon);

    TimeStr t = ptr->time();
    QString ts;
    ts.sprintf("%02i/%02i/%04i %02i:%02i\n",t.day,t.month,t.year,t.hour,t.minute);

    WMlineListBoxItem *litem = new WMlineListBoxItem(msglist, 2);

    QString event = ptr->typeString();
//    event += ptr->typeString();
    event += "\n";

    litem->setPixmap(0, pm);
    litem->setText(0, (const char *)event);
    litem->setText(1, (const char*)ts);
    litem->setText(2, body);
    QString num;
    num.setNum(n);
    litem->setText(3,(const char*)num);
    n++;
  }
  msglist->update();
}

InOutEventsWidget::InOutEventsWidget(QWidget *p, InOut d) : QWidget(p)
{
  direction = d;

  QBoxLayout *topLayout = new QVBoxLayout(this, 7);

  msglist = new QListView(this);
  msglist->addColumn(i18n("Event"));
  msglist->addColumn(i18n("Time"));
  msglist->addColumn(i18n("Date"));
  msglist->addColumn(i18n("Message"));
  msglist->setAllColumnsShowFocus(TRUE);
  msglist->setFrameStyle(QFrame::WinPanel | QFrame::Sunken);
  msglist->setMinimumSize(300, 300);
  topLayout->addWidget(msglist, 1);

  connect(msglist,SIGNAL(doubleClicked(QListViewItem *)),p->parent(),SLOT(showDetail()));
  connect(msglist,SIGNAL(returnPressed(QListViewItem *)),p->parent(),SLOT(showDetail()));
  connect(msglist,SIGNAL(selectionChanged()),p->parent(),SLOT(listSelected()));

  topLayout->activate();
}

void InOutEventsWidget::resizeEvent(QResizeEvent *ev)
{
  QWidget::resizeEvent(ev);

  QSize s = msglist->verticalScrollBar()->sizeHint();
  int w = msglist->width() - s.width() - 4;
  msglist->setColumnWidth(0,w < 480 ? w/6 : 80);
  msglist->setColumnWidth(1,w < 480 ? w/6 : 70);
  msglist->setColumnWidth(2,w < 480 ? w/6 : 80);
  msglist->setColumnWidth(3,w > 480 ? w - 230 : 250);
  msglist->update();
}

int InOutEventsWidget::currentItem()
{
  int n;
  WMlineListBoxItem *i = (WMlineListBoxItem*)msglist->currentItem();
  n = atoi(i->text(4));
  return n;
}

void InOutEventsWidget::refresh(QList<icqMessage> *l,int asc)
{
  msglist->clear();

  KIconLoader *il = new KIconLoader();

  QPixmap pm;
  QPixmap pmap_url = il->loadIcon("url.xpm");
  QPixmap pmap_msg = il->loadIcon("message.xpm");
  QPixmap pmap_auth = il->loadIcon("auth.xpm");
  QPixmap pmap_exclam = il->loadIcon("exclam.xpm");
  QPixmap pmap_file = il->loadIcon("file.xpm");
  QPixmap pmap_chat = il->loadIcon("chat.xpm");

  icqMessage *ptr;
  int n = 0;

  for(ptr = (asc ? l->first() : l->last()); ptr != 0L; 
    ptr = (asc ? l->next() : l->prev()))
  {
    QString icon, event;
    QString body = ptr->body();

    if((direction == MSG_IN && (ptr->type() == MSG_SENT || ptr->type() == MSG_URL_SENT ||
                                ptr->type() == MSG_AUTH_GRANT || ptr->type() == MSG_AUTH_REFUSE)) ||
       (direction == MSG_OUT && (ptr->type() == MSG_RECV || ptr->type() == MSG_URL_RECV ||
                                 ptr->type() == MSG_AUTH_RECV || ptr->type() == MSG_ADDED ||
                                 ptr->type() == MSG_PAGER_RECV || ptr->type() == MSG_EXPRESS_RECV)))
    {
      n++;
      continue;
    }

    switch(ptr->type())
    {
      case MSG_SENT:
      case MSG_RECV:
        event = i18n("Message\n");
        pm = pmap_msg;
        break;
      case MSG_URL_SENT:
      case MSG_URL_RECV:
        event = i18n("URL\n");
        pm = pmap_url;
        body = (QString)ptr->url() + "\n" + body;
        break;
      case MSG_AUTH_RECV:
        event = i18n("Authorization Requested\n");
        pm = pmap_auth;
        body = (QString)ptr->nick() + ": " + (QString)ptr->first() + " " +
               (QString)ptr->last() + " <" + (QString)ptr->email() + ">\n" +
               body;
        break;
      case MSG_AUTH_GRANT:
        event = i18n("Authorization Granted\n");
        pm = pmap_auth;
        body = (QString)ptr->nick() + ": " + (QString)ptr->first() + " " +
               (QString)ptr->last() + " <" + (QString)ptr->email() + ">\n" +
               body;
        break;
      case MSG_AUTH_REFUSE:
        event = i18n("Authorization Refused\n");
        pm = pmap_auth;
        body = (QString)ptr->nick() + ": " + (QString)ptr->first() + " " +
               (QString)ptr->last() + " <" + (QString)ptr->email() + ">\n" +
               body;
        break;
      case MSG_ADDED:
        event = i18n("You've Been Added To Contact List\n");
        pm = pmap_exclam;
        body = (QString)ptr->nick() + ": " + (QString)ptr->first() + " " +
               (QString)ptr->last() + " <" + (QString)ptr->email() + ">";
        break;
      case MSG_PAGER_RECV:
        event = i18n("WebPager Message\n");
        pm = pmap_exclam;
        body = (QString)ptr->nick() + ": " + " <" + (QString)ptr->email() + ">";
        break;
      case MSG_EXPRESS_RECV:
        event = i18n("WebPager Message\n");
        pm = pmap_exclam;
        body = (QString)ptr->nick() + ": " + " <" + (QString)ptr->email() + ">";
        break;
      case MSG_CHAT_REQ_RECV:
        event = i18n("Chat Request Received\n");
        pm = pmap_chat;
        break;
      case MSG_FILE_REQ_RECV:
        event = i18n("File Request Received\n");
        pm = pmap_file;
        break;

      default:
        icon = "null.xpm";
    }

    TimeStr t = ptr->time();
    QString date, time;
    date.sprintf("%02i/%02i/%04i\n",t.day,t.month,t.year);
    time.sprintf("%02i:%02i\n",t.hour,t.minute);

    QString num;
    num.setNum(n);

    WMlineListBoxItem *litem = new WMlineListBoxItem(msglist, 1);
    litem->setText(4,(const char*)num);

    litem->setPixmap(0, pm);
    litem->setText(0, (const char*)event);
    litem->setText(1, (const char*)time);
    litem->setText(2, (const char*)date);
    litem->setText(3, (const char*)body);

    n++;
  }
  msglist->update();
}
