/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
// $Id: infoaddr.cpp,v 1.2 2000/06/25 18:11:50 denis Exp $
// $Log: infoaddr.cpp,v $
// Revision 1.2  2000/06/25 18:11:50  denis
// Frequent absence of country name was fixed!
//
// Revision 1.1  2000/05/24 16:58:18  denis
// Sources have been splitted to the whole bunch of appropriate
// directories. Every directory is end up with static library. All
// of them are linked together to the final kicq binary.
//
// Revision 1.9  2000/01/10 16:08:28  denis
// Headers reorganized for simplify dependencies and speedup compilation.
//
// Revision 1.8  1999/07/23 12:37:34  denis
// Layout management support implemented.
// Cleaned up.
//
// Revision 1.7  1999/07/16 15:48:09  denis
// Cleaned up.
//
// Revision 1.6  1999/07/16 13:10:57  denis
// Internalization support added.
//
// Revision 1.5  1999/07/12 15:23:23  cproch
// - changed icq_* function calls and callback function to support
//   icqlib with ICQLINK identifier.
// - look for FIXME in mainwindow.cpp
//
// Revision 1.4  1998/12/08 16:17:26  denis
// Cleaned up a little before releasing
//

#include <icq.h>
#include <qwidget.h>
#include <qlayout.h>
#include <kapp.h>

#include "infoaddr.h"

#include "infoaddr.moc"

AddrWidget::AddrWidget(QWidget *parent) : QWidget(parent)
{
  QGridLayout *topLayout = new QGridLayout(this, 7, 7, 0);
  topLayout->setRowStretch(0, 1);
  topLayout->setRowStretch(3, 1);
  topLayout->setRowStretch(6, 1);
  topLayout->setColStretch(1, 6);
  topLayout->setColStretch(3, 3);
  topLayout->setColStretch(5, 2);
  topLayout->addRowSpacing(0, 7);
  topLayout->addRowSpacing(3, 7);
  topLayout->addRowSpacing(6, 7);
  topLayout->addColSpacing(0, 7);
  topLayout->addColSpacing(2, 7);
  topLayout->addColSpacing(3, 70);
  topLayout->addColSpacing(4, 7);
  topLayout->addColSpacing(5, 40);
  topLayout->addColSpacing(6, 7);

  QLabel *citylabel = new QLabel(i18n("City"),this);
  citylabel->setMinimumSize(citylabel->sizeHint());
  topLayout->addWidget(citylabel, 1, 1);

  QLabel *statelabel = new QLabel(i18n("State"),this);
  statelabel->setMinimumSize(statelabel->sizeHint());
  topLayout->addWidget(statelabel, 1, 3);

  QLabel *ziplabel = new QLabel(i18n("Zip"),this);
  ziplabel->setMinimumSize(ziplabel->sizeHint());
  topLayout->addWidget(ziplabel, 1, 5);

  cityline = new QLineEdit(this);
  cityline->setFocusPolicy(QWidget::NoFocus);
  cityline->setMinimumSize(cityline->sizeHint());
  cityline->setMaximumHeight(cityline->sizeHint().height());
  topLayout->addWidget(cityline, 2, 1);

  stateline = new QLineEdit(this);
  stateline->setFocusPolicy(QWidget::NoFocus);
  stateline->setMinimumSize(70, stateline->sizeHint().height());
  stateline->setMaximumHeight(stateline->sizeHint().height());
  topLayout->addWidget(stateline, 2, 3);

  zipline = new QLineEdit(this);
  zipline->setFocusPolicy(QWidget::NoFocus);
  zipline->setMinimumSize(40, zipline->sizeHint().height());
  zipline->setMaximumHeight(zipline->sizeHint().height());
  topLayout->addWidget(zipline, 2, 5);

  QLabel *countrylabel = new QLabel(i18n("Country"),this);
  countrylabel->setMinimumSize(countrylabel->sizeHint());
  topLayout->addWidget(countrylabel, 4, 1);

  QLabel *timelabel = new QLabel(i18n("Local Time"),this);
  timelabel->setMinimumSize(timelabel->sizeHint());
  topLayout->addMultiCellWidget(timelabel, 4, 4, 3, 5);

  countryline = new QLineEdit(this);
  countryline->setFocusPolicy(QWidget::NoFocus);
  countryline->setMinimumSize(countryline->sizeHint());
  countryline->setMaximumHeight(countryline->sizeHint().height());
  topLayout->addWidget(countryline, 5, 1);

  timeline = new QLineEdit(this);
  timeline->setFocusPolicy(QWidget::NoFocus);
  timeline->setMinimumSize(110, timeline->sizeHint().height());
  timeline->setMaximumHeight(timeline->sizeHint().height());
  topLayout->addMultiCellWidget(timeline, 5, 5, 3, 5);

  topLayout->activate();
}

void AddrWidget::setInfo(const char *pcity, const char *pstate, const char *pzip,
                         unsigned short pcountry, unsigned char pcountry_stat,
                         const char *ptime)
{
  cityline->setText(strlen(pcity)==0?i18n("Not Entered"):pcity);
  stateline->setText(strlen(pstate)==0?i18n("Not Entered"):pstate);
  zipline->setText(pzip);
//  if(pcountry_stat == 0xFE) it could be anything: 0xFC, 0xFA, 0x9C...
  if(pcountry)
    countryline->setText(icq_GetCountryName(pcountry));
  else
    countryline->setText(i18n("Not Entered"));
  timeline->setText(ptime);
}
