/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
// $Id: infomain.cpp,v 1.3 2000/07/21 18:37:30 denis Exp $
// $Log: infomain.cpp,v $
// Revision 1.3  2000/07/21 18:37:30  denis
// Small typos were fixed.
//
// Revision 1.2  2000/05/29 02:14:39  bills
// minor spelling/grammar fixes
//
// Revision 1.1  2000/05/24 16:58:18  denis
// Sources have been splitted to the whole bunch of appropriate
// directories. Every directory is end up with static library. All
// of them are linked together to the final kicq binary.
//
// Revision 1.19  2000/04/09 18:32:27  denis
// Cleanups.
//
// Revision 1.18  2000/04/07 13:30:22  guruz
// Checks for valid IP in the info dialog
//
// Revision 1.17  2000/01/10 16:08:30  denis
// Headers reorganized for simplify dependencies and speedup compilation.
//
// Revision 1.16  1999/12/12 18:08:28  denis
// Existing kapp->topWidget()'s chnaged to MW macros.
//
// Revision 1.15  1999/12/02 14:24:13  denis
// Cleanups.
//
// Revision 1.14  1999/11/21 18:55:34  guruz
// Added "Send E-Mail" in User List Context Menu
//
// Revision 1.13  1999/10/15 02:52:12  kyle
// infomain.cpp - fixed compiler warnings
// infowindow.cpp - fixed gender
// kurllineedit.cpp - fixed disable behavior
//
// Revision 1.12  1999/10/06 19:11:03  denis
// Cleanups.
//
// Revision 1.11  1999/10/05 17:42:37  guruz
// Changed some visual things
//
// Revision 1.10  1999/10/03 19:07:37  guruz
// There is now the possibility to open URLs that are in the URL Line Edits !
//
// Revision 1.9  1999/10/02 12:35:42  guruz
// Changed some things in the KLineEditWithPopupMenu
// Made Homepage Info Line Edit Popup Menu
//
// Revision 1.8  1999/09/29 17:29:40  denis
// Markus Goetz <guruz@gmx.de> patches applied.
//
// Revision 1.7  1999/07/23 12:37:36  denis
// Layout management support implemented.
// Cleaned up.
//
// Revision 1.6  1999/07/16 15:48:11  denis
// Cleaned up.
//
// Revision 1.5  1999/07/16 13:11:20  denis
// Internalization support added.
//
// Revision 1.4  1998/12/08 16:17:47  denis
// Cleaned up a little before releasing
//

#include <stdlib.h> 
#include <qlayout.h>
#include <kapp.h>

#include "infomain.h"
#include "kwebbrowser.h"
#include "mainwindow.h"
#include "klineeditwithpopupmenu.h"

#include "infomain.moc"

MainWidget::MainWidget(QWidget *parent) : QWidget(parent)
{
  QGridLayout *topLayout = new QGridLayout(this, 10, 5, 0);
  topLayout->setRowStretch(0, 1);
  topLayout->setRowStretch(3, 1);
  topLayout->setRowStretch(6, 1);
  topLayout->setRowStretch(9, 1);
  topLayout->setColStretch(1, 1);
  topLayout->setColStretch(3, 1);
  topLayout->addRowSpacing(0, 7);
  topLayout->addRowSpacing(3, 7);
  topLayout->addRowSpacing(6, 7);
  topLayout->addRowSpacing(9, 7);
  topLayout->addColSpacing(0, 7);
  topLayout->addColSpacing(2, 7);
  topLayout->addColSpacing(4, 7);

  QLabel *uinlabel = new QLabel(i18n("ICQ #"),this);
  uinlabel->setMinimumSize(uinlabel->sizeHint());
  topLayout->addWidget(uinlabel, 1, 1);

  uinline = new KLineEditWithPopupMenu(this);
  uinline->setFocusPolicy(QWidget::NoFocus);
  uinline->setMinimumSize(uinline->sizeHint());
  uinline->setMaximumHeight(uinline->sizeHint().height());
  uinline->popup()->insertItem("Open Icq World Wide Pager", this, SLOT(slotPager()));
  topLayout->addWidget(uinline, 2, 1);

  QLabel *iplabel = new QLabel(i18n("Current/Last IP"),this);
  iplabel->setMinimumSize(iplabel->sizeHint());
  topLayout->addWidget(iplabel, 1, 3);

  ipline = new KLineEditWithPopupMenu(this);
  ipline->setFocusPolicy(QWidget::NoFocus);
  ipline->setMinimumSize(ipline->sizeHint());
  ipline->setMaximumHeight(ipline->sizeHint().height());
  ipline->popup()->insertItem("Ping IP", this, SLOT(slotPingIp()));
  ipline->popup()->insertSeparator();
  ipline->popup()->insertItem("Connect to FTP Server", this, SLOT(slotConnectToFTP()));
  ipline->popup()->insertItem("Connect to HTTP Server", this, SLOT(slotConnectToHTTP()));
  ipline->popup()->insertItem("Telnet", this, SLOT(slotTelnet()));
  topLayout->addWidget(ipline, 2, 3);

  QLabel *nicklabel = new QLabel(i18n("Nickname"),this);
  nicklabel->setMinimumSize(nicklabel->sizeHint());
  topLayout->addWidget(nicklabel, 4, 1);

  nickline = new QLineEdit(this);
  nickline->setFocusPolicy(QWidget::NoFocus);
  nickline->setMinimumSize(nickline->sizeHint());
  nickline->setMaximumHeight(nickline->sizeHint().height());
  topLayout->addWidget(nickline, 5, 1);

  QLabel *emaillabel = new QLabel(i18n("EMail"),this);
  emaillabel->setMinimumSize(emaillabel->sizeHint());
  topLayout->addWidget(emaillabel, 4, 3);

  emailline = new KLineEditWithPopupMenu(this);
  emailline->setFocusPolicy(QWidget::NoFocus);
  emailline->setMinimumSize(emailline->sizeHint());
  emailline->setMaximumHeight(emailline->sizeHint().height());
  emailline->popup()->insertItem("Send mail with kmail", this, SLOT(slotMail()));
  topLayout->addWidget(emailline, 5, 3);

  QLabel *firstlabel = new QLabel(i18n("First Name"),this);
  firstlabel->setMinimumSize(firstlabel->sizeHint());
  topLayout->addWidget(firstlabel, 7, 1);

  firstline = new QLineEdit(this);
  firstline->setFocusPolicy(QWidget::NoFocus);
  firstline->setMinimumSize(firstline->sizeHint());
  firstline->setMaximumHeight(firstline->sizeHint().height());
  topLayout->addWidget(firstline, 8, 1);

  QLabel *lastlabel = new QLabel(i18n("Last Name"),this);
  lastlabel->setMinimumSize(lastlabel->sizeHint());
  topLayout->addWidget(lastlabel, 7, 3);

  lastline = new QLineEdit(this);
  lastline->setFocusPolicy(QWidget::NoFocus);
  lastline->setMinimumSize(lastline->sizeHint());
  lastline->setMaximumHeight(lastline->sizeHint().height());
  topLayout->addWidget(lastline, 8, 3);

  topLayout->activate();
}

void MainWidget::setInfo(unsigned long puin, unsigned long pip, const char *pnick,
                         const char *pemail, const char *pfirst, const char *plast)
{
  QString stmp;
  stmp.setNum(puin);
  uinline->setText((const char*)stmp);
  if(pip!=0) 
  {
    stmp.sprintf("%i.%i.%i.%i", (int)((pip >> 24) & 0x000000ff),
                 (int)((pip >> 16) & 0x000000ff),
                 (int)((pip >> 8) & 0x000000ff), (int)(pip & 0x000000ff));
    IPvalid = true;
  }
  else 
  {
    stmp = i18n("N/A");
    IPvalid = false;
  }
  ipline->setText((const char*)stmp);
  nickline->setText(strlen(pnick)==0?i18n("Not Entered"):pnick);
  emailline->setText(strlen(pemail)==0?i18n("Not Entered"):pemail);
  firstline->setText(strlen(pfirst)==0?i18n("Not Entered"):pfirst);
  lastline->setText(strlen(plast)==0?i18n("Not Entered"):plast);
}

void MainWidget::slotPingIp()
{
  if(isIPValid())
    system(QString("xterm -e ping ") + ipline->text() + " &");
  else
    QMessageBox::warning(this, i18n("Error"), i18n("Cannot determine IP of this user!"));
}

void MainWidget::slotMail()
{
  MW->mailRequest(emailline->text()); 
}

void MainWidget::slotConnectToFTP()
{
  if(isIPValid())
    KWebBrowser::callWebBrowser(QString("ftp://") + ipline->text());
  else
    QMessageBox::warning(this, i18n("Error"), i18n("Cannot determine IP of this user!"));
}

void MainWidget::slotConnectToHTTP()
{
  if(isIPValid())
    KWebBrowser::callWebBrowser(QString("http://") + ipline->text());
  else
    QMessageBox::warning(this, i18n("Error"), i18n("Cannot determine IP of this user!"));
}

void MainWidget::slotTelnet()
{
  if(isIPValid())
    system(QString("xterm -e telnet ") + ipline->text() + " &");
  else
    QMessageBox::warning(this, i18n("Error"), i18n("Cannot determine IP of this user!"));
}

void MainWidget::slotPager()
{
  KWebBrowser::callWebBrowser(QString("http://wwp.mirabilis.com/") + uinline->text());
}

bool MainWidget::isIPValid()
{
  return IPvalid;
}
