/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
// $Id: infowindow.cpp,v 1.2 2000/06/17 17:38:57 denis Exp $
// $Log: infowindow.cpp,v $
// Revision 1.2  2000/06/17 17:38:57  denis
// Contact List code was rewritten from scratch! Although we are trying
// to not add new features in anticipation of the next stable release,
// but this rewrite solves many things, removes many hacks and stabilizes
// whole program.
// One of bug, needs to mention, is an absence of IP address in Info
// window. The root of problem was in Contact List, because in any time
// there were several ContactItems in memory for single user - they were
// "smeared" on the whole code. Now it's clean and strict - all Contact
// List operations can be done only by ContactList class.
//
// Revision 1.1  2000/05/24 16:58:18  denis
// Sources have been splitted to the whole bunch of appropriate
// directories. Every directory is end up with static library. All
// of them are linked together to the final kicq binary.
//
// Revision 1.15  2000/01/10 16:08:37  denis
// Headers reorganized for simplify dependencies and speedup compilation.
//
// Revision 1.14  1999/12/02 14:24:50  denis
// Random IcqWindow segfaults fixed.
// Cleanups.
//
// Revision 1.13  1999/10/15 02:52:13  kyle
// infomain.cpp - fixed compiler warnings
// infowindow.cpp - fixed gender
// kurllineedit.cpp - fixed disable behavior
//
// Revision 1.12  1999/10/14 13:00:14  denis
// Cleanups.
//
// Revision 1.11  1999/09/29 17:32:56  denis
// Cleanups.
//
// Revision 1.10  1999/07/23 12:37:43  denis
// Layout management support implemented.
// Cleaned up.
//
// Revision 1.9  1999/07/16 16:55:00  denis
// Switched "male" and "female" codes (they are different in v2 and v5)
//
// Revision 1.8  1999/07/16 15:48:17  denis
// Cleaned up.
//
// Revision 1.7  1999/07/16 13:13:07  denis
// Internalization support added.
// Smart loading of spinning face animation implemented.
//
// Revision 1.6  1999/07/12 15:23:24  cproch
// - changed icq_* function calls and callback function to support
//   icqlib with ICQLINK identifier.
// - look for FIXME in mainwindow.cpp
//
// Revision 1.5  1999/04/17 20:22:17  bills
// modified for new icqwindow base class
//
// Revision 1.4  1998/12/08 16:18:46  denis
// Cleaned up a little before releasing
//

#include <qmsgbox.h>
#include <qmovie.h>
#include <qlayout.h>
#include <kapp.h>

#include "infowindow.h"
#include "mainwindow.h"
#include "infomain.h"
#include "infoaddr.h"
#include "infomore.h"
#include "infoabout.h"
#include "infonotes.h"

#include "infowindow.moc"

InfoWindow::InfoWindow(unsigned long puin) : IcqWindow(), foreignInfo(uin)
{
  uin = puin;
  local = MW->contactList().isUINInContactList(uin);

  initLayout();
}

InfoWindow::InfoWindow(ContactItem fi) : IcqWindow(), foreignInfo(fi)
{
  uin = 0;
  local = FALSE;

  initLayout();
}

void InfoWindow::initLayout()
{
  setCaption(i18n("Info"));

  QBoxLayout *topLayout = new QVBoxLayout(this, 7);

  tabctl = new KTabCtl(this);
  mainwidget = new MainWidget(tabctl);
  addrwidget = new AddrWidget(tabctl);
  morewidget = new MoreWidget(tabctl);
  aboutwidget = new AboutWidget(tabctl);
  if(local)
    noteswidget = new NotesWidget(tabctl);
  tabctl->addTab(mainwidget,i18n("Main"));
  tabctl->addTab(addrwidget,i18n("Address"));
  tabctl->addTab(morewidget,i18n("More"));
  tabctl->addTab(aboutwidget,i18n("About"));
  if(local)
    tabctl->addTab(noteswidget,i18n("My Notes"));
  topLayout->addWidget(tabctl, 1);

  QBoxLayout *butLayout = new QHBoxLayout(7);
  topLayout->addLayout(butLayout);

  butLayout->addSpacing(32+4);
  butLayout->addStretch(10);

  donebtn = new QPushButton(i18n("Done"),this);
  donebtn->setMaximumHeight(donebtn->sizeHint().height());
  connect(donebtn, SIGNAL(clicked()), SLOT(doneCommand()));
  donebtn->setDefault(true);
  butLayout->addWidget(donebtn);

  butLayout->addSpacing(10);

  updatebtn = new QPushButton(i18n("Update"),this);
  updatebtn->setMaximumHeight(updatebtn->sizeHint().height());
  connect(updatebtn, SIGNAL(clicked()), SLOT(updateCommand()));
  butLayout->addWidget(updatebtn);

  int w = QMAX(donebtn->sizeHint().width(), updatebtn->sizeHint().width());
  donebtn->setMinimumSize(w, donebtn->sizeHint().height());
  updatebtn->setMinimumSize(w, updatebtn->sizeHint().height());

  butLayout->addStretch(10);

  updateanim = new QLabel(this);
  updateanim->setFixedSize(QSize(32, 32)+QSize(4, 4));
  updateanim->setFrameStyle(QFrame::WinPanel | QFrame::Sunken);
  updateanim->setAlignment(AlignCenter);
  QString filename = kapp->localkdedir()+"/share/apps/"+kapp->appName()+"/pics/search.gif";
  QFile testfile(filename);
  if(!testfile.exists())
    filename = kapp->kde_datadir()+"/"+kapp->appName()+"/pics/search.gif";
  updateanim->setMovie(QMovie((const char*)filename));
  updateanim->movie()->pause();
  updateanim->movie()->restart();
  butLayout->addWidget(updateanim);

  topLayout->activate();

  setInfo();

  resize(1,1);
}

void InfoWindow::doneCommand()
{
  if(local)
    if(MW->contactList().getNotesByUIN(uin) != noteswidget->text())
      MW->contactList().setNotesByUIN(uin, noteswidget->text());
  close();
}

void InfoWindow::updateCommand()
{
  if(MW->isLogged())
  {
    unsigned long u = uin?uin:foreignInfo.getUIN();
    updateanim->movie()->unpause();
    updatebtn->setEnabled(FALSE);
    icq_SendInfoReq(&MW->icqlink, u);
    icq_SendExtInfoReq(&MW->icqlink, u);
    upd_count=2;
  }
  else
    QMessageBox::warning(this, i18n("Error"), i18n("You are not connected to the icq network!"));
}

void InfoWindow::setMain(unsigned long puin, unsigned long pip, const char *pnick,
                         const char *pemail, const char *pfirst, const char *plast)
{
  mainwidget->setInfo(puin, pip, pnick, pemail, pfirst, plast);
}

void InfoWindow::setAddr(const char *pcity, const char *pstate, const char *pzip,
                         unsigned short pcountry, char pcountry_stat, const char *ptime)
{
  addrwidget->setInfo(pcity, pstate, pzip, pcountry, pcountry_stat, ptime);
}

void InfoWindow::setMore(const char *php, unsigned int page, const char *pphone, const char *pgender)
{
  morewidget->setInfo(php, page, pphone, pgender);
}

void InfoWindow::setAbout(const char *pabout)
{
  aboutwidget->setInfo(pabout);
}

void InfoWindow::setNotes(const char *pnotes)
{
  if(local)
    noteswidget->setInfo(pnotes);
}

void InfoWindow::setInfo()
{
  if(uin)
  {
    // UIN in our contact list
    QString stmp;
    const ContactItem *info = MW->contactList().getInfoByUIN(uin);

    if(!info)
    {
      printf("InfoWindow::setInfo(bogus UIN)\n");
      return;
    }

    setMain(uin, info->getLastIP(), info->getNick(), info->getEMail(), info->getFirstName(),
            info->getLastName());
    setAddr(info->getCity(), info->getState(), info->getZip(), info->getCountry(),
            info->getCountryStat(), i18n("N/A"));
    setMore(info->getHomePage(), info->getAge(), info->getPhone(),
            info->getGender()==1?i18n("Female"):info->getGender()==2?i18n("Male"):i18n("Not Entered"));
    setAbout(info->getAbout());
    if(local)
      setNotes(info->getNotes());
  }
  else
  {
    // UIN isn't in contact list
    setMain(foreignInfo.getUIN(), foreignInfo.getLastIP(), foreignInfo.getNick(),
            foreignInfo.getEMail(), foreignInfo.getFirstName(), foreignInfo.getLastName());
    setAddr(foreignInfo.getCity(), foreignInfo.getState(), foreignInfo.getZip(),
            foreignInfo.getCountry(), foreignInfo.getCountryStat(), i18n("N/A"));
    setMore(foreignInfo.getHomePage(), foreignInfo.getAge(), foreignInfo.getPhone(),
            foreignInfo.getGender()==1?i18n("Female"):foreignInfo.getGender()==2?i18n("Male"):
            i18n("Not Entered"));
    setAbout(foreignInfo.getAbout());
  }
}

unsigned long InfoWindow::getUIN()
{
  if(uin)
    return uin;
  else
    return foreignInfo.getUIN();
}

void InfoWindow::stopCommand()
{
  updateanim->movie()->pause();
  updatebtn->setEnabled(TRUE);
}

void InfoWindow::updateInfo(const char *nick, const char *first, const char *last,
                            const char *email, char auth)
{
  if(uin)
    MW->contactList().updateInfoByUIN(uin, nick, first, last, email, auth);
  else
    foreignInfo.updateInfo(nick, first, last, email, auth);
  setInfo();
  if(--upd_count==0)
    stopCommand();
  update();
}

void InfoWindow::updateExtInfo(const char *city, unsigned short country_code, char country_stat,
                               const char *state, unsigned short age, char gender,
                               const char *phone, const char *hp, const char *about)
{
  if(uin)
    MW->contactList().updateExtInfoByUIN(uin, city, country_code, country_stat, state, age,
                                         gender, phone, hp, about);
  else
    foreignInfo.updateExtInfo(city, country_code, country_stat, state, age, gender, phone,
                              hp, about);
  setInfo();
  if(--upd_count==0)
    stopCommand();
  update();
}
