/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * $Id: mainconfig.cpp,v 1.4 2000/07/11 18:33:29 denis Exp $
 * $Log: mainconfig.cpp,v $
 * Revision 1.4  2000/07/11 18:33:29  denis
 * saveConfigSound() now receives one more parameter "clear".
 * Cleanups.
 *
 * Revision 1.3  2000/07/10 09:24:24  pcadach
 * Recording of first-time registration information updated. See ChangeLog.
 *
 * Revision 1.2  2000/07/09 18:59:09  denis
 * Multi-owner feature was added.
 * Note: move your ~/.kde/share/config/kicqrc to
 * ~/.kde/share/apps/kicq/owners/<UIN>.conf and create
 * ~/.kde/share/config/kicqrc with section [Owner] and key UIN=<UIN>
 * where <UIN> is your current UIN (ICQ#) -- kicq-upgrade.sh will changed
 * later.
 *
 * Revision 1.1  2000/06/17 16:49:36  denis
 * Whole mainwindow.cpp was splitted for several smaller files:
 * mainconfig.cpp, specialops.cpp and, of course, mainwindow.cpp
 *
 */

#include <qdir.h>
#include <kwm.h>
#include <ksimpleconfig.h>

#include "mainwindow.h"
#include "icqsound.h"
#include "userlist.h"

bool MainWindow::checkConfig()
{
  KConfig *config = kapp->getConfig();
  config->setGroup("Owner");
  unsigned long uin = config->readUnsignedLongNumEntry("UIN");
  if(!uin)
    return FALSE;
  else
    return TRUE;
}

QString MainWindow::getOwnerConfigName(unsigned long u)
{
  QString ownerfname;
  ownerfname.sprintf("%s/owners/%lu.conf", (const char*)appdir, u);
  return ownerfname;
}

QString MainWindow::getOwnerConfigName()
{
  return getOwnerConfigName(getOwnerConfigUIN());
}

unsigned long MainWindow::getOwnerConfigUIN()
{
  KConfig *config = kapp->getConfig();
  config->setGroup("Owner");
  unsigned long u = config->readUnsignedLongNumEntry("UIN");
  return u;
}

void MainWindow::setOwnerConfigUIN(unsigned long u)
{
  KConfig *config = kapp->getConfig();
  config->setGroup("Owner");
  config->writeEntry("UIN", u);
  config->sync();
}

icqStatus MainWindow::loadConfig()
{
  KSimpleConfig config(getOwnerConfigName());
  config.setGroup("User");
  uin = getOwnerConfigUIN(); // config.readUnsignedLongNumEntry("UIN");
  passwd = getPassword();
  nick = config.readEntry("Nick","");
  config.setGroup("Server");
  server = config.readEntry("Host","");
  port = config.readUnsignedNumEntry("Port",4000);
  config.setGroup("Proxy");
  useproxy = config.readNumEntry("Use",FALSE);
  proxyhost = config.readEntry("Host", "");
  proxyport = config.readUnsignedNumEntry("Port",1080);
  proxyauth = config.readNumEntry("Auth",FALSE);
  proxyuser = config.readEntry("User","");
  proxypass = config.readEntry("Password","");
  config.setGroup("Sound");
  playWav = config.readNumEntry("playWav", FALSE);
  config.setGroup("Other");
  icq_Russian = config.readNumEntry("Koi2win", FALSE);
  closeRecv = config.readNumEntry("CloseRecv", TRUE);
  convertCRLF = config.readNumEntry("ConvertCRLF", TRUE);
  quote = config.readNumEntry("Quote", FALSE);
  reconnect = config.readNumEntry("Reconnect", TRUE);
  icqStatus st = (icqStatus)config.readNumEntry("Status", stOffline);
  setAllOnline(config.readBoolEntry("AllOnline", TRUE));
  config.setGroup("Log");
  icq_LogLevel = config.readNumEntry("Level",0);
  logfile = config.readEntry("File","");
  config.setGroup("Auto-Away");
  autoawaytimeout = config.readNumEntry("Away",30);
  autonatimeout = config.readNumEntry("NA",30);
  autoawayenabled = config.readBoolEntry("Enable", true);
  autoawaytona = config.readBoolEntry("AwayToNA", true);
  config.setGroup("Properties");
  QString prop = config.readEntry("Main");
  if(!prop.isEmpty())
    setGeometry(KWM::setProperties(winId(), prop));
  if(config.readNumEntry("Restored", TRUE))
    show();
  (*sound) << config;
  return st;
}

void MainWindow::saveConfigOwner(unsigned long u, const char *nick, const char *pass)
{
  KSimpleConfig config(getOwnerConfigName(u));
  config.setGroup("User");
  config.writeEntry("Nick",nick);
  setPassword(config, pass);
  config.sync();
}

void MainWindow::saveConfigOwner(const char *nick, const char *pass)
{
  saveConfigOwner(getOwnerConfigUIN(), nick, pass);
}

void MainWindow::saveConfigServer(unsigned long u, const char *host, unsigned short port)
{
  KSimpleConfig config(getOwnerConfigName(u));
  config.setGroup("Server");
  config.writeEntry("Host",host);
  config.writeEntry("Port",(unsigned int)port);
  config.sync();
}

void MainWindow::saveConfigServer(const char *host, unsigned short port)
{
  saveConfigServer(getOwnerConfigUIN(), host, port);
}

void MainWindow::saveConfigProxy(unsigned long u, int pruse, const char *prhost,
                                 unsigned short prport, int prauth, const char *pruser,
                                 const char *prpass)
{
  KSimpleConfig config(getOwnerConfigName(u));
  config.setGroup("Proxy");
  config.writeEntry("Use",pruse);
  config.writeEntry("Host",prhost);
  config.writeEntry("Port",(int)prport);
  config.writeEntry("Auth",prauth);
  config.writeEntry("User",pruser);
  config.writeEntry("Password",prpass);
  config.sync();
}

void MainWindow::saveConfigProxy(int pruse, const char *prhost, unsigned short prport,
                                 int prauth, const char *pruser, const char *prpass)
{
  saveConfigProxy(getOwnerConfigUIN(), pruse, prhost, prport, prauth, pruser, prpass);
}

void MainWindow::saveConfigLog(unsigned long u, int loglvl, const char *logfile)
{
  KSimpleConfig config(getOwnerConfigName(u));
  config.setGroup("Log");
  icq_LogLevel = loglvl;
  config.writeEntry("Level",loglvl);
  config.writeEntry("File",logfile);
  config.sync();
}

void MainWindow::saveConfigLog(int loglvl, const char *logfile)
{
  saveConfigLog(getOwnerConfigUIN(), loglvl, logfile);
}

void MainWindow::saveConfigSound(unsigned long u, int wav, bool clear)
{
  KSimpleConfig config(getOwnerConfigName(u));
  config.setGroup("Sound");
  config.writeEntry("playWav", wav);
  if(clear)
    sound->clear();
  (*sound) >> config;
  config.sync();
}

void MainWindow::saveConfigSound(int wav, bool clear)
{
  saveConfigSound(getOwnerConfigUIN(), wav, clear);
}

void MainWindow::saveConfigAway(unsigned long u, int awaytimeout, int natimeout, bool awayenabled,
                                bool awaytona)
{
  KSimpleConfig config(getOwnerConfigName(u));
  config.setGroup("Auto-Away");
  config.writeEntry("Away", awaytimeout);
  autoawaytimeout = awaytimeout;
  config.writeEntry("NA", natimeout);
  autonatimeout = natimeout;
  config.writeEntry("Enable", awayenabled);
  autoawayenabled = awayenabled;
  config.writeEntry("AwayToNA", awaytona);
  autoawaytona = awaytona;
  config.sync();
}

void MainWindow::saveConfigAway(int awaytimeout, int natimeout, bool awayenabled, bool awaytona)
{
  saveConfigAway(getOwnerConfigUIN(), awaytimeout, natimeout, awayenabled, awaytona);
}

void MainWindow::saveConfigOther(unsigned long u, int rus, int closereply, int crlf, int quoteorig, int recon)
{
  KSimpleConfig config(getOwnerConfigName(u));
  config.setGroup("Other");
  icq_Russian = rus;
  config.writeEntry("Koi2win",icq_Russian);
  config.writeEntry("CloseRecv",closereply);
  config.writeEntry("ConvertCRLF",crlf);
  config.writeEntry("Quote",quoteorig);
  config.writeEntry("Reconnect",recon);
  config.writeEntry("Status",(int)cur_status);
  config.sync();
}

void MainWindow::saveConfigOther(int rus, int closereply, int crlf, int quoteorig, int recon)
{
  saveConfigOther(getOwnerConfigUIN(), rus, closereply, crlf, quoteorig, recon);
}

void MainWindow::saveProperties(unsigned long u)
{
  KSimpleConfig config(getOwnerConfigName(u));
  config.setGroup("Properties");
  config.writeEntry("Main", KWM::getProperties(winId()));
  config.writeEntry("Restored", (int)isVisible());
  config.sync();
}

void MainWindow::saveProperties()
{
  saveProperties(getOwnerConfigUIN());
}

void MainWindow::saveStatus(unsigned long u)
{
  KSimpleConfig config(getOwnerConfigName(u));
  config.setGroup("Other");
  config.writeEntry("Status", (unsigned int)cur_status);
  config.writeEntry("AllOnline",listwidget->getAllOnline());
  config.sync();
}

void MainWindow::saveStatus()
{
  saveStatus(getOwnerConfigUIN());
}

QString MainWindow::getPassword()
{
  KSimpleConfig config(getOwnerConfigName());
  config.setGroup("User");

  QString oldpasswd = config.readEntry("Password", "");

  if(!oldpasswd.isEmpty())
  {
    config.writeEntry("EPassword", encryptStr(oldpasswd)); 
    config.writeEntry("Password", ""); 
  }

  QString passwd = decryptStr(config.readEntry("EPassword", ""));

  return passwd;
}

void MainWindow::setPassword(KSimpleConfig &config, QString password)
{
  config.setGroup("User");

  config.writeEntry("EPassword", encryptStr(password));
  config.writeEntry("Password", "");
}

void MainWindow::saveSessionConfig()
{
  saveProperties();
  saveStatus();
}

// I stole this thing from kmail ... password == encryptStr(enryptStr(password)) ! (guruz)
// I hope it works.
QString MainWindow::encryptStr(const QString aStr)
{
  if(aStr.isEmpty())
    return "";

  unsigned int i, val;
  unsigned int len = aStr.length();
  QString result(len+1);

  for(i=0; i<len; i++)
  {
    val = aStr[i] - ' ';
    val = (255-' ') - val;
    result[i] = (char)(val + ' ');
  }
  result[i] = '\0';

  return result;
}

QString MainWindow::decryptStr(const QString aStr)
{
  return encryptStr(aStr);
}
