/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
#ifndef _MAINWINDOW_
#define _MAINWINDOW_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <qlistbox.h>
#include <qwidget.h>
#include <qpushbt.h>
#include <qsocknot.h>
#include <qmsgbox.h>
#include <qstring.h>
#include <qtoolbutton.h>
#include <qpopupmenu.h>

#include <ktopwidget.h>
#include <kmenubar.h>
#include <ktoolbar.h>

#include <icq.h>

#include "alertbox.h"
#include "connectbutton.h"
#include "contactlist.h"
#include "icqwindow.h"

#define MENU_SHUTDOWN     1
#define MENU_PREFERENCES  2
#define MENU_SECURITY     3

#define	MENU_COPYUINTOCLIPBOARD 10
#define	MENU_COPYIPTOCLIPBOARD  11

#define MW ((MainWindow*)(kapp->topWidget()))

class UserList;
class SystemBar;
class ConnectButton;
class Preferences;
class FindWindow;
class DockWidget;
class IcqSound;
class RegWindow;
class IcqAnimButton;

class MainWindow : public KTopLevelWidget {
  Q_OBJECT
public:
  ICQLINK icqlink;
  int playWav;
  IcqSound *sound;
  QPopupMenu *statemenu;

  MainWindow();
  ~MainWindow();
  int isLogged() { return logged; }
  const char *logFile() { return (const char*)logfile; }
  static void logEvent(ICQLINK*, time_t, unsigned char, const char*);
  void delAlertWin(AlertMsgBox*);
  void infoWindow(ContactItem);
  void infoWindow(unsigned long);
  void updateHistoryWindows(unsigned long) ;
  void setIcon(QPixmap);
  void togglePixmap(int);
  int isUnreadMsgs();
  QString contactsDir() { return contdir; }
  QPopupMenu *stateMenu() { return statemenu; }
  UserList *listWidget() { return listwidget; }
  FindWindow *findWindow() { return findwin; }
  RegWindow *regWindow() { return regwin; }
  ContactList &contactList() { return contactlist; }
  int isConvertCRLF() { return convertCRLF; }
  int isQuote() { return quote; }
  int isCloseRecv() { return closeRecv; }
  void updateContact();
  void addIcqWindow(IcqWindow*);
  void delIcqWindow(IcqWindow*);
  void saveConfigOwner(const char*, const char*);
  void saveConfigOwner(unsigned long, const char*, const char*);
  void saveConfigServer(const char*, unsigned short);
  void saveConfigServer(unsigned long, const char*, unsigned short);
  void saveConfigProxy(int, const char*, unsigned short, int, const char*, const char*);
  void saveConfigProxy(unsigned long, int, const char*, unsigned short, int, const char*, const char*);
  void saveConfigLog(int, const char*);
  void saveConfigLog(unsigned long, int, const char*);
  void saveConfigSound(int, bool clear = FALSE);
  void saveConfigSound(unsigned long, int, bool clear = FALSE);
  void saveConfigOther(int, int, int, int, int);
  void saveConfigOther(unsigned long, int, int, int, int, int);
  void saveConfigAway(int, int, bool, bool);
  void saveConfigAway(unsigned long, int, int, bool, bool);
  void updateBlinking();
  void messageRequest(unsigned long, const char*);
  void updateVisibleList();
  void prepareNewUIN(unsigned long);
public slots:
  void setStatus(int);
  void stateCommand();
  void recvRequest(int);
  void onlineAlert();
  void visibleUser();
  void messageRequest();
  void urlRequest();
  void authRequest();
  void chatRequest();
  void fileRequest();
  void historyRequest();
  void infoRequest();
  void renameRequest();
  void delUserRequest();
  void readFirstRequest();
  void recvRequest();
  void mailRequest(QString address);
  void mailRequest();
  void specialOpRequest(int);
  void webBrowserRequest(QString url);
  void webBrowserRequest(int userNumber, QString protocol);
  void tetrinetRequest(QString ip);
  void tetrinetRequest(int userNumber);
  void telnetRequest(QString ip);
  void telnetRequest(int userNumber);
  void pingRequest(QString ip);
  void pingRequest(int userNumber);
  void tracerouteRequest(QString ip);
  void tracerouteRequest(int userNumber);
  QString intIP2StringIP(unsigned long);
signals:
protected:
  bool autoawayenabled; // is auto-away enabled?
  bool autoawaytona;    // change to NA if user set away?
  bool autoawayset;     // is auto-away active?
  int autoawaytimer;    // timer to cause timerEvent which calls MainWindow::autoAway()
  int autoawaytimeout;  // how long to wait before changing status to away
  int autonatimeout;    // how long to wait before changing status from away to NA
  int logged;
  int keepalivetimer;
  int tcptimer;
  unsigned long uin;
  QString passwd;
  QString nick;
  QString server;
  unsigned short port;
  int useproxy;
  QString proxyhost;
  unsigned short proxyport;
  int proxyauth;
  QString proxyuser, proxypass;
  QString logfile;
  int bell;
  int userPlay;
  int closeRecv;
  int convertCRLF;
  int quote;
  int reconnect;
  icqStatus log_status;
  icqStatus cur_status;
  QWidget *panelwidget;
  UserList *listwidget;
  SystemBar *syswidget;
  DockWidget *dockwidget;
//  QPushButton *menubutton;
  IcqAnimButton *menubutton;
  ConnectButton *statebutton;
  QPushButton *addbutton;
  QToolButton *allbutton;
  QPopupMenu *icqmenu;
  Preferences *prefwin;
  FindWindow *findwin;
  RegWindow *regwin;
  QString appdir, contdir;
  QPixmap blinkPixmap[2];
  QPixmap stateIcon;
  int blinkFlag;
  int blinkPixNum;
  QList<IcqWindow> icqWinList;
  QList<AlertMsgBox> alertWinList;
  int icqSocket, icqProxySocket;
  QSocketNotifier *socketnotifier, *proxynotifier;
  QTimer *libTimer;
  ContactList contactlist;
  QPopupMenu *ownersmenu;

  void resizeEvent(QResizeEvent*);
  void closeEvent(QCloseEvent*);
  void setupGUICallbacks();
  virtual void timerEvent(QTimerEvent*);
  void login(unsigned long);
  void disconnect();
  bool checkConfig();
  void setTitle();
  QStrList getUINDirList();
  void stopAll();
  void doNotify(unsigned long, int, unsigned int, void*);
  void updateRecvWindows();
  void setKeyboardLED(bool on);
  QString encryptStr(QString aStr);
  QString decryptStr(const QString aStr);
  QString getPassword();
  void setPassword(KSimpleConfig&, QString password);
  icqStatus loadConfig();
  DockWidget *dockWidget() { return dockwidget; }
  void saveProperties();
  void saveStatus();
  void saveProperties(unsigned long);
  void saveStatus(unsigned long);
  void saveUser(unsigned long, QString, QString, QString, QString, unsigned char);
  void initContact();
  void addAlertWin(unsigned long);
  void infoReplyHandler(unsigned long, const char*, const char*, const char*,
       const char*, char);
  void infoExtReplyHandler(unsigned long, const char*, unsigned short, char,
       const char*, unsigned short, char, const char*, const char*, const char*);
  icqStatus status();
  void setBlinking(int);
  unsigned long getFirstUnreadMsgUIN();
  void setBlinkPixmap(const char*);
  void setToolTip();
  void autoAway();
  void messageRequest(int);
  void chatRequest(int);
  void fileRequest(int);
  void urlRequest(int);
  void authRequest(int);
  void historyRequest(int) ;
  void setLogStatus();
  void setAllOnline(bool);
  bool getAllOnline();
  void setOnlineStatus(icqStatus, unsigned long);
  QString getOwnerConfigName();
  QString getOwnerConfigName(unsigned long);
  unsigned long getOwnerConfigUIN();
  void setOwnerConfigUIN(unsigned long);
  QStrList getConfigDirList();
  void fillOwnersPopup();
  void startOwner();

  // Callbacks
  static void loggedIn(ICQLINK*);
  static void disconnected(ICQLINK*);
  static void userOnline(ICQLINK*, unsigned long, unsigned long,
              unsigned long, unsigned short, unsigned long, unsigned char);
  static void userStatusUpdate(ICQLINK*, unsigned long, unsigned long);
  static void userOffline(ICQLINK*, unsigned long);
  static void webPager(ICQLINK*, unsigned char, unsigned char, unsigned char,
              unsigned char, unsigned short, const char*, const char*,
              const char*);
  static void mailExpress(ICQLINK*, unsigned char, unsigned char,
              unsigned char, unsigned char, unsigned short, const char*,
              const char*, const char*);
  static void message(ICQLINK*, unsigned long, unsigned char,
              unsigned char, unsigned char, unsigned char, unsigned short,
              const char*);
  static void url(ICQLINK*, unsigned long, unsigned char, unsigned char,
              unsigned char, unsigned char, unsigned short, const char*,
              const char*);
  static void chatReq(ICQLINK*, unsigned long, unsigned char,
              unsigned char, unsigned char, unsigned char, unsigned short,
              const char*,unsigned long);
  static void fileReq(ICQLINK *link, unsigned long, unsigned char, unsigned char,
              unsigned char, unsigned char, unsigned short, const char*,
              const char *, unsigned long, unsigned long);
  static void AuthRequest(ICQLINK*, unsigned long, unsigned char,
              unsigned char, unsigned char, unsigned char, unsigned short,
              const char*, const char*, const char*, const char*, const char*);
  static void infoReply(ICQLINK*, unsigned long, const char*, const char*,
              const char*, const char*, char);
  static void infoExtReply(ICQLINK*, unsigned long, const char*,
              unsigned short, char, const char*, unsigned short, char,
              const char*, const char*, const char*);
  static void requestNotify(ICQLINK*, unsigned long, int, unsigned int,
              void*);
  static void authreq(ICQLINK*, unsigned long, unsigned char,
              unsigned char, unsigned char, unsigned char, unsigned short,
              const char*, const char*, const char*, const char*, const char*);
  static void added(ICQLINK*, unsigned long, unsigned char, unsigned char,
              unsigned char, unsigned char, unsigned short, const char*,
              const char*,const char*, const char*);
  static void setTimeout(ICQLINK*, long);
protected slots:
  void quitRequest();
  void addCommand();
  void allCommand();
  void menuCommand();
  void icqCommand(int);
  void saveSessionConfig();
  void dataReceived(int);
  void registerCommand();
  void debug1Request();
  void debug2Request();
  void debug3Request();
  void debug4Request();
  void debug5Request();
  void debug6Request();
  void debug7Request();
  void libTimerDone();
  void changeOwner(int);
private:
private slots:
};
#endif /* _MAINWINDOW_ */
