/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
// $Id: fwdlist.cpp,v 1.2 2000/06/17 17:08:05 denis Exp $
// $Log: fwdlist.cpp,v $
// Revision 1.2  2000/06/17 17:08:05  denis
// Cleanups.
//
// Revision 1.1  2000/05/24 16:58:31  denis
// Sources have been splitted to the whole bunch of appropriate
// directories. Every directory is end up with static library. All
// of them are linked together to the final kicq binary.
//
// Revision 1.9  2000/03/30 15:08:38  denis
// ANSI fixes.
//
// Revision 1.8  2000/01/10 16:08:18  denis
// Headers reorganized for simplify dependencies and speedup compilation.
//
// Revision 1.7  1999/12/12 18:08:19  denis
// Existing kapp->topWidget()'s chnaged to MW macros.
//
// Revision 1.6  1999/09/29 17:30:38  denis
// Cleanups.
//
// Revision 1.5  1999/07/16 15:47:57  denis
// Cleaned up.
//

#include <kapp.h>
#include <kiconloader.h>

#include "fwdlist.h"
#include "mainwindow.h"
#include "userlist.h"

#include "fwdlist.moc"

FwdItem::FwdItem(FwdList *p,unsigned long u,bool on,const char *t) : 
  QListBoxItem()
{
  parent = p;
  uin = u;
  online = on;
  nick = t;
  selected = 0;
}

void FwdItem::paint(QPainter *p)
{
  QFontMetrics fm = p->fontMetrics();
  if(uin == 0)
  {
    int w = parent->width()-5;
//    int h = parent->height()-5;
    p->setPen(QColor(white));
    p->drawLine(0,fm.lineSpacing()/2,w/2-fm.width(nick)/2-6,fm.lineSpacing()/2);
    p->drawLine(w/2+fm.width(nick)/2+5,fm.lineSpacing()/2,w,fm.lineSpacing()/2);
    p->setPen(QColor(black));
    p->drawLine(0,fm.lineSpacing()/2-1,w/2-fm.width(nick)/2-6,fm.lineSpacing()/2-1);
    p->drawLine(w/2+fm.width(nick)/2+5,fm.lineSpacing()/2-1,w,fm.lineSpacing()/2-1);
    p->drawText(0,0,w,fm.lineSpacing(),AlignCenter,nick);
  }
  else
  {
    p->drawText(20,fm.ascent() + fm.leading()/2+2,nick);

    if(isOn())
      p->setPen(QColor(black));
    else
      p->setPen(QColor(white));

    p->drawLine(3,7,8,2);
    p->drawLine(4,7,8,3);
    p->drawLine(9,2,14,7);
    p->drawLine(9,3,14,7);

    if(isOn())
      p->setPen(QColor(white));
    else
      p->setPen(QColor(black));

    p->drawLine(3,8,8,13);
    p->drawLine(4,8,8,12);
    p->drawLine(9,13,14,8);
    p->drawLine(9,12,13,8);
  }
} 

int FwdItem::height(const QListBox *lb) const
{
  return QMAX(18,lb->fontMetrics().lineSpacing() + 3);
}

int FwdItem::width(const QListBox *lb) const
{
  return 18 + lb->fontMetrics().width(nick) + 6;
}


FwdList::FwdList(QWidget *parent,const char *name,unsigned long uin) : QListBox(parent,name)
{
  QPalette pal = palette();
  QColorGroup colgrp = pal.normal();
  pal.setNormal(QColorGroup(colgrp.foreground(),colgrp.background(),colgrp.light(),
    colgrp.dark(),colgrp.mid(),colgrp.text(),colgrp.background()));

  colgrp = pal.active();  
  pal.setActive(QColorGroup(colgrp.foreground(),colgrp.background(),colgrp.light(),
    colgrp.dark(),colgrp.mid(),colgrp.text(),colgrp.background()));

  setPalette(pal);

  defUIN = uin;
  changeList();
}

FwdList::~FwdList()
{
}

void FwdList::mousePressEvent(QMouseEvent *me)
{
  QListBox::mousePressEvent(me);
  if(me->button() == LeftButton)
    toggle(currentItem());
}

void FwdList::toggle(int n)
{
  FwdItem *fitem = (FwdItem *)item(n);
  if(fitem->isOn())
    fitem->setOn(FALSE);
  else
    fitem->setOn(TRUE);
  
  int yPos;
  itemYPos(n,&yPos);
  updateItem(n,FALSE);
}

void FwdList::paintCell(QPainter *p, int row, int col)
{
  FwdItem *fitem = (FwdItem *)item(row);
  if(!fitem) return;

  QColorGroup g = colorGroup();
  if(isSelected(row) && fitem->UIN() != 0)
  {
    QColor fc(blue);
    p->fillRect(0,0,cellWidth(col),cellHeight(row), fc);
    p->setPen(QColor(white));
    p->setBackgroundColor(fc);
  }
  else
  {
    p->setBackgroundColor(g.base());
    if(fitem->isOnline())
      p->setPen(QColor(blue));
    else
      p->setPen(QColor(red));
  }

  fitem->paint(p);
  p->setBackgroundColor(g.base());
  p->setPen(g.text());
}

void FwdList::changeList()
{
  setAutoUpdate(FALSE);
  QListBox::clear();

  if(MW->contactList().onlineListCount())
  {
    FwdItem *sep = new FwdItem(this, 0, 0, i18n("Online"));
    insertItem(sep);
  }

  QListIterator<ContactItem> it = MW->contactList().getOnlineListIterator();
  for(it.toFirst(); it.current(); ++it)
  {
    QString t = it.current()->getName();
    FwdItem *fitem = new FwdItem(this, it.current()->getUIN(), TRUE, t);
    if(fitem->UIN() == defUIN)
      fitem->setOn(TRUE);

    insertItem(fitem);
  }

  if(MW->contactList().offlineListCount())
  {
    FwdItem *sep = new FwdItem(this, 0, 0, i18n("Offline"));
    insertItem(sep);
  }

  QListIterator<ContactItem> it1 = MW->contactList().getOfflineListIterator();
  for(it1.toFirst(); it1.current(); ++it1)
  {
    QString t = it1.current()->getName();
    FwdItem *fitem = new FwdItem(this, it1.current()->getUIN(), FALSE, t);
    if(fitem->UIN() == defUIN)
      fitem->setOn(TRUE);

    insertItem(fitem);
  }

  setAutoUpdate(TRUE);
  update();
}

QList<UserItem> *FwdList::getSelectedListPtr()
{
  FwdItem *fitem;
  UserItem *selected;

  selectedList.clear();

  for(unsigned int i = 0; i < count(); i++)
  {
    fitem = (FwdItem*)item(i);
    if(fitem->isOn())
    {
      selected = new UserItem(fitem->UIN());
      selectedList.insert(0,selected);
    }
  }

  return &selectedList;
}
