/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
// $Id: recvwindow.cpp,v 1.6 2000/07/30 18:53:35 denis Exp $
// $Log: recvwindow.cpp,v $
// Revision 1.6  2000/07/30 18:53:35  denis
// Warnings were fixed.
//
// Revision 1.5  2000/07/28 18:41:17  denis
// Simplified and fixed problems with off-screen positioning.
//
// Revision 1.4  2000/07/14 15:30:34  bills
// changed to use standard Qt layouts.
//
// Revision 1.3  2000/06/17 17:39:08  denis
// Contact List code was rewritten from scratch! Although we are trying
// to not add new features in anticipation of the next stable release,
// but this rewrite solves many things, removes many hacks and stabilizes
// whole program.
// One of bug, needs to mention, is an absence of IP address in Info
// window. The root of problem was in Contact List, because in any time
// there were several ContactItems in memory for single user - they were
// "smeared" on the whole code. Now it's clean and strict - all Contact
// List operations can be done only by ContactList class.
//
// Revision 1.2  2000/06/15 01:55:56  bills
// fixed to use renamed chat functions
//
// Revision 1.1  2000/05/24 16:58:31  denis
// Sources have been splitted to the whole bunch of appropriate
// directories. Every directory is end up with static library. All
// of them are linked together to the final kicq binary.
//
// Revision 1.40  2000/05/10 19:22:12  denis
// Empty line removed... ;)
//
// Revision 1.39  2000/05/05 03:39:35  bills
// added code and member variables to RecvFile sub-widget to display file and
// size from file request message.
//
// Revision 1.38  2000/05/04 16:12:49  bills
// added code to disable action buttons for expired messages, disabled
// non-working decline buttons, made chat and file xfer windows automatically
// close when accepted.
//
// Revision 1.37  2000/03/30 14:48:19  denis
// Indentation and ANSI fixes.
//
// Revision 1.36  2000/02/07 03:00:52  bills
// fixed bug in accepting chat requests
//
// Revision 1.35  2000/02/03 15:53:12  bills
// fixed a number of bugs with new recvwindow code, including 1) window
// wasn't closing when replying when close-on-reply option was set, 2)
// crashes when certain events were viewed, 3) a weird flicker bug when
// hitting Read Next, and 4) the authorization message boxes weren't
// displaying properly (when you click authorize or decline).
//
// Revision 1.34  2000/01/20 20:48:14  tim
// show authorize widget
//
// Revision 1.33  2000/01/16 21:37:48  bills
// implemented save/save as action buttons, changed code to use new file
// session interface
//
// Revision 1.32  2000/01/10 16:09:17  denis
// Headers reorganized for simplify dependencies and speedup compilation.
//
// Revision 1.31  2000/01/07 04:17:29  bills
// _major_ rewrite.  split each event type out into an individual widget, and
// implemented geometry management using classes from qtlayouts
//
// Revision 1.30  1999/12/28 01:51:22  bills
// added new code to handle file transfers
//
// Revision 1.29  1999/12/12 18:10:50  denis
// Authorization handling fixed.
//
// Revision 1.28  1999/12/02 14:32:50  denis
// Layout management implemented.
// Preferences dialog style switched from "Tabbed" dialog to
// "Netscape Preferences"-like style.
//
// Revision 1.27  1999/10/03 12:13:41  guruz
// Various Changes
//
// Revision 1.26  1999/09/29 17:35:07  denis
// Cleanups.
//
// Revision 1.25  1999/07/23 12:35:59  denis
// Cleaned up.
//
// Revision 1.24  1999/07/19 13:15:02  tim
// - added support for MSG_ADDED_RECV ("you were added to contact list")
//
// Revision 1.23  1999/07/16 15:48:52  denis
// Cleaned up.
//
// Revision 1.22  1999/07/16 13:14:02  denis
// Internalization support added.
//
// Revision 1.21  1999/07/12 15:23:29  cproch
// - changed icq_* function calls and callback function to support
//   icqlib with ICQLINK identifier.
// - look for FIXME in mainwindow.cpp
//
// Revision 1.20  1999/04/17 20:22:21  bills
// modified for new icqwindow base class
//
// Revision 1.19  1999/04/04 22:04:03  bills
// added support for MSG_CHAT_REQ_RECV and chat window handling
//
// Revision 1.18  1999/03/24 11:53:23  denis
// *** empty log message ***
//
// Revision 1.17  1999/03/16 01:09:23  tim
// Cleanup authorize code.
//
// Revision 1.16  1999/03/05 13:04:41  denis
// Applied patch to handle authorization request messages by Bogdan Surdu
// <tim@ima-info.ro>
//
// Revision 1.15  1998/12/11 18:23:26  whale
// Added support for MSG_AUTH_RECV (incoming auth requests)
//
// Revision 1.14  1998/12/08 16:29:28  denis
// Cleaned up a little before releasing
//

#include <icq.h>
#include <qpalette.h>
#include <qobjectlist.h>
#include <qlayout.h>
#include <kapp.h>
#include <kiconloader.h>
#include <kfiledialog.h>

#include "recvwindow.h"
#include "mainwindow.h"
#include "chatwindow.h"
#include "filewindow.h"
#include "userlist.h"
#include "kurllineedit.h"

#include "recvwindow.moc"

RecvDetail::RecvDetail(icqMessage *msg, QWidget *parent) 
  : QGroupBox(msg->typeString(), parent)
{
  message = msg;
  layout = new QVBoxLayout(this, 9, 5);
  layout->addSpacing(7);
}

void RecvDetail::addActionFrame()
{
  actionFrame = new QGroupBox(i18n("Response Action:"), this);
  QVBoxLayout *xlayout = new QVBoxLayout(actionFrame, 9, 5);
  xlayout->addSpacing(6);

  actionLayout = new QHBoxLayout(5);
  xlayout->addLayout(actionLayout);
  xlayout->activate();
}

void RecvDetail::addActionButton(QString text, QString slot, bool enable)
{
  QPushButton *btn = new QPushButton(text, actionFrame);
  btn->setMinimumSize(btn->sizeHint());
  btn->setEnabled(enable);
  connect(btn, SIGNAL(clicked()), slot);
  actionLayout->addWidget(btn);
}

void RecvDetail::addRequest()
{
  MW->contactList().addNewToContactListByUIN(message->uin());
}

void RecvDetail::replyRequest()
{
  MW->messageRequest(message->uin(), message->body());

  if(MW->isCloseRecv())
    topLevelWidget()->close();
}

void RecvDetail::activateLayout()
{
  layout->activate();
}

// Message event (including EmailExpress & WebPager)
RecvMessage::RecvMessage(icqMessage *msg, QWidget *parent)
  : RecvDetail(msg, parent)
{
  QMultiLineEdit *msgbody = new QMultiLineEdit(this);
  msgbody->setText(message->body());
  msgbody->setFocusPolicy(QWidget::NoFocus);
  layout->addWidget(msgbody, 1);

  addActionFrame();
  actionLayout->addStretch(1);
  addActionButton(i18n("Add To Contact List"), SLOT(addRequest()),
                  !MW->contactList().isUINInContactList(msg->uin()));
  addActionButton(i18n("Reply"), SLOT(replyRequest()), TRUE);
  actionLayout->addStretch(1);
  layout->addWidget(actionFrame);
}

// URL event
RecvURL::RecvURL(icqMessage *msg, QWidget *parent)
  : RecvDetail(msg, parent)
{
  QLabel *desclabel = new QLabel(i18n("URL Description:"), this);
  QMultiLineEdit *urldesc = new QMultiLineEdit(this);
  urldesc->setFocusPolicy(QWidget::NoFocus);
  urldesc->setText(message->body());
  desclabel->setBuddy(urldesc);
  desclabel->setMinimumSize(desclabel->sizeHint());

  QLabel *urllabel = new QLabel(i18n("URL:"), this);
  KURLLineEdit *urledit = new KURLLineEdit(this);
  urledit->setText(message->url());
  urledit->setFocusPolicy(QWidget::NoFocus);
  urledit->setMinimumSize(urledit->sizeHint());
  urllabel->setBuddy(urledit);
  urllabel->setMinimumSize(urllabel->sizeHint());

  layout->addWidget(desclabel);
  layout->addWidget(urldesc, 1);
  layout->addWidget(urllabel);
  layout->addWidget(urledit);

  addActionFrame();
  actionLayout->addStretch(1);
  addActionButton(i18n("Add To Contact List"), SLOT(addRequest()),
                  !MW->contactList().isUINInContactList(msg->uin()));
  addActionButton(i18n("Reply"), SLOT(replyRequest()), TRUE);
  actionLayout->addStretch(1);
  layout->addWidget(actionFrame);
}

// Authorize events
RecvAuth::RecvAuth(icqMessage *msg, QWidget *parent)
  : RecvDetail(msg, parent)
{
  QMultiLineEdit *msgbody = new QMultiLineEdit(this);
  msgbody->setText(message->body());
  layout->addWidget(msgbody, 1);

  addActionFrame();
  actionLayout->addStretch(1);
  addActionButton(i18n("Decline"), SLOT(declineRequest()), FALSE); // not implemented
  addActionButton(i18n("Authorize"), SLOT(authorizeRequest()), TRUE);
  actionLayout->addStretch(1);
  layout->addWidget(actionFrame);
}

void RecvAuth::authorizeRequest()
{
//  icqMessage *msg = list->at(idx);

//  printf(i18n("[Sending AUTH to UIN %lu]\n"),msg->uin());
  if(MW->isLogged())
    icq_SendAuthMsg(&MW->icqlink, message->uin());

  QMessageBox::information(topLevelWidget(),i18n("Authorization Granted"),
               i18n("User has been authorized to add to Contact List"),
               i18n("OK"), 0);

  /* save history */
/*  time_t curtm = time(0L);
  struct tm *t = localtime(&curtm);
  TimeStr time(t->tm_hour,t->tm_min,t->tm_mday,t->tm_mon+1,t->tm_year+1900);

  icqMessage msgTemp(msg->uin(),time,"",MSG_AUTH_GRANT);
  QString historypath = MW->contactsDir();
  msgTemp.save(historypath);
  MW->updateHistoryWindows(msg->uin());*/
}

void RecvAuth::declineRequest()
{
//  icqMessage *msg = list->at(idx);
//  QWidget *widget = parentWidget();
//  while (!widget->isTopLevel()) 
//    widget=widget->parentWidget();

  QMessageBox::information(topLevelWidget(),i18n("Auth declined"),
               i18n("User is not authorized to add to Contact List"),
               i18n("OK"), 0);

  /* save history */
/*  time_t curtm = time(0L);
  struct tm *t = localtime(&curtm);
  TimeStr time(t->tm_hour,t->tm_min,t->tm_mday,t->tm_mon+1,t->tm_year+1900);

  icqMessage msgTemp(msg->uin(),time,"",MSG_AUTH_REFUSE);
  QString historypath = MW->contactsDir();
  msgTemp.save(historypath);
  MW->updateHistoryWindows(msg->uin());*/
}

// Chat Request Event
RecvChatReq::RecvChatReq(icqMessage *msg, QWidget *parent)
  : RecvDetail(msg, parent)
{
  QMultiLineEdit *msgbody = new QMultiLineEdit(this);
  msgbody->setText(message->body());
  layout->addWidget(msgbody, 1);

  bool enableFlag;
  addActionFrame();
  actionLayout->addStretch(1);
  if(!message->isNew())
    enableFlag = FALSE;
  else
    enableFlag = TRUE;
  addActionButton(i18n("Decline"), SLOT(declineRequest()), FALSE); // not implemented yet
  addActionButton(i18n("Accept"), SLOT(acceptRequest()), enableFlag);
  actionLayout->addStretch(1);
  layout->addWidget(actionFrame);
}

void RecvChatReq::acceptRequest()
{
  ChatWindow *pChatWindow = new ChatWindow(message->seq(), message->uin());
  pChatWindow->show();
  icq_AcceptChatRequest(&MW->icqlink, message->uin(), message->seq());
  if(MW->isCloseRecv())
    topLevelWidget()->close();
}

void RecvChatReq::declineRequest()
{
  // TODO: add chat req decline code, needs icqlib support 1st
}

// File Request
RecvFileReq::RecvFileReq(icqMessage *msg, QWidget *parent)
  : RecvDetail(msg, parent)
{
  QMultiLineEdit *msgbody = new QMultiLineEdit(this);
  msgbody->setText(message->body());
  layout->addWidget(msgbody, 1);

  QHBoxLayout *boxLayout = new QHBoxLayout(5);
  layout->addLayout(boxLayout);

  QLabel *nameLabel = new QLabel(i18n("File:"),this);
  QLineEdit *fileedit = new QLineEdit(this);
  fileedit->setText(msg->file());
  fileedit->setFocusPolicy(QWidget::NoFocus);
  fileedit->setMinimumSize(fileedit->sizeHint());
  nameLabel->setMinimumSize(nameLabel->sizeHint());

  QLabel *sizeLabel = new QLabel(i18n("Size:"),this);
  QLineEdit *sizeedit = new QLineEdit(this);
  sizeedit->setText(QString().sprintf("%lu", msg->fileSize()));
  sizeedit->setFocusPolicy(QWidget::NoFocus);
  sizeedit->setMinimumSize(sizeedit->sizeHint());
  sizeLabel->setMinimumSize(sizeLabel->sizeHint());

  boxLayout->addWidget(nameLabel);
  boxLayout->addWidget(fileedit, 1);
  boxLayout->addWidget(sizeLabel);
  boxLayout->addWidget(sizeedit);

  bool enableFlag;
  addActionFrame();
  actionLayout->addStretch(1);
  if(!message->isNew())
    enableFlag = FALSE;
  else
    enableFlag = TRUE;
  addActionButton(i18n("Decline"), SLOT(declineRequest()), FALSE); // not implemented yet
  addActionButton(i18n("Save"), SLOT(saveRequest()), enableFlag);
  addActionButton(i18n("Save As..."), SLOT(saveAsRequest()), enableFlag);
  actionLayout->addStretch(1);
  layout->addWidget(actionFrame);
}

void RecvFileReq::saveRequest()
{
  icq_FileSession *pfile = icq_AcceptFileRequest(&MW->icqlink,
    message->uin(), message->seq());
  FileWindow *pFileWindow = new FileWindow(pfile, message->seq());
  pFileWindow->show();
  if(MW->isCloseRecv())
    topLevelWidget()->close();
}

void RecvFileReq::saveAsRequest()
{
  QString directory;
  char working_dir[1024];

  getcwd(working_dir, 1024);

  directory=KFileBaseDialog::getDirectory(working_dir);

  if(!directory.isEmpty())
  {
    icq_FileSession *pfile = icq_AcceptFileRequest(&MW->icqlink,
      message->uin(), message->seq());
    FileWindow *pFileWindow = new FileWindow(pfile, message->seq());
    pFileWindow->show();
    pFileWindow->setWorkingDirectory(directory);
    if(MW->isCloseRecv())
      topLevelWidget()->close();
  }
}

void RecvFileReq::declineRequest()
{
  // TODO: add file req decline code, needs icqlib support 1st
}

// Added event
RecvAdded::RecvAdded(icqMessage *msg, QWidget *parent)
  : RecvDetail(msg, parent)
{
  QLabel *addedlabel = new QLabel(i18n(
    "The user added you to his/her contact list.\n\n" \
    "You can share the details of your mutual contacts\n" \
    "by clicking on the \"Send Contacts\" button."),
    this);
  addedlabel->setAlignment(AlignCenter);
  addedlabel->setMinimumSize(addedlabel->sizeHint());
  layout->addWidget(addedlabel, 1);

  addActionFrame();
  actionLayout->addStretch(1);
  addActionButton(i18n("Add To Contact List"), SLOT(addRequest()),
                  !MW->contactList().isUINInContactList(msg->uin()));
//  addActionButton(i18n("Send Contacts"), SLOT(sendContactsRequest()), FALSE); // not implemented
  addActionButton(i18n("Reply"), SLOT(replyRequest()), TRUE);
  actionLayout->addStretch(1);
  layout->addWidget(actionFrame);
}

void RecvAdded::sendContactsRequest()
{
  // TODO: not implemented yet.
}

// Main class
RecvWindow::RecvWindow(QList<icqMessage> *l, int i, unsigned long u, bool d)
  : IcqWindow(d)
{
  list = l;
  start = idx = i;
  msgUin = u;
  delflag = d;

  resize(1, 1);

  KIconLoader *il = kapp->getIconLoader();

  page = new QVBoxLayout(this, 5, 7);

  // From box
  QGroupBox *fromGroup = new QGroupBox(i18n("From"), this);

  QLabel *handlelabel = new QLabel(i18n("Handle:"), fromGroup);
  handleline = new QLineEdit(fromGroup);
  handleline->setFocusPolicy(QWidget::NoFocus);
  handlelabel->setBuddy(handleline);
  handlelabel->setMinimumSize(handlelabel->sizeHint());
  handleline->setMinimumSize(handleline->sizeHint());
  handleline->setMinimumWidth(handleline->fontMetrics().width("QQQQQQQQQQQQ"));

  QLabel *emaillabel = new QLabel(i18n("EMail:"), fromGroup);
  emailline = new QLineEdit(fromGroup);
  emailline->setFocusPolicy(QWidget::NoFocus);
  emaillabel->setBuddy(emailline);
  emaillabel->setMinimumSize(emaillabel->sizeHint());
  emailline->setMinimumSize(emailline->sizeHint());

  QPushButton *infobutton = new QPushButton(fromGroup);
  QPixmap infopix = il->loadIcon("info.xpm");
  infobutton->setPixmap(infopix);
  infobutton->setMinimumSize(infopix.width()+6, infopix.height()+6);

  connect(infobutton, SIGNAL(clicked()), SLOT(infoRequest()));

  QLabel *uinlabel = new QLabel(i18n("ICQ#:"), fromGroup);
  uinline = new QLineEdit(fromGroup);
  uinline->setFocusPolicy(QWidget::NoFocus);
  uinlabel->setBuddy(uinline);
  uinline->setMinimumSize(uinline->sizeHint());
  uinline->setMinimumWidth(uinline->fontMetrics().width("0000000000"));
  uinlabel->setMinimumSize(uinlabel->sizeHint());

  QLabel *datelabel = new QLabel(i18n("Date:"), fromGroup);
  dateline = new QLineEdit(fromGroup);
  dateline->setFocusPolicy(QWidget::NoFocus);
  datelabel->setBuddy(dateline);
  dateline->setMinimumSize(dateline->sizeHint());
  dateline->setMinimumWidth(dateline->fontMetrics().width(" 00/00/0000 "));
  datelabel->setMinimumSize(datelabel->sizeHint());

  QLabel *timelabel = new QLabel(i18n("Time:"), fromGroup);
  timeline = new QLineEdit(fromGroup);
  timeline->setFocusPolicy(QWidget::NoFocus);
  timelabel->setBuddy(timeline);
  timeline->setMinimumSize(timeline->sizeHint());
  timeline->setMinimumWidth(timeline->fontMetrics().width(" 24:60 "));
  timelabel->setMinimumSize(timelabel->sizeHint());

  // set up widget layout
  QGridLayout *fromGrid = new QGridLayout(fromGroup, 4, 9, 4, 5);

  fromGrid->addWidget(handlelabel,1,1);
  fromGrid->addWidget(handleline,1,2);
  fromGrid->addWidget(emaillabel,1,3);
  fromGrid->addMultiCellWidget(emailline,1,1,4,6);
  fromGrid->addWidget(infobutton, 1, 7);

  fromGrid->addWidget(uinlabel,2,1);
  fromGrid->addWidget(uinline,2,2);
  fromGrid->addWidget(datelabel,2,3);
  fromGrid->addWidget(dateline,2,4);
  fromGrid->addWidget(timelabel,2,5);
  fromGrid->addMultiCellWidget(timeline,2,2,6,7);

  // set up spacing for the layout borders, which exist to skip over the
  // QGroupBox title
  fromGrid->addRowSpacing(0, 6);

  // hack to make layout look right - not sure why this is necessary, but
  // initial size is wrong when infobutton is fixed size and this is not
  // done
  fromGrid->addColSpacing(6, timeline->minimumSize().width() - infopix.width() + 6);

  // set the stretch factors for the columns, so only the line edits stretch
  fromGrid->setColStretch(2, 1);
  fromGrid->setColStretch(4, 1);
  fromGrid->setColStretch(6, 1);

  fromGrid->activate();

  page->addWidget(fromGroup);

  // Detail widget
  detail = new QWidgetStack(this);
  detail->setBackgroundMode(QWidget::NoBackground);
  QLabel *tmpWidget = new QLabel(this);
  // 15 rows should be enough for message body and action buttons
  tmpWidget->setMinimumHeight(tmpWidget->fontMetrics().height()*15);
  detail->addWidget(tmpWidget, 1);
  page->addWidget(detail, 1);

  // footer buttons
  QPushButton *closebtn = new QPushButton(i18n("Close"), this);
  closebtn->setMinimumSize(closebtn->sizeHint());
  connect(closebtn, SIGNAL(clicked()), SLOT(closeBtn()));

  nextbtn = new QPushButton(i18n("Read Next"),this);
  nextbtn->setMinimumSize(nextbtn->sizeHint());
  connect(nextbtn, SIGNAL(clicked()), SLOT(readNext()));

  // footer layout
  QHBoxLayout *footerBox = new QHBoxLayout(5);
  page->addLayout(footerBox);
  footerBox->addStretch(2);
  footerBox->addWidget(closebtn);
  footerBox->addWidget(nextbtn);

  // Set initial detail widget
  readNext();
  page->activate();
}

void RecvWindow::setUIN(unsigned long u)
{
  QString text;
  text.setNum(u);
  uinline->setText(text);
}

unsigned long RecvWindow::UIN()
{
  return msgUin;
}

void RecvWindow::setName(const char *name)
{
  nick = name;
  handleline->setText(name);
}

void RecvWindow::setEMail(const char *pemail)
{
  email = pemail;
  emailline->setText(email);
}

void RecvWindow::setTime(TimeStr msgtime)
{
  QString date;
  QString time;
  date.sprintf("%02i/%02i/%04i", msgtime.day, msgtime.month, msgtime.year);
  dateline->setText(date);
  time.sprintf("%02i:%02i", msgtime.hour, msgtime.minute);
  timeline->setText(time);
}

void RecvWindow::closeBtn()
{
  close();
}

void RecvWindow::infoRequest()
{
  MW->infoWindow(msgUin);
}

void RecvWindow::readNext()
{
  if(start != idx && list->count() - 1 > (unsigned int)idx) 
    idx++;
  if(start == idx)
    start = -1;
 
  icqMessage *msg = list->at(idx);

  makeWindow(msg);

  if(msg->type() != MSG_AUTH_RECV && msg->type() != MSG_PAGER_RECV &&
     msg->type() != MSG_EXPRESS_RECV)
  {
    setName(MW->contactList().getNickByUIN(msg->uin()));
    setEMail(MW->contactList().getEMailByUIN(msg->uin()));
  }
  else
  {
    if(msg->nick())
      setName(msg->nick());
    else
      setName(MW->contactList().getNickByUIN(msg->uin()));
    if(msg->email())
      setEMail(msg->email());
    else
      setEMail(MW->contactList().getEMailByUIN(msg->uin()));
  }
  setUIN(msg->uin());
  setTime(msg->time());

  msg->expire();

  QString eventInfo;

  if(list->count() - 1 > (unsigned int)idx) 
  {
    nextbtn->setEnabled(TRUE);
    eventInfo = "";
  }
  else
  {
    nextbtn->setEnabled(FALSE);
    if(delflag)
      MW->listWidget()->setBlinkingByUIN(msg->uin(),FALSE);
    eventInfo = i18n(" (No More Events)");
  }
  if(delflag)
    MW->updateBlinking();

  recvCaption = msg->typeTitleString();

  setCaption((const char*)(recvCaption + eventInfo));
}

void RecvWindow::updateNextAvailability()
{
  if(list->count() - 1 > (unsigned int)idx)
  {
    nextbtn->setEnabled(TRUE);
    icqMessage *msg = list->at(idx);
    recvCaption = msg->typeTitleString();
    setCaption(recvCaption);
  }
  else
    nextbtn->setEnabled(FALSE);
}

void RecvWindow::makeWindow(icqMessage *msg)
{
  static int x=2; // used for widget IDs in the detail widget stack
  RecvDetail *oldDetail = (RecvDetail*)detail->visibleWidget();

  RecvDetail *newDetail = 0L;

  switch(msg->type())
  {
    case MSG_SENT:
    case MSG_RECV:
    case MSG_PAGER_RECV:
    case MSG_EXPRESS_RECV:
      newDetail = new RecvMessage(msg, detail);
      break;
    case MSG_URL_SENT:
    case MSG_URL_RECV:
      newDetail = new RecvURL(msg, detail);
      break;
    case MSG_CHAT_REQ_RECV:
      newDetail = new RecvChatReq(msg, detail);
      break;
    case MSG_FILE_REQ_RECV:
      newDetail = new RecvFileReq(msg, detail);
      break;
    case MSG_AUTH_RECV:
    case MSG_AUTH_GRANT:
    case MSG_AUTH_REFUSE:
      newDetail = new RecvAuth(msg, detail);
      break;
    case MSG_ADDED:
      newDetail = new RecvAdded(msg, detail);
      break;
    default:
      break;
  }

  if(newDetail)
  {
    detail->addWidget(newDetail, x++);
    detail->raiseWidget(newDetail);

    if(oldDetail)
    {
      detail->removeWidget(oldDetail);
      delete oldDetail;
    }
  }

  newDetail->activateLayout();

  return;
}
