/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
// $Id: urlsndwin.cpp,v 1.5 2000/07/28 18:36:27 denis Exp $
// $Log: urlsndwin.cpp,v $
// Revision 1.5  2000/07/28 18:36:27  denis
// Code for setting per-user "Send Thru Server" status added.
//
// Revision 1.4  2000/07/23 16:31:19  denis
// Alt+S accelerator along with Ctrl+Enter for "send" action was added.
//
// Revision 1.3  2000/07/09 18:42:39  denis
// Attempt to add Alt+S accelerator.
//
// Revision 1.2  2000/06/17 17:11:58  denis
// Cleanups.
//
// Revision 1.1  2000/05/24 16:58:31  denis
// Sources have been splitted to the whole bunch of appropriate
// directories. Every directory is end up with static library. All
// of them are linked together to the final kicq binary.
//
// Revision 1.17  2000/01/10 16:09:25  denis
// Headers reorganized for simplify dependencies and speedup compilation.
//
// Revision 1.16  1999/12/02 14:08:51  denis
// Random IcqWindow segfaults fixed.
// Cleanups.
//
// Revision 1.15  1999/10/03 12:13:43  guruz
// Various Changes
//
// Revision 1.14  1999/09/29 17:35:51  denis
// Cleanups.
//
// Revision 1.13  1999/07/16 15:49:02  denis
// Cleaned up.
//
// Revision 1.12  1999/07/16 13:04:31  denis
// Internalization support added.
// Smart loading of spinning face animation implemented.
//
// Revision 1.11  1999/07/12 15:23:33  cproch
// - changed icq_* function calls and callback function to support
//   icqlib with ICQLINK identifier.
// - look for FIXME in mainwindow.cpp
//
// Revision 1.10  1999/07/03 02:44:10  bills
// added fail dialog
//
// Revision 1.9  1999/04/17 20:22:25  bills
// modified for new icqwindow base class
//
// Revision 1.8  1999/03/31 01:27:08  bills
// added tcpflag check to sendURL
//
// Revision 1.7  1999/03/28 02:02:37  bills
// fixed number of args to icq_SendURL
//
// Revision 1.6  1998/12/08 16:33:51  denis
// Cleaned up a little before releasing
//

#include <qmsgbox.h>
#include <qmovie.h>
#include <qregexp.h>
#include <qkeycode.h>
#include <qaccel.h>
#include <kapp.h>
#include <kiconloader.h>

#include "urlsndwin.h"
#include "mainwindow.h"
#include "icqmessage.h"
#include "userlist.h"
#include "faildialog.h"
#include "kurllineedit.h"

#include "urlsndwin.moc"

URLSndWindow::URLSndWindow(const char *n, unsigned long u, const char *em) : IcqWindow()
{
  QString text;
  QPixmap infopix;
  KIconLoader *il = kapp->getIconLoader();
  setCaption(i18n("Send URL"));
  tobox = new QGroupBox(i18n("To"),this);
  sendbutton = new QPushButton(i18n("&Send"),this);
  QAccel *a = new QAccel(this);
  a->connectItem(a->insertItem(Key_Return+CTRL), this, SLOT(sendURL()));
  connect(sendbutton, SIGNAL(clicked()),SLOT(sendURL()));
  cancelbutton = new QPushButton(i18n("&Cancel"),this);
  connect(cancelbutton, SIGNAL(clicked()),SLOT(cancelURL()));
  uinline = new QLineEdit(this);
  uinline->setFocusPolicy(QWidget::NoFocus);
  uinlabel = new QLabel(uinline,i18n("ICQ#:"),this);
  handleline = new QLineEdit(this);
  handleline->setFocusPolicy(QWidget::NoFocus);
  handlelabel = new QLabel(handleline,i18n("Handle:"),this);
  emailline = new QLineEdit(this);
  emailline->setFocusPolicy(QWidget::NoFocus);
  emaillabel = new QLabel(emailline,i18n("EMail:"),this);
  infobutton = new QPushButton(this);
  infopix = il->loadIcon("info.xpm");
  infobutton->setPixmap(infopix);
  infobutton->resize(infopix.width()+6, infopix.height()+6);
  connect(infobutton, SIGNAL(clicked()), SLOT(infoRequest()));
  urlbox = new KURLLineEdit(this);
  urllabel = new QLabel(urlbox,i18n("Select/Enter URL:"), this);
  editbox = new QMultiLineEdit(this);
  editlabel = new QLabel(editbox,i18n("Enter URL Description:"),this);
  updateanim = new QLabel(this);
  QString filename = kapp->localkdedir()+"/share/apps/"+kapp->appName()+"/pics/search.gif";
  QFile testfile(filename);
  if(!testfile.exists())
    filename = kapp->kde_datadir()+"/"+kapp->appName()+"/pics/search.gif";
  updateanim->setMovie(QMovie((const char*)filename));
  updateanim->setFrameStyle(QFrame::WinPanel | QFrame::Sunken);
  updateanim->setAlignment(AlignCenter);
  updateanim->movie()->pause();
  setName(n);
  setUIN(u);
  setEMail(em);
  urlbox->setFocus();
  resize(350,250);
  setMinimumSize(350,250);
  faildialog=0L;
}

URLSndWindow::~URLSndWindow()
{
  if(sendbutton)
    delete sendbutton;
  if(cancelbutton)
    delete cancelbutton;
  if(tobox)
    delete tobox;
  if(uinlabel)
    delete uinlabel;
  if(handlelabel)
    delete handlelabel;
  if(emaillabel)
    delete emaillabel;
  if(uinline)
    delete uinline;
  if(handleline)
    delete handleline;
  if(emailline)
    delete emailline;
  if(infobutton)
    delete infobutton;
  if(editbox)
    delete editbox;
  if(editlabel)
    delete editlabel;
  if(urlbox)
    delete urlbox;
  if(urllabel)
    delete urllabel;
  if(updateanim)
    delete updateanim;
}

void URLSndWindow::resizeEvent(QResizeEvent *re)
{
  IcqWindow::resizeEvent(re);
  QSize uinlblsize,handlelblsize,emaillblsize,toboxsize,editlblsize,btn1size,btn2size,urllblsize,urlboxsize;
  toboxsize.setWidth(width()-6);
  uinlblsize = uinlabel->sizeHint();
  handlelblsize = handlelabel->sizeHint();
  emaillblsize = emaillabel->sizeHint();
  int wl = QMAX(uinlblsize.width(),emaillblsize.width());
  int w = (toboxsize.width()-uinlblsize.width()-handlelblsize.width()-36)/2;
  uinline->resize(w,uinline->sizeHint().height());
  handleline->resize(w,uinline->sizeHint().height());
  emailline->resize(toboxsize.width()-infobutton->width()-62,uinline->sizeHint().height());
  uinlabel->resize(wl,uinline->height());
  handlelabel->resize(handlelblsize.width(),handleline->height());
  emaillabel->resize(wl,emailline->height());
  uinlabel->move(12,15);
  uinline->move(3+uinlabel->x()+uinlabel->width(),15);
  handlelabel->move(12+uinline->x()+uinline->width(),15);
  handleline->move(3+handlelabel->x()+handlelabel->width(),15);
  emaillabel->move(12,20+uinline->height());
  emailline->move(3+emaillabel->x()+emaillabel->width(),20+uinline->height());
  tobox->setGeometry(3,3,toboxsize.width(),uinline->height()+emailline->height()+23);
  infobutton->move(7+emailline->x()+emailline->width(),emailline->y()+
                   (emailline->height()-infobutton->height())/2);
  btn1size = sendbutton->sizeHint();
  btn2size = cancelbutton->sizeHint();
  int h = QMAX(btn1size.height(),btn2size.height());
  w = QMAX(QMAX(btn1size.width(),btn2size.width()),75);
  sendbutton->setGeometry(width()/2-w-6,height()-h-12,w,h);
  cancelbutton->setGeometry(width()/2+6,height()-h-12,w,h);
  urllblsize = urllabel->sizeHint();
  urlboxsize = urlbox->sizeHint();
  urllabel->setGeometry(6,8+tobox->height(),urllblsize.width(),urllblsize.height());
  urlbox->setGeometry(3,urllabel->y()+urllabel->height()+4,width()-6,urlboxsize.height());
  editlblsize = editlabel->sizeHint();
  editlabel->setGeometry(6,5+urlbox->y()+urlbox->height(),editlblsize.width(),editlblsize.height());
  editbox->setGeometry(3,editlabel->y()+editlabel->height()+4,width()-6,height()-editlabel->y()-
                       editlabel->height()-h-28);
  updateanim->setGeometry(width()-40,height()-40,32,32);
  updateanim->movie()->restart();
}

void URLSndWindow::setUIN(unsigned long u)
{
  uin = u;
  QString text;
  text.setNum(uin);
  uinline->setText(text);
}

void URLSndWindow::setName(const char *name)
{
  handleline->setText(name);
}

void URLSndWindow::setEMail(const char *email)
{
  emailline->setText(email);
}

void URLSndWindow::sendURL()
{
  sendURL(MW->contactList().getThruServerByUIN(uin)?ICQ_SEND_THRUSERVER:ICQ_SEND_BESTWAY);
}

void URLSndWindow::sendURL(int method)
{
  QString tmp;
  if(MW->isLogged())
  {
    tmp = editbox->text();
    if(MW->isConvertCRLF())
      tmp.replace(QRegExp("\n"),"\r\n");
    sequence = icq_SendURL(&MW->icqlink, uin,(const char*)urlbox->text(),(const char*)tmp,method);
    updateanim->movie()->unpause();
    sendbutton->setEnabled(FALSE);
  }
  else
    QMessageBox::warning(this, i18n("Error"), i18n("You are not connected to the icq network!"));
}

void URLSndWindow::cancelURL()
{
  if(sequence==0)
    close();
  else
    stopCommand();
}

void URLSndWindow::infoRequest()
{
  MW->infoWindow(uin);
}

void URLSndWindow::stopCommand()
{
  updateanim->movie()->pause();
  sendbutton->setEnabled(TRUE);
  sequence = 0;
}

void URLSndWindow::saveHistory()
{
  time_t curtm = time(0L);
  struct tm *t = localtime(&curtm);
  TimeStr time(t->tm_hour,t->tm_min,t->tm_mday,t->tm_mon+1,t->tm_year+1900) ;

  icqMessage msg(uin,time,editbox->text(),MSG_URL_SENT) ;
  QString historypath = MW->contactsDir() ;
  msg.save(historypath) ;
  MW->updateHistoryWindows(msg.uin()) ;
}

void URLSndWindow::requestNotify(int type, int length, void *data)
{
  if(type==ICQ_NOTIFY_ACK)
    saveHistory();

  if(type==ICQ_NOTIFY_FAILED)
  {
    stopCommand();
    if (!faildialog) {
      faildialog=new FailDialog(this, "fail dialog", TRUE);
      faildialog->move(x()+50, y()+50);
      faildialog->resize(width()-100, height()-100);
      connect(faildialog, SIGNAL(done(int)), this, SLOT(failResult(int)));
    }
    faildialog->show();
  }

  IcqWindow::requestNotify(type, length, data);
}

void URLSndWindow::failResult(int i)
{
  faildialog->close();
  faildialog=0L;

  if(i==3)
    cancelURL();
  if(i==2)
    sendURL();
  if(i==1)
  {
    MW->contactList().setThruServerByUIN(uin, TRUE);
    sendURL(ICQ_SEND_THRUSERVER);
  }
}
