/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
// $Id: preferences.cpp,v 1.5 2000/07/11 17:59:32 denis Exp $
// $Log: preferences.cpp,v $
// Revision 1.5  2000/07/11 17:59:32  denis
// Calling saveConfig*() functions without UIN.
//
// Revision 1.4  2000/07/10 09:24:25  pcadach
// Recording of first-time registration information updated. See ChangeLog.
//
// Revision 1.3  2000/07/09 18:40:21  denis
// setUser() renamed to setOwner()
// setDefault() added.
// setSound() changed to accept "clear" parameter.
//
// Revision 1.2  2000/06/17 17:12:35  denis
// Cleanups.
//
// Revision 1.1  2000/05/24 16:59:04  denis
// Sources have been splitted to the whole bunch of appropriate
// directories. Every directory is end up with static library. All
// of them are linked together to the final kicq binary.
//
// Revision 1.17  2000/01/10 16:09:05  denis
// Headers reorganized for simplify dependencies and speedup compilation.
//
// Revision 1.16  1999/12/02 14:32:34  denis
// Layout management implemented.
// Preferences dialog style switched from "Tabbed" dialog to
// "Netscape Preferences"-like style.
//
// Revision 1.15  1999/11/11 15:10:36  guruz
// - Added Base for Webpager Messages. Please type "make fixme"
// - Removed Segfault when kicq is started the first time
//
// Revision 1.14  1999/10/09 16:13:48  guruz
// Enhanced the Quick Phrase Feature. It now has an own Preferences Tab
//
// Revision 1.13  1999/10/06 19:11:30  denis
// Cleanups.
//
// Revision 1.12  1999/10/04 19:15:59  guruz
// -Added "make fixme"
// -The URL Edits are now a bit better
// -The Browser Tab of the Preferences window has another caption
//
// Revision 1.11  1999/10/03 19:07:40  guruz
// There is now the possibility to open URLs that are in the URL Line Edits !
//
// Revision 1.10  1999/09/29 17:34:11  denis
// kyle@13th-floor.org auto-away patch applied.
//
// Revision 1.9  1999/07/16 15:48:38  denis
// Cleaned up.
//
// Revision 1.8  1999/07/16 13:12:19  denis
// Internalization support added.
//
// Revision 1.7  1998/12/08 16:25:37  denis
// Reconnect on disconnect preference option added
// Cleaned up a little before releasing
//
// Revision 1.6  1998/11/20 12:17:56  denis
// Cleaned a little

#include <qlayout.h>

#include "preferences.h"
#include "mainwindow.h"
#include "prefuser.h"
#include "prefserver.h"
#include "preflog.h"
#include "prefother.h"
#include "prefproxy.h"
#include "prefsound.h"
#include "prefaway.h"
#include "prefbrowser.h"
#include "prefqp.h"
#include "icqsound.h"

#include "preferences.moc"

Preferences::Preferences() : QDialog(0, 0, TRUE)
{
  QListViewItem *listItem;
  PrefItem *prefItem;

  setCaption(i18n("Preferences"));

  pageList.setAutoDelete(FALSE);

  QBoxLayout *topLayout = new QVBoxLayout(this, 7);

  QBoxLayout *hLayout = new QHBoxLayout(7);
  topLayout->addLayout(hLayout, 1);

  prefList = new QListView(this);
  prefList->setAllColumnsShowFocus(TRUE);
  prefList->setFrameStyle(QFrame::WinPanel | QFrame::Sunken);
  prefList->setSorting(-1);
  prefList->addColumn(i18n("Category"));
  prefList->setRootIsDecorated(TRUE);
  connect(prefList, SIGNAL(currentChanged(QListViewItem*)), this, SLOT(curChanged(QListViewItem*)));
  hLayout->addWidget(prefList);

  QWidget *prefWidget = new QWidget(this);
  hLayout->addWidget(prefWidget, 1);

  QGridLayout *widgetLayout = new QGridLayout(prefWidget, 1, 1, 0);

  listItem = new QListViewItem(prefList, i18n("Other"));
  otherwidget = new OtherWidget(prefWidget);
  widgetLayout->addWidget(otherwidget, 0, 0);
  prefItem = new PrefItem(otherwidget, listItem);
  pageList.append(prefItem);

  listItem = new QListViewItem(prefList, i18n("QuickPhrases"));
  qpwidget = new QPPrefWidget(prefWidget);
  widgetLayout->addWidget(qpwidget, 0, 0);
  prefItem = new PrefItem(qpwidget, listItem);
  pageList.append(prefItem);

  listItem = new QListViewItem(prefList, i18n("MIME Bindings"));
  browserwidget = new BrowserPrefWidget(prefWidget);
  widgetLayout->addWidget(browserwidget, 0, 0);
  prefItem = new PrefItem(browserwidget, listItem);
  pageList.append(prefItem);

  listItem = new QListViewItem(prefList, i18n("Logging"));
  loggingwidget = new LoggingWidget(prefWidget);
  widgetLayout->addWidget(loggingwidget, 0, 0);
  prefItem = new PrefItem(loggingwidget, listItem);
  pageList.append(prefItem);

  listItem = new QListViewItem(prefList, i18n("Sound"));
  soundwidget = new SoundWidget(prefWidget);
  widgetLayout->addWidget(soundwidget, 0, 0);
  prefItem = new PrefItem(soundwidget, listItem);
  pageList.append(prefItem);

  listItem = new QListViewItem(prefList, i18n("Away"));
  awaywidget = new AwayWidget(prefWidget);
  widgetLayout->addWidget(awaywidget, 0, 0);
  prefItem = new PrefItem(awaywidget, listItem);
  pageList.append(prefItem);

  listItem = new QListViewItem(prefList, i18n("Proxy"));
  proxywidget = new ProxyWidget(prefWidget);
  widgetLayout->addWidget(proxywidget, 0, 0);
  prefItem = new PrefItem(proxywidget, listItem);
  pageList.append(prefItem);

  listItem = new QListViewItem(prefList, i18n("Server"));
  serverwidget = new ServerWidget(prefWidget);
  widgetLayout->addWidget(serverwidget, 0, 0);
  prefItem = new PrefItem(serverwidget, listItem);
  pageList.append(prefItem);

  listItem = new QListViewItem(prefList, i18n("User"));
  userwidget = new UserWidget(prefWidget);
  widgetLayout->addWidget(userwidget, 0, 0);
  prefItem = new PrefItem(userwidget, listItem);
  pageList.append(prefItem);

  QBoxLayout *butLayout = new QHBoxLayout(7);
  topLayout->addLayout(butLayout);

  butLayout->addStretch(1);

  QPushButton *okBtn = new QPushButton(i18n("Ok"), this);
  okBtn->setFixedHeight(okBtn->sizeHint().height());
  connect(okBtn, SIGNAL(clicked()), SLOT(accept()));
  butLayout->addWidget(okBtn);

  QPushButton *cancelBtn = new QPushButton(i18n("Cancel"), this);
  cancelBtn->setFixedHeight(cancelBtn->sizeHint().height());
  connect(cancelBtn, SIGNAL(clicked()), SLOT(reject()));
  butLayout->addWidget(cancelBtn);

  int w = QMAX(okBtn->sizeHint().width(), cancelBtn->sizeHint().width());
  okBtn->setFixedWidth(w);
  cancelBtn->setFixedWidth(w);

  prefList->setMinimumSize(prefList->sizeHint()+QSize(4, 4));

  butLayout->addStretch(1);

  topLayout->activate();
  curPage = userwidget;
  setPage(curPage);

  resize(1, 1);
}

Preferences::~Preferences()
{
}

void Preferences::setOwner(unsigned long uin, const char *nick, const char *pass)
{
  userwidget->setInfo(uin, nick, pass);
}

unsigned long Preferences::getOwner()
{
  return userwidget->uin();
}

void Preferences::setServer(const char *host, int port)
{
  serverwidget->setInfo(host, port);
}

void Preferences::setProxy(bool use, const char *host, unsigned short port,
                           bool auth, const char *user, const char *pass)
{
  proxywidget->setInfo(use, host, port, auth, user, pass);
}

void Preferences::setLogging(int lvl, const char *logfile)
{
  loggingwidget->setInfo(lvl, logfile);
}

void Preferences::setSound(bool wav, bool clear)
{
  soundwidget->setInfo(wav);
  if(clear)
    MW->sound->clear();
}

void Preferences::setOther(bool rus, bool clsrpl, bool crlf, bool quote, bool recon)
{
  otherwidget->setInfo(rus, clsrpl, crlf, quote, recon);
}

void Preferences::setAway(int awaytimeout, int natimeout, bool awayenable, bool awaytona)
{
  awaywidget->setInfo(awaytimeout, natimeout, awayenable, awaytona);
}

void Preferences::setDefault()
{
  setOwner(0, "", "");
  setServer("icq.mirabilis.com", 4000);
  setProxy(FALSE, "", 1080, FALSE, "", "");
  setLogging(0, "");
  setSound(FALSE, TRUE);
  setOther(FALSE, TRUE, TRUE, FALSE, TRUE);
  setAway(30, 30, TRUE, TRUE);
}

void Preferences::accept()
{
  QDialog::accept();
  MW->saveConfigOwner(userwidget->nick(), userwidget->pass());
  MW->saveConfigServer(serverwidget->host(),serverwidget->port());
  MW->saveConfigProxy(proxywidget->use(), proxywidget->host(), proxywidget->port(),
                      proxywidget->auth(),proxywidget->user(),proxywidget->pass());
  MW->saveConfigLog(loggingwidget->level(),loggingwidget->logfile());
  MW->saveConfigSound(soundwidget->wav());
  MW->saveConfigOther(otherwidget->rus(), otherwidget->closereply(),
                      otherwidget->crlf(), otherwidget->quote(),
                      otherwidget->reconnect());
  MW->saveConfigAway(awaywidget->away(), awaywidget->na(), awaywidget->enable(),
                     awaywidget->awaytona());
  browserwidget->save();
  qpwidget->save();
}

void Preferences::setPage(QWidget *w)
{
  PrefItem *item = pageList.first();
  while(item)
  {
    if(item->widget == w)
    {
      item->widget->show();
      prefList->setCurrentItem(item->name);
      prefList->setSelected(item->name, TRUE);
    }
    else
      item->widget->hide();
    item = pageList.next();
  }
  prefList->setFocus();
}

void Preferences::curChanged(QListViewItem *li)
{
  PrefItem *item = pageList.first();
  while(item)
  {
    if(item->name == li)
      setPage(item->widget);
    item = pageList.next();
  }
}
