/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
// $Id: prefproxy.cpp,v 1.1 2000/05/24 16:59:04 denis Exp $
// $Log: prefproxy.cpp,v $
// Revision 1.1  2000/05/24 16:59:04  denis
// Sources have been splitted to the whole bunch of appropriate
// directories. Every directory is end up with static library. All
// of them are linked together to the final kicq binary.
//
// Revision 1.6  2000/01/10 16:09:09  denis
// Headers reorganized for simplify dependencies and speedup compilation.
//
// Revision 1.5  1999/12/02 14:32:40  denis
// Layout management implemented.
// Preferences dialog style switched from "Tabbed" dialog to
// "Netscape Preferences"-like style.
//
// Revision 1.4  1999/07/16 15:48:44  denis
// Cleaned up.
//
// Revision 1.3  1999/07/16 13:12:22  denis
// Internalization support added.
//
// Revision 1.2  1998/11/19 13:17:15  denis
// Username/Password Auth with check box added
// Password input line's echo character changed
//

#include <qlayout.h>
#include <kapp.h>

#include "prefproxy.h"

#include "prefproxy.moc"

ProxyWidget::ProxyWidget(QWidget *parent) : QWidget(parent)
{
  QBoxLayout *topLayout = new QVBoxLayout(this);

  QLabel *mainlabel = new QLabel(i18n("Enter Proxy Settings:"),this);
  mainlabel->setMinimumSize(mainlabel->sizeHint());
  topLayout->addWidget(mainlabel);

  topLayout->addSpacing(7);

  QGroupBox *proxybox = new QGroupBox(i18n("SOCKS5 Proxy"), this);

  QBoxLayout *boxLayout = new QVBoxLayout(proxybox, 10, 7);

  boxLayout->addSpacing(7);

  usebox = new QCheckBox(i18n("Use SOCKS5 Proxy"), proxybox);
  connect(usebox, SIGNAL(toggled(bool)), SLOT(useToggled(bool)));
  usebox->setMinimumSize(usebox->sizeHint());
  boxLayout->addWidget(usebox);

  QLabel *hostlabel = new QLabel(i18n("Server:"), proxybox);
  hostlabel->setMinimumSize(hostlabel->sizeHint());
  boxLayout->addWidget(hostlabel);

  hostline = new QLineEdit(proxybox);
  hostline->setMinimumSize(hostline->sizeHint());
  hostline->setMaximumHeight(hostline->sizeHint().height());
  boxLayout->addWidget(hostline, 1);

  QLabel *portlabel = new QLabel(i18n("Port:"), proxybox);
  portlabel->setMinimumSize(portlabel->sizeHint());
  boxLayout->addWidget(portlabel);

  portline = new KIntegerLine(proxybox);
  portline->setMinimumSize(portline->sizeHint());
  portline->setMaximumHeight(portline->sizeHint().height());
  boxLayout->addWidget(portline, 1);

  authbox = new QCheckBox(i18n("RFC1929 Username/Password Authentication"), proxybox);
  connect(authbox, SIGNAL(toggled(bool)), SLOT(authToggled(bool)));
  authbox->setMinimumSize(authbox->sizeHint());
  boxLayout->addWidget(authbox);

  QLabel *userlabel = new QLabel(i18n("User:"), proxybox);
  userlabel->setMinimumSize(userlabel->sizeHint());
  boxLayout->addWidget(userlabel);

  userline = new QLineEdit(proxybox);
  userline->setMinimumSize(userline->sizeHint());
  userline->setMaximumHeight(userline->sizeHint().height());
  boxLayout->addWidget(userline, 1);

  QLabel *passlabel = new QLabel(i18n("Password:"), proxybox);
  passlabel->setMinimumSize(passlabel->sizeHint());
  boxLayout->addWidget(passlabel);

  passline = new QLineEdit(proxybox);
  passline->setEchoMode(QLineEdit::Password);
  passline->setMinimumSize(passline->sizeHint());
  passline->setMaximumHeight(passline->sizeHint().height());
  boxLayout->addWidget(passline, 1);

  boxLayout->activate();
  topLayout->addWidget(proxybox);

  topLayout->addStretch(1);

  topLayout->activate();
}

void ProxyWidget::setInfo(int use, const char *host, int port, int auth, const char *user, const char *pass)
{
  usebox->setChecked(use);
  hostline->setText(host);
  portline->setValue(port);
  authbox->setChecked(auth);
  userline->setText(user);
  passline->setText(pass);
  if(!use)
  {
    hostline->setEnabled(FALSE);
    portline->setEnabled(FALSE);
    authbox->setEnabled(FALSE);
    userline->setEnabled(FALSE);
    passline->setEnabled(FALSE);
  }
  if(!auth)
  {
    userline->setEnabled(FALSE);
    passline->setEnabled(FALSE);
  }
}

int ProxyWidget::use()
{
  return usebox->isChecked();
}

const char *ProxyWidget::host()
{
  return hostline->text();
}

int ProxyWidget::port()
{
  return portline->value();
}

int ProxyWidget::auth()
{
  return authbox->isChecked();
}

const char *ProxyWidget::user()
{
  return userline->text();
}

const char *ProxyWidget::pass()
{
  return passline->text();
}

void ProxyWidget::useToggled(bool p)
{
  if(p)
  {
    hostline->setEnabled(TRUE);
    portline->setEnabled(TRUE);
    authbox->setEnabled(TRUE);
    if(authbox->isChecked())
    {
      userline->setEnabled(TRUE);
      passline->setEnabled(TRUE);
    }
    else
    {
      userline->setEnabled(FALSE);
      passline->setEnabled(FALSE);
    }
  }
  else
  {
    hostline->setEnabled(FALSE);
    portline->setEnabled(FALSE);
    authbox->setEnabled(FALSE);
    userline->setEnabled(FALSE);
    passline->setEnabled(FALSE);
  }
}

void ProxyWidget::authToggled(bool p)
{
  if(p)
  {
    userline->setEnabled(TRUE);
    passline->setEnabled(TRUE);
  }
  else
  {
    userline->setEnabled(FALSE);
    passline->setEnabled(FALSE);
  }
}
