/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
// $Id: prefsound.cpp,v 1.3 2000/07/21 19:13:47 denis Exp $
// $Log: prefsound.cpp,v $
// Revision 1.3  2000/07/21 19:13:47  denis
// Sound events were modified to be strict.
//
// Revision 1.2  2000/07/09 18:44:05  denis
// Cleanups.
//
// Revision 1.1  2000/05/24 16:59:04  denis
// Sources have been splitted to the whole bunch of appropriate
// directories. Every directory is end up with static library. All
// of them are linked together to the final kicq binary.
//
// Revision 1.10  2000/01/10 16:09:14  denis
// Headers reorganized for simplify dependencies and speedup compilation.
//
// Revision 1.9  1999/12/02 14:32:46  denis
// Layout management implemented.
// Preferences dialog style switched from "Tabbed" dialog to
// "Netscape Preferences"-like style.
//
// Revision 1.8  1999/09/29 17:34:57  denis
// Cleanups.
//
// Revision 1.7  1999/07/16 15:48:48  denis
// Cleaned up.
//
// Revision 1.6  1999/07/16 13:23:59  denis
// i18n support added.
// Smart loading of sound files implemented.
//
// Revision 1.5  1999/05/04 00:05:48  bills
// added .wav file sound support!
//
// Revision 1.4  1998/12/08 16:28:03  denis
// Cleaned up a little before releasing
//
// Revision 1.3  1998/11/23 10:09:03  denis
// Cleaned a little
//

#include <qdir.h>
#include <qdict.h>
#include <qlayout.h>
#include <kapp.h>

#include "prefsound.h"
#include "mainwindow.h"
#include "icqsound.h"

#include "prefsound.moc"

SoundWidget::SoundWidget(QWidget *parent) : QWidget(parent)
{
  QBoxLayout *topLayout = new QVBoxLayout(this);

  QLabel *mainlabel = new QLabel(i18n("Enter Sound Settings:"),this);
  mainlabel->setMinimumSize(mainlabel->sizeHint());
  topLayout->addWidget(mainlabel);

  topLayout->addSpacing(7);

  QGroupBox *soundbox = new QGroupBox(i18n("Sounds"), this);

  QBoxLayout *boxLayout = new QVBoxLayout(soundbox, 10, 7);

  boxLayout->addSpacing(7);

  QBoxLayout *butLayout = new QHBoxLayout(10);
  boxLayout->addLayout(butLayout);

  wavbox = new QCheckBox(i18n("Play sounds"), soundbox);
  wavbox->setMinimumSize(wavbox->sizeHint());
  butLayout->addWidget(wavbox);

  butLayout->addStretch(1);

  testbutton = new QPushButton(i18n("&Test"), soundbox);
  testbutton->setMinimumSize(testbutton->sizeHint());
  butLayout->addWidget(testbutton);

  QBoxLayout *listLayout = new QHBoxLayout(10);
  boxLayout->addLayout(listLayout, 1);

  soundlist = new QListBox(soundbox);
  soundlist->insertItem("(none)");
  soundlist->insertItem("(beep)");

  QString soundpath = kapp->localkdedir()+"/share/apps/"+kapp->appName()+"/sounds/";
  QDir dirLocal(soundpath, "*.wav", QDir::Name, (QDir::Readable | QDir::Files));

  if(dirLocal.exists())
    if(dirLocal.entryList())
      soundlist->insertStrList(dirLocal.entryList());

  soundlist->setMinimumHeight(150);

  soundpath = kapp->kde_datadir()+"/"+kapp->appName()+"/sounds/";
  QDir dir(soundpath, "*.wav", QDir::Name, (QDir::Readable | QDir::Files));

  if(dir.exists())
    if(dir.entryList())
      soundlist->insertStrList(dir.entryList());

  connect(soundlist, SIGNAL(highlighted(int)), SLOT(soundSelected(int)));

  eventlist = new QListBox(soundbox);
  eventlist->insertStrList(MW->sound->getEventList());
  eventlist->setMinimumHeight(150);

  int w = QMAX(soundlist->sizeHint().width(), eventlist->sizeHint().width());

  soundlist->setMinimumWidth(w);
  listLayout->addWidget(soundlist, 1);

  eventlist->setMinimumWidth(w);
  listLayout->addWidget(eventlist, 1);

  connect(eventlist, SIGNAL(highlighted(int)), SLOT(eventSelected(int)));

  connect(testbutton, SIGNAL(clicked()), SLOT(testClicked()));

  boxLayout->activate();
  topLayout->addWidget(soundbox, 1);

  topLayout->activate();
}

void SoundWidget::setInfo(int wav)
{
  wavbox->setChecked(wav);
}

int SoundWidget::wav()
{
  return wavbox->isChecked();
}

void SoundWidget::soundSelected(int soundnum)
{
  const char *soundfile=soundlist->text(soundnum);
  int eventnum=eventlist->currentItem();

  if(eventnum > -1)
  {
    QString event=eventlist->text(eventnum);
    MW->sound->setEvent(event, soundfile);
  }
}

void SoundWidget::eventSelected(int i)
{
  if(!MW->sound->getEvent(eventlist->text(i)))
  {
    soundlist->setCurrentItem(0);
  }
  else
  {
    QString soundfile=MW->sound->getEvent(eventlist->text(i));

    int p=0;
    int len=soundlist->count();
    bool found=false;

    while(!found && p<len)
    {
      if(soundfile==(soundlist->text(p)))
        found=true;
      else
        p++;
    }

    if(found)
      soundlist->setCurrentItem(p);
    else
      soundlist->setCurrentItem(0);
  }

  soundlist->centerCurrentItem();
}

void SoundWidget::testClicked()
{
  int soundnum=soundlist->currentItem();

  if(soundnum > 0)
    MW->sound->play(soundlist->text(soundnum));
}
