/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
// $Id: regwin.cpp,v 1.12 2000/07/25 17:21:18 denis Exp $
// $Log: regwin.cpp,v $
// Revision 1.12  2000/07/25 17:21:18  denis
// When no registered users were found in the system on startup,
// open Registration Wizard.
//
// Revision 1.11  2000/07/24 19:02:29  denis
// Cleanups.
//
// Revision 1.10  2000/07/24 04:01:06  bills
// added nick to icq_init call
//
// Revision 1.9  2000/07/22 17:00:33  denis
// Registration error handling was hopefully finished.
//
// Revision 1.8  2000/07/21 19:28:41  denis
// New User Registration Wizard was modified. Now it looks like original
// icq99b as much as possible (except page6)!
// New options on page7:
//   "Publish my online presence on the World Wide Web" and
//   "Do not publish IP address"
// Window for "Add your own personalized message" was added.
// Window for "More Email addresses" was added.
// Window for "For Tech Support" was added.
// Some work for error handling during registration was added.
//
// Revision 1.7  2000/07/11 18:40:36  denis
// New user registration now saves data for new and existing UIN.
// Debug messages were removed.
//
// Revision 1.6  2000/07/10 14:54:54  pcadach
// Store registration information when new user registered.
//
// Revision 1.5  2000/07/09 18:51:43  denis
// WebPresence should be turned on by default for new registered users.
//
// Revision 1.4  2000/06/30 17:10:46  denis
// Some corrections for possible memleaks by Paul Cadach were added.
//
// Revision 1.3  2000/06/25 17:59:10  denis
// RegWindow constructor was splitted to several files, by one on every
// page.
// Some debugging code along with useful one were added - trying to catch
// up this stupid segfault in RegWindow...
//
// Revision 1.2  2000/05/29 02:14:39  bills
// minor spelling/grammar fixes
//
// Revision 1.1  2000/05/24 16:59:21  denis
// Sources have been splitted to the whole bunch of appropriate
// directories. Every directory is end up with static library. All
// of them are linked together to the final kicq binary.
//
// Revision 1.16  2000/05/06 18:24:08  denis
// Existing UIN registration finished. Note: any registered data isn't
// yet saved to the config file.
// Limit State field for 2 characters.
// "Wrong Password" and "Incorrect UIN" callbacks added.
//
// Revision 1.15  2000/05/04 15:45:38  denis
// New user registration procedure almost finished.
// Existing user registration procedure isn't finished.
//
// Revision 1.14  2000/03/30 14:56:22  denis
// Empty destructor added to avoid some GCC problems on FreeBSD.
//
// Revision 1.13  2000/01/10 16:09:19  denis
// Headers reorganized for simplify dependencies and speedup compilation.
//
// Revision 1.12  1999/12/02 14:30:42  denis
// Clear widget focus on tab/page switching.
// Close window by "Close" button.
// Cleanups.
//
// Revision 1.11  1999/09/29 17:38:58  denis
// Cleanups.
//
// Revision 1.10  1999/07/16 15:48:54  denis
// Cleaned up.
//

#include <qmovie.h>
#include <qregexp.h>

#include "regwin.h"
#include "aboutwin.h"
#include "techwin.h"
#include "emailswin.h"
#include "mainwindow.h"

#include "regwin.moc"

RegWindow::RegWindow(QWidget *parent) : QDialog(parent, 0L, TRUE), icqSN(0), icqProxySN(0),
                                        pageList(), icqinit(false), ackList()
{
  KIconLoader *il = kapp->getIconLoader();
  setCaption(i18n("ICQ Registration Wizard"));

  icqServer = "icq.mirabilis.com";
  icqPort = 4000;
  aboutInfo = "";
  secondaryEmail = "";
  previousEmail = "";

  QBoxLayout *topLayout = new QVBoxLayout(this, 10);

  QBoxLayout *secondLayout = new QHBoxLayout(10);
  topLayout->addLayout(secondLayout, 1);

  QLabel *leftPic = new QLabel(this);
  QPixmap regPix = il->loadIcon("reg.xpm");
  leftPic->setFrameStyle(QFrame::WinPanel | QFrame::Sunken);
  leftPic->setAlignment(AlignCenter);
  leftPic->setMargin(0);
  leftPic->setFixedSize(regPix.size()+QSize(4, 4));
  leftPic->setPixmap(regPix);
  secondLayout->addWidget(leftPic);

  rightPanel = new QWidget(this);
  secondLayout->addWidget(rightPanel, 1);

  panelLayout = new QGridLayout(rightPanel, 1, 1);

  pageList.setAutoDelete(FALSE);
  ackList.setAutoDelete(TRUE);

  setupPage0();
  setupPage1();
  setupPage2();
  setupPage3();
  setupPage4();
  setupPage5();
  setupPage6();
  setupPage7();
  setupPage8();
//---------------------------------------------------

  panelLayout->activate();

  KSeparator *separator = new KSeparator(this);
  topLayout->addWidget(separator);

  QBoxLayout *buttonLayout = new QHBoxLayout(0);
  topLayout->addLayout(buttonLayout);

  QPushButton *techBtn = new QPushButton(i18n("For Tech Support"), this);
  techBtn->setMinimumSize(techBtn->sizeHint());
  connect(techBtn, SIGNAL(clicked()), SLOT(slotTechBtn()));
  buttonLayout->addWidget(techBtn);

  buttonLayout->addSpacing(30);
  buttonLayout->addStretch(1);

  backBtn = new QPushButton(i18n("< Back"), this);
  backBtn->setMinimumSize(backBtn->sizeHint());
  connect(backBtn, SIGNAL(clicked()), SLOT(prevPage()));
  buttonLayout->addWidget(backBtn);

  nextBtn = new QPushButton(i18n("Next >"), this);
  nextBtn->setMinimumSize(nextBtn->sizeHint());
  connect(nextBtn, SIGNAL(clicked()), SLOT(nextPage()));
  buttonLayout->addWidget(nextBtn);

  buttonLayout->addSpacing(20);

  QPushButton *cancelBtn = new QPushButton(i18n("Cancel"), this);
  cancelBtn->setMinimumSize(cancelBtn->sizeHint());
  connect(cancelBtn, SIGNAL(clicked()), SLOT(reject()));
  buttonLayout->addWidget(cancelBtn);

  topLayout->activate();

  libTimer = new QTimer(this);
  connect(libTimer, SIGNAL(timeout()), this, SLOT(libTimerDone()));

  curPage = 0;
  setPage(curPage);
  resize(1, 1);
}

RegWindow::~RegWindow()
{
  if(icqSN)
  {
    delete icqSN;
    icqSN = 0L;
  }
  if(icqProxySN)
  {
    delete icqProxySN;
    icqProxySN = 0L;
  }
  pageList.clear();
  ackList.clear();
}

void RegWindow::slotP1LanBtn(bool state)
{
  if(!state)
  {
    p1FWNoBtn->setEnabled(FALSE);
    p1FWYesBtn->setEnabled(FALSE);
  }
  else
  {
    p1FWNoBtn->setEnabled(TRUE);
    p1FWYesBtn->setEnabled(TRUE);
  }
}

void RegWindow::slotP2TimeOut(bool state)
{
  if(!state)
    p2TMLine->setEnabled(FALSE);
  else
    p2TMLine->setEnabled(TRUE);
}

void RegWindow::slotP3rfcBtn(bool state)
{
  if(!state)
  {
    p3NameLine->setEnabled(FALSE);
    p3PassLine->setEnabled(FALSE);
  }
  else
  {
    p3NameLine->setEnabled(TRUE);
    p3PassLine->setEnabled(TRUE);
  }
}

void RegWindow::slotP5EmailLine(const char *txt)
{
  if(QString(txt).isEmpty())
    p5EmailsBtn->setEnabled(FALSE);
  else
    p5EmailsBtn->setEnabled(TRUE);
}

void RegWindow::slotP5EmailsBtn()
{
  EMailsWindow *ewin = new EMailsWindow(secondaryEmail, previousEmail, this);
  if(ewin->exec() == QDialog::Accepted)
  {
    secondaryEmail = ewin->secondary();
    previousEmail = ewin->previous();
  }
  delete ewin;
}

void RegWindow::slotP6AboutBtn()
{
  AboutWindow *awin = new AboutWindow(aboutInfo, this);
  if(awin->exec() == QDialog::Accepted)
    aboutInfo = awin->about();
  delete awin;
}

void RegWindow::slotP8RetryBtn()
{
  showStatus(i18n("Registering User"));
  registerUIN();
}

void RegWindow::slotP8FirewallBtn()
{
  p1LanBtn->setChecked(TRUE);
  p1FWYesBtn->setChecked(TRUE);
  curPage = 2;
  setPage(curPage);
}

void RegWindow::slotTechBtn()
{
  TechWindow *twin = new TechWindow(icqServer, icqPort, this);
  if(twin->exec() == QDialog::Accepted)
  {
    icqServer = twin->host();
    icqPort = twin->port();
  }
  delete twin;
}

void RegWindow::setPage(int page)
{
  int p;
  QWidget *ptr, *item = pageList.first();
  if(!page)
    backBtn->setEnabled(FALSE);
  else
    backBtn->setEnabled(TRUE);
  if((unsigned)page==pageList.count()-1)
  {
    nextBtn->setEnabled(FALSE);
    nextBtn->setText(i18n("Done"));
    showStatus(i18n("Registering User"));
  }
  else
  {
    nextBtn->setEnabled(TRUE);
    nextBtn->setText(i18n("Next >"));
  }
  p = page;
  while(item)
  {
    if(!p)
      item->show();
    else
      item->hide();
    ptr = item->focusWidget();
    if(ptr)
      ptr->clearFocus();
    item = pageList.next();
    p--;
  }

  if((unsigned)page==pageList.count()-1)
    registerUIN();
}

void RegWindow::registerUIN()
{
  bool useproxy = (p1LanBtn->isChecked() && p1FWYesBtn->isChecked() && p2SocksBtn->isChecked());
  QString passwd, nick;
  bool existUIN = p0ExistBtn->isChecked();

  if(existUIN)
  {
    uin = p4UINLine->value();
    passwd = QString(p4PassLine->text()).left(8);
    nick = "";
  }
  else
  {
    uin = 0;
    passwd = QString(p7PassLine->text()).left(8);
    nick = p5NickLine->text();
  }

  icq_Init(&icqlink, uin, (const char*)passwd, (const char*)nick);
  icqinit = true;

  if(useproxy)
    icq_SetProxy(&icqlink, p3HostLine->text(), p3PortLine->value(), p3rfcBtn->isChecked(),
                 p3NameLine->text(), p3PassLine->text());
  else
    icq_UnsetProxy(&icqlink);

  // Set callbacks
  icqlink.icq_Log = logEvent;
  icqlink.icq_WrongPassword = wrongPasswd;
  icqlink.icq_InvalidUIN = invalidUIN;
  icqlink.icq_SetTimeout = setTimeout;
  if(!existUIN)
    icqlink.icq_Logged = loggedIn;
  else
    icqlink.icq_Logged = loggedInExist;
  icqlink.icq_Disconnected = disconnected;
  if(!existUIN)
    icqlink.icq_NewUIN = newUIN;
  icqlink.icq_RequestNotify = requestNotify;

  if(icq_Connect(&icqlink, icqServer, icqPort)<=0)
    return;

  // Create main socket notifier
  icqSocket = icq_GetSok(&icqlink);
  if(icqSN)
  {
    delete icqSN;
    icqSN = 0L;
  }
  icqSN = new QSocketNotifier(icqSocket, QSocketNotifier::Read);
  connect(icqSN, SIGNAL(activated(int)), this, SLOT(dataReceived(int)));

  // Create proxy socket notifier
  if(useproxy)
  {
    icqProxySocket = icq_GetProxySok(&icqlink);
    // FIXME: Verify icqProxySocket is valid (i.e. >= 0)
    if(icqProxySN)
    {
      delete icqProxySN;
      icqProxySN = 0L;
    }
    icqProxySN = new QSocketNotifier(icqProxySocket, QSocketNotifier::Read);
    connect(icqProxySN, SIGNAL(activated(int)), this, SLOT(dataReceived(int)));
  }

  p8SpinLabel->movie()->unpause();

  if(!existUIN)
    icq_RegNewUser(&icqlink, (const char*)passwd);
  else
    icq_Login(&icqlink, STATUS_ONLINE);
}

void RegWindow::nextPage()
{
  QString email;
  bool existUIN;
  switch(curPage)
  {
    case 0:
      curPage = 1;
      break;
    case 1:
      if(p1LanBtn->isChecked() && p1FWYesBtn->isChecked())
        curPage = 2;
      else
      {
        if(p0ExistBtn->isChecked())
          curPage = 4;
        else
          curPage = 5;
      }
      break;
    case 2:
      if(p2SocksBtn->isChecked())
        curPage = 3;
      else
      {
        if(p0ExistBtn->isChecked())
          curPage = 4;
        else
          curPage = 5;
      }
      break;
    case 3:
      if(!strlen(p3HostLine->text()))
      {
        QMessageBox::critical(this, i18n("Error"), i18n("Not a valid Socks Host"));
        return;
      }
      if(!strlen(p3PortLine->text()))
      {
        QMessageBox::critical(this, i18n("Error"),
                              i18n("The field Socks Port is not a valid integer number"));
        return;
      }
      if(p3rfcBtn->isChecked())
      {
        if(!strlen(p3NameLine->text()))
        {
          QMessageBox::critical(this, i18n("Error"), i18n("Not a valid Socks Username"));
          return;
        }
        if(!strlen(p3PassLine->text()))
        {
          QMessageBox::warning(this, i18n("Error"), i18n("You must supply a Socks Password"));
          return;
        }
      }
      if(p0ExistBtn->isChecked())
        curPage = 4;
      else
        curPage = 5;
      break;
    case 4:
      if(!strlen(p4UINLine->text()))
      {
        QMessageBox::critical(this, i18n("Error"), i18n("The field UIN is not a valid integer number"));
        return;
      }
      else if(p4UINLine->value() < 1001 || p4UINLine->value() > 2147483646)
      {
        QMessageBox::critical(this, i18n("Error"),
                              i18n("The field UIN is out of range (1001-2147483646)"));
        return;
      }
      if(!strlen(p4PassLine->text()))
      {
        QMessageBox::warning(this, i18n("Error"), i18n("You must supply a password"));
        return;
      }
      curPage = 8;
      break;
    case 5:
      email = p5EmailLine->text();
      if(!email.isEmpty())
      {
        if(email.find('@')<0 || email.find('@')==0 || (unsigned)email.find('@')==email.length()-1)
        {
          QMessageBox::critical(this, i18n("Error"), i18n("Not a valid e-mail address"));
          return;
        }
      }
      if(email.isEmpty() && !strlen(p5NickLine->text()) &&
         !strlen(p5FirstLine->text()) && !strlen(p5LastLine->text()))
      {
        if(QMessageBox::information(this, i18n("Confirm"), i18n(
           "You've entered no details.\n"\
           "Users won't be able to find\n"\
           "you except by your UIN.\n"\
           "Enter one or more details?"),
           QMessageBox::Yes | QMessageBox::Default,
           QMessageBox::No | QMessageBox::Escape)==QMessageBox::Yes)
          return;
      }
      curPage = 6;
      break;
    case 7:
      if(!strlen(p7PassLine->text()))
      {
        QMessageBox::warning(this, i18n("Error"), i18n("You must supply a password"));
        return;
      }
      if(strcmp(p7PassLine->text(), p7ConfirmLine->text()))
      {
        QMessageBox::warning(this, i18n("Error"), i18n("Confirm password does not match"));
        return;
      }
      if(QMessageBox::warning(this, i18n("Confirm"), i18n(
        "It is very important to remember your password.\n"\
        "We recommend to write it down and put it in a secure\n"\
        "place to prevent unathorized access to your account.\n"\
        "Continue with registration?"),
        QMessageBox::Yes | QMessageBox::Default,
        QMessageBox::No | QMessageBox::Escape)==QMessageBox::No)
        return;
      curPage = 8;
      break;
    case 8:
      existUIN = p0ExistBtn->isChecked();
      if(!existUIN)
        saveNewUIN();
      else
        saveExistUIN();
      accept();
      break;
    default:
      curPage++;
      break;
  }
  curPage = QMIN(pageList.count()-1, (unsigned)curPage);
  setPage(curPage);
}

void RegWindow::saveNewUIN()
{
  MW->saveConfigOwner(uin, p5NickLine->text(), QString(p7PassLine->text()).left(8));
  MW->saveConfigServer(uin, icqServer, icqPort);
  bool useProxy = (p1LanBtn->isChecked() && p1FWYesBtn->isChecked() && p2SocksBtn->isChecked());
  MW->saveConfigProxy(uin, useProxy, p3HostLine->text(), p3PortLine->value(), p3rfcBtn->isChecked(),
                      p3NameLine->text(), p3PassLine->text());
  MW->saveConfigLog(uin, 0, "");
  MW->saveConfigSound(uin, FALSE, TRUE);
  MW->saveConfigOther(uin, FALSE, TRUE, TRUE, FALSE, TRUE);
  MW->saveConfigAway(uin, 30, 30, TRUE, TRUE);
  MW->prepareNewUIN(uin);
}

void RegWindow::saveExistUIN()
{
  MW->saveConfigOwner(uin, "", QString(p4PassLine->text()).left(8));
  MW->saveConfigServer(uin, icqServer, icqPort);
  bool useProxy = (p1LanBtn->isChecked() && p1FWYesBtn->isChecked() && p2SocksBtn->isChecked());
  MW->saveConfigProxy(uin, useProxy, p3HostLine->text(), p3PortLine->value(), p3rfcBtn->isChecked(),
                      p3NameLine->text(), p3PassLine->text());
  MW->saveConfigLog(uin, 0, "");
  MW->saveConfigSound(uin, FALSE, TRUE);
  MW->saveConfigOther(uin, FALSE, TRUE, TRUE, FALSE, TRUE);
  MW->saveConfigAway(uin, 30, 30, TRUE, TRUE);
  MW->prepareNewUIN(uin);
}

void RegWindow::prevPage()
{
  switch(curPage)
  {
    case 4:
    case 5:
      if(p1LanBtn->isChecked() && p1FWYesBtn->isChecked())
      {
        if(p2SocksBtn->isChecked())
          curPage = 3;
        else
          curPage = 2;
      }
      else
        curPage = 1;
      break;
    case 8:
      if(p0ExistBtn->isChecked())
        curPage = 4;
      else
        curPage = 7;
      break;
    default:
      curPage--;
      break;
  }
  curPage = QMAX(0, curPage);
  setPage(curPage);
}

void RegWindow::dataReceived(int sok)
{
  if(sok==icqSocket)
    icq_HandleServerResponse(&icqlink);
  bool useproxy = (p1LanBtn->isChecked() && p1FWYesBtn->isChecked() && p2SocksBtn->isChecked());
  if(useproxy)
    if(sok==icqProxySocket)
      icq_HandleProxyResponse(&icqlink);
}

void RegWindow::loggedIn(ICQLINK *link)
{
  RegWindow *win = MW->regWindow();
  if(!win)
    return;
  win->updateInfo();
}

void RegWindow::loggedInExist(ICQLINK *link)
{
  RegWindow *win = MW->regWindow();
  if(!win)
    return;
  win->existUINisOK();
}

void RegWindow::disconnected(ICQLINK *link)
{
}

void RegWindow::newUIN(ICQLINK *link, unsigned long uin)
{
  RegWindow *win = MW->regWindow();
  if(!win)
    return;
  win->newLogin(uin);
}

void RegWindow::newLogin(unsigned long u)
{
  icqlink.icq_NewUIN = 0; // Do not call us anymore!
  uin = u;
  icqlink.icq_Uin = uin;
  icq_ContactClear(&icqlink);
  icq_Login(&icqlink, STATUS_ONLINE);
}

void RegWindow::updateInfo()
{
  int cnt_stat = 0, cnt_code = 0;
  ackNum *an;
  an = new ackNum(icq_UpdateNewUserInfo(&icqlink, p5NickLine->text(), p5FirstLine->text(),
                  p5LastLine->text(), p5EmailLine->text()));
  ackList.append(an);
  an = new ackNum(icq_UpdateAuthInfo(&icqlink, p7AuthBtn->isChecked()));
  ackList.append(an);
  QString cnt = p6CountryLine->currentText();
  if(!cnt.isEmpty())
    cnt_stat = 0xFE;
  for(int i=0; icq_Countries[i].code!=0xffff; i++)
    if(cnt == icq_Countries[i].name)
      cnt_code = icq_Countries[i].code;
  an = new ackNum(icq_UpdateMetaInfoSet(&icqlink, p5NickLine->text(), p5FirstLine->text(),
                  p5LastLine->text(), p5EmailLine->text(), secondaryEmail, previousEmail,
                  p6CityLine->text(), p6StateLine->text(), p6PhoneLine->text(), "" /* fax */,
                  "" /* street */, "" /* cellular */, 0 /* zip */, cnt_code, cnt_stat,
                  p5HideEml->isChecked()));
  ackList.append(an);
  an = new ackNum(icq_UpdateMetaInfoHomepage(&icqlink, p6AgeLine->value(), p6PageLine->text(),
                  0 /* year */, 0 /* month */, 0 /*  day */, 0 /* lang1 */, 0 /* lang2 */,
                  0 /* lang3 */));
  ackList.append(an);
  an = new ackNum(icq_UpdateMetaInfoAbout(&icqlink, aboutInfo));
  ackList.append(an);
  an = new ackNum(icq_UpdateMetaInfoSecurity(&icqlink, p7AuthBtn->isChecked(),
                  p7WebAwareBtn->isChecked(), p7HideIPBtn->isChecked()));
  ackList.append(an);
}

void RegWindow::requestNotify(ICQLINK *link, unsigned long id, int type, unsigned int length, void *data)
{
  RegWindow *win = MW->regWindow();
  if(!win)
    return;
  if(type == ICQ_NOTIFY_SUCCESS)
    win->ackSrv(id);
}

void RegWindow::ackSrv(unsigned short seq)
{
  ackNum *ptr;
  for(ptr = ackList.first(); ptr != 0L; ptr = ackList.next())
    if(ptr->num==seq)
    {
      ackList.removeRef(ptr);
      if(ackList.isEmpty())
      {
        QString uins;
        uins.sprintf(i18n("New UIN: %lu"), uin);
        showStatus(uins);
        stop(TRUE);
      }
    }
}

void RegWindow::wrongPasswd(ICQLINK *link)
{
  MW->regWindow()->showStatus(i18n("Wrong password!\n"));
  MW->regWindow()->stop(FALSE);
}

void RegWindow::invalidUIN(ICQLINK *link)
{
  MW->regWindow()->showStatus(i18n("Invalid UIN!\n"));
  MW->regWindow()->stop(FALSE);
}

void RegWindow::logEvent(ICQLINK *link, time_t time, unsigned char level, const char *str)
{
  QString msg(str);
  msg.replace(QRegExp("\n"), " ");
  if(level <= ICQ_LOG_ERROR)
  {
    MW->regWindow()->stop(FALSE);
    MW->regWindow()->showStatus(msg);
  }
}

void RegWindow::stop(bool ok)
{
  p8SpinLabel->movie()->pause();
  nextBtn->setEnabled(ok);
  if(icqSN)
  {
    delete icqSN;
    icqSN = 0L;
  }
  if(icqProxySN)
  {
    delete icqProxySN;
    icqProxySN = 0L;
  }
  if(icqinit)
    disconnect();
  if(!ok)
  {
    p8RetryBtn->show();
    p8RetryLabel->show();
    p8FirewallBtn->show();
    p8FirewallLabel->show();
  }
}

void RegWindow::existUINisOK()
{
  QString uins;
  uins.sprintf(i18n("Existing UIN: %lu"), uin);
  showStatus(uins);
  stop(TRUE);
}

void RegWindow::done(int r)
{
  stop(FALSE);
  QDialog::done(r);
}

void RegWindow::libTimerDone()
{
  if(icqinit)
    icq_HandleTimeout(&icqlink);
}

void RegWindow::setTimeout(ICQLINK *link, long interval)
{
  RegWindow *win = MW->regWindow();
  if(!win)
    return;
  if(!win->libTimer)
    return;
  if(interval < 0)
  {
    if(win->libTimer->isActive())
      win->libTimer->stop();
  }
  else
    win->libTimer->start(interval * 1000, TRUE);
}

void RegWindow::disconnect()
{
  icq_Disconnect(&icqlink);
  setTimeout(&icqlink, -1);
  icqinit = false;
  icq_Done(&icqlink);
}

void RegWindow::showStatus(QString msg)
{
  p8StatusLabel->setText((const char*)msg);
}
