/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
// $Id: findwindow.cpp,v 1.3 2000/06/17 17:39:19 denis Exp $
// $Log: findwindow.cpp,v $
// Revision 1.3  2000/06/17 17:39:19  denis
// Contact List code was rewritten from scratch! Although we are trying
// to not add new features in anticipation of the next stable release,
// but this rewrite solves many things, removes many hacks and stabilizes
// whole program.
// One of bug, needs to mention, is an absence of IP address in Info
// window. The root of problem was in Contact List, because in any time
// there were several ContactItems in memory for single user - they were
// "smeared" on the whole code. Now it's clean and strict - all Contact
// List operations can be done only by ContactList class.
//
// Revision 1.2  2000/05/29 02:14:39  bills
// minor spelling/grammar fixes
//
// Revision 1.1  2000/05/24 16:59:32  denis
// Sources have been splitted to the whole bunch of appropriate
// directories. Every directory is end up with static library. All
// of them are linked together to the final kicq binary.
//
// Revision 1.16  2000/05/04 15:39:42  denis
// Callbacks moved to the ICQLINK structure.
//
// Revision 1.15  2000/03/30 15:37:25  denis
// Double/int warnings fixed.
//
// Revision 1.14  2000/01/10 16:08:16  denis
// Headers reorganized for simplify dependencies and speedup compilation.
//
// Revision 1.13  1999/12/02 14:10:53  denis
// Clear widget focus on tab/page switching.
// Cleanups.
//
// Revision 1.12  1999/09/29 17:37:38  denis
// Layout widget stuff implemented.
//
// Revision 1.11  1999/07/16 15:47:55  denis
// Cleaned up.
//
// Revision 1.10  1999/07/16 13:03:46  denis
// Internalization support added.
// Smart loading of spinning face animation implemented.
//
// Revision 1.9  1999/07/12 15:23:19  cproch
// - changed icq_* function calls and callback function to support
//   icqlib with ICQLINK identifier.
// - look for FIXME in mainwindow.cpp
//
// Revision 1.8  1999/05/03 22:17:44  billb
// added sunken border around animation
//
// Revision 1.7  1998/12/08 16:15:49  denis
// Cleaned up a little before releasing
//

#include <icq.h>
#include <qmovie.h>
#include <qmsgbox.h>
#include <qgrpbox.h>
#include <qlayout.h>
#include <kapp.h>

#include "findwindow.h"
#include "mainwindow.h"
#include "userlist.h"
#include "emailwidget.h"
#include "detailswidget.h"
#include "uinwidget.h"

#include "findwindow.moc"

FindWindow::FindWindow() : QDialog()
{
  setCaption(i18n("Search Engine"));

  QBoxLayout *topLayout = new QVBoxLayout(this, 7);

  QBoxLayout *h1Layout = new QHBoxLayout(7);
  topLayout->addLayout(h1Layout);

  tabctl = new KTabCtl(this);
  emailwidget = new EMailWidget(tabctl);
  detailswidget = new DetailsWidget(tabctl);
  uinwidget = new UINWidget(tabctl);
  tabctl->addTab(emailwidget,i18n("EMail"));
  tabctl->addTab(detailswidget,i18n("Details"));
  tabctl->addTab(uinwidget,i18n("ICQ #"));
  connect(tabctl, SIGNAL(tabSelected(int)), SLOT(changeTab(int)));
  h1Layout->addWidget(tabctl, 1);

  QGroupBox *btnbox = new QGroupBox(this);

  QBoxLayout *boxLayout = new QVBoxLayout(btnbox, 7);

  searchbtn = new QPushButton(i18n("Search"),btnbox);
  searchbtn->setFixedHeight(searchbtn->sizeHint().height());
  searchbtn->setDefault(TRUE);
  connect(searchbtn, SIGNAL(clicked()), SLOT(searchCommand()));

  stopbtn = new QPushButton(i18n("Stop"),btnbox);
  stopbtn->setFixedHeight(stopbtn->sizeHint().height());
  stopbtn->setEnabled(FALSE);
  connect(stopbtn, SIGNAL(clicked()), SLOT(stopCommand()));

  newbtn = new QPushButton(i18n("New Search"),btnbox);
  newbtn->setFixedHeight(newbtn->sizeHint().height());
  connect(newbtn, SIGNAL(clicked()), SLOT(clearCommand()));

  int w = QMAX(QMAX(searchbtn->sizeHint().width(), stopbtn->sizeHint().width()),
               newbtn->sizeHint().width());
  searchbtn->setFixedWidth(w);
  stopbtn->setFixedWidth(w);
  stopbtn->setFixedWidth(w);

  boxLayout->addWidget(searchbtn);
  boxLayout->addWidget(stopbtn);
  boxLayout->addWidget(newbtn);

  searchanim = new QLabel(btnbox);
  searchanim->setFixedSize(QSize(32, 32)+QSize(4, 4));
  searchanim->setFrameStyle(QFrame::WinPanel | QFrame::Sunken);
  searchanim->setAlignment(AlignCenter);
  searchanim->setMargin(0);
  QString filename = kapp->localkdedir()+"/share/apps/"+kapp->appName()+"/pics/search.gif";
  QFile testfile(filename);
  if(!testfile.exists())
    filename = kapp->kde_datadir()+"/"+kapp->appName()+"/pics/search.gif";
  searchanim->setMovie(QMovie((const char*)filename));
  searchanim->movie()->pause();
  searchanim->movie()->restart();
  boxLayout->addWidget(searchanim);

  boxLayout->activate();
  h1Layout->addWidget(btnbox);

  foundlist = new QListView(this);
  foundlist->addColumn(i18n("ICQ #"));
  foundlist->addColumn(i18n("Handle"));
  foundlist->addColumn(i18n("First Name"));
  foundlist->addColumn(i18n("Last Name"));
  foundlist->addColumn(i18n("EMail"));
  foundlist->addColumn(i18n("A"));
  foundlist->setAllColumnsShowFocus(TRUE);
  foundlist->setSorting(0);
  foundlist->setFrameStyle(QFrame::WinPanel | QFrame::Sunken);
  foundlist->setMultiSelection(TRUE);
  foundlist->setMinimumHeight(100);
  connect(foundlist, SIGNAL(rightButtonClicked(QListViewItem*, const QPoint&, int)),
          SLOT(popupRequest(QListViewItem*, const QPoint&, int)));
  topLayout->addWidget(foundlist, 1);

  topLayout->activate();

  KIconLoader *il = kapp->getIconLoader();
  popup = new QPopupMenu();
  popup->insertItem(il->loadIcon("add.xpm"), i18n("Add to contact list"), this, SLOT(addRequest()));
  popup->insertItem(il->loadIcon("info.xpm"), i18n("Info"), this, SLOT(infoRequest()));

  resize(1, 1);
}

void FindWindow::resizeEvent(QResizeEvent *re)
{
  QDialog::resizeEvent(re);
  foundlist->setUpdatesEnabled(FALSE);
  foundlist->setColumnWidth(0, (int)(foundlist->width()*0.15));
  foundlist->setColumnWidth(1, (int)(foundlist->width()*0.15));
  foundlist->setColumnWidth(2, (int)(foundlist->width()*0.18));
  foundlist->setColumnWidth(3, (int)(foundlist->width()*0.21));
  foundlist->setColumnWidth(4, (int)(foundlist->width()*0.22));
  foundlist->setColumnWidth(5, (int)(foundlist->width()-
                            foundlist->columnWidth(0)-
                            foundlist->columnWidth(1)-
                            foundlist->columnWidth(2)-
                            foundlist->columnWidth(3)-
                            foundlist->columnWidth(4)-20));
  foundlist->triggerUpdate();
  foundlist->setUpdatesEnabled(TRUE);
  foundlist->repaint();
}

void FindWindow::searchCommand()
{
  if(MW->isLogged())
  {
    if(curtab==0 && !strlen(emailwidget->email()))
    {
      QMessageBox::warning(this, i18n("Error"), i18n("Empty E-Mail query"));
      return;
    }
    if(curtab==1 && !strlen(detailswidget->nick()) && !strlen(detailswidget->first()) &&
       !strlen(detailswidget->last()))
    {
      QMessageBox::warning(this, i18n("Error"), i18n("Empty Details query"));
      return;
    }
    if(curtab==2 && uinwidget->isEmpty())
    {
      QMessageBox::warning(this, i18n("Error"), i18n("Empty ICQ# query"));
      return;
    }
    searchanim->movie()->unpause();
    searchbtn->setEnabled(FALSE);
    stopbtn->setEnabled(TRUE);
    newbtn->setEnabled(FALSE);
    MW->icqlink.icq_UserFound = userFound;
    MW->icqlink.icq_SearchDone = searchDone;
    if(curtab==0)
      icq_SendSearchReq(&MW->icqlink, emailwidget->email(),"","","");
    else if(curtab==1)
      icq_SendSearchReq(&MW->icqlink, "",detailswidget->nick(),
                        detailswidget->first(),detailswidget->last());
    else if(curtab==2)
      icq_SendSearchUINReq(&MW->icqlink, uinwidget->uin());
  }
  else
    QMessageBox::warning(this, i18n("Error"), i18n("You are not connected to the icq network!"));
}

void FindWindow::stopCommand()
{
  searchanim->movie()->pause();
  if(!isVisible())
    searchanim->movie()->restart();
  searchbtn->setEnabled(TRUE);
  stopbtn->setEnabled(FALSE);
  newbtn->setEnabled(TRUE);
}

void FindWindow::clearCommand()
{
  emailwidget->clear();
  detailswidget->clear();
  uinwidget->clear();
  foundlist->clear();
  foundlist->repaint();
}

void FindWindow::addUserLine(unsigned long uin, const char *nick, const char *first,
                             const char *last, const char *email, char auth)
{
  QString iuin;
  QListViewItem *tmp;
  iuin.setNum(uin);

  tmp = new QListViewItem(foundlist, (const char*)iuin, nick, first, last, email,
                          auth==1?i18n("n"):i18n("y"));
}

void FindWindow::userFound(ICQLINK *link, unsigned long uin, const char *nick,
                           const char *first, const char *last, const char *email, char auth)
{
  MW->findWindow()->addUserLine(uin,nick,first,last,email,auth);
}

void FindWindow::searchDone(ICQLINK *link)
{
  MW->findWindow()->stopCommand();
}

void FindWindow::addRequest()
{
  int addFlag = FALSE;
  unsigned long uin;
  QListViewItem *ptr = foundlist->firstChild();
  while(ptr)
  {
    if(foundlist->isSelected(ptr))
    {
      uin = QString(ptr->text(0)).toULong();
      if(!MW->contactList().isUINInContactList(uin))
      {
        ContactItem *item = new ContactItem(uin, ptr->text(1), ptr->text(2), ptr->text(3),
                                            ptr->text(4), ptr->text(5) == i18n("n")?1:0);
        MW->contactList().appendItem(item);
        addFlag = TRUE;
      }
    }
    ptr = ptr->nextSibling();
  }
  if(addFlag)
    MW->updateVisibleList();
}

void FindWindow::infoRequest()
{
  QListViewItem *ptr = foundlist->firstChild();
  while(ptr)
  {
    if(foundlist->isSelected(ptr))
    {
      ContactItem item(QString(ptr->text(0)).toULong(), ptr->text(1), ptr->text(2), ptr->text(3),
                       ptr->text(4), ptr->text(5) == i18n("n")?1:0);
      MW->infoWindow(item);
    }
    ptr = ptr->nextSibling();
  }
}

void FindWindow::popupRequest(QListViewItem *item, const QPoint &pos, int)
{
  if(!item)
    return;
  if(!foundlist->isSelected(item))
  {
    foundlist->setUpdatesEnabled(FALSE);
    QListViewItem *ptr = foundlist->firstChild();
    while(ptr)
    {
      foundlist->setSelected(ptr, FALSE);
      ptr = ptr->nextSibling();
    }
    foundlist->setSelected(item, TRUE);
    foundlist->triggerUpdate();
    foundlist->setUpdatesEnabled(TRUE);
    foundlist->repaint();
  }
  popup->popup(pos);
}

void FindWindow::changeTab(int tab)
{
  QWidget *ptr;
  curtab = tab;
  ptr = emailwidget->focusWidget();
  if(ptr)
    ptr->clearFocus();
  ptr = detailswidget->focusWidget();
  if(ptr)
    ptr->clearFocus();
  ptr = uinwidget->focusWidget();
  if(ptr)
    ptr->clearFocus();
}
