/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
// $Id: icqmessage.cpp,v 1.3 2000/07/30 18:53:49 denis Exp $
// $Log: icqmessage.cpp,v $
// Revision 1.3  2000/07/30 18:53:49  denis
// Warnings were fixed.
//
// Revision 1.2  2000/07/28 18:31:06  denis
// Cleanups.
//
// Revision 1.1  2000/06/17 17:39:28  denis
// Contact List code was rewritten from scratch! Although we are trying
// to not add new features in anticipation of the next stable release,
// but this rewrite solves many things, removes many hacks and stabilizes
// whole program.
// One of bug, needs to mention, is an absence of IP address in Info
// window. The root of problem was in Contact List, because in any time
// there were several ContactItems in memory for single user - they were
// "smeared" on the whole code. Now it's clean and strict - all Contact
// List operations can be done only by ContactList class.
//
// Revision 1.1  2000/05/24 16:58:07  denis
// Sources have been splitted to the whole bunch of appropriate
// directories. Every directory is end up with static library. All
// of them are linked together to the final kicq binary.
//
// Revision 1.20  2000/05/05 03:38:48  bills
// added member variable for filesize, added accessors for file and filesize,
// added code to save method to save filesize for MSG_FILE_REQ_RECV
//
// Revision 1.19  2000/03/30 14:38:37  denis
// Some indentation fixed.
//
// Revision 1.18  2000/02/03 15:53:48  bills
// added new typeString and typeTitleString methods to move message strings
// to one location
//
// Revision 1.17  2000/01/10 16:08:22  denis
// Headers reorganized for simplify dependencies and speedup compilation.
//
// Revision 1.16  1999/12/28 01:43:30  bills
// added new code to handle file transfers
//
// Revision 1.15  1999/12/02 14:14:21  denis
// WebPager and EMailExpress messages handlers added.
// Try to fix Authorization request problems.
// Cleanups.
//
// Revision 1.14  1999/10/14 14:18:38  denis
// Removed i18n from history file delimiters.
//
// Revision 1.13  1999/09/29 17:31:32  denis
// "You were added" message support finished.
//
// Revision 1.12  1999/07/16 15:48:01  denis
// Cleaned up.
//
// Revision 1.11  1999/07/16 13:10:08  denis
// Internalization support added.
//
// Revision 1.10  1999/04/04 22:03:58  bills
// added support for MSG_CHAT_REQ_RECV and chat window handling
//
// Revision 1.9  1999/03/05 13:04:37  denis
// Applied patch to handle authorization request messages by Bogdan Surdu
// <tim@ima-info.ro>
//
// Revision 1.8  1998/12/11 18:20:04  whale
// Added support for MSG_AUTH_RECV (incoming auth requests)
//
// Revision 1.7  1998/12/08 16:16:50  denis
// Cleaned up a little before releasing
//

#include <stdio.h>
#include <qregexp.h>
#include <kapp.h>

#include "icqmessage.h"

const char *icqMessage::icqMsgTypeStrings[] = {
  "Unknown", "Message Sent", "Message Received", "URL Sent", "URL Received",
  "Authorization Requested", "Authorization Granted", "Authorization Refused",
  "Chat Request Received", "Added to Contact List", "WebPager Received",
  "EMailExpress Received", "File Request Received"
};

const char *icqMessage::icqMsgTypeTitles[] = {
  "Unknown", "Outgoing Message", "Incoming Message", "Outgoing URL",
  "Incoming URL", "Authorization Request", "Authorization Granted",
  "Authorization Refused", "Incoming Chat Request", "System Message",
  "Incoming WebPager Message", "Incoming EmailExpress Message",
  "Incoming File Request"
};

/* Message */
icqMessage::icqMessage(unsigned long uin, TimeStr time, const char *body, icqMsgType type)
{
  msgType = type;
  msgUin = uin;
  msgTime = time;
  msgBody = body;
  msgNew = TRUE;
  msgBody.replace(QRegExp("\r"), "");

}

icqMessage::icqMessage(unsigned long uin, TimeStr time, const char *body, unsigned long seq, icqMsgType type)
{
  msgType = type ;
  msgUin = uin;
  msgTime = time;
  msgBody = body;
  msgSeq = seq;
  msgNew = TRUE;
  msgBody.replace(QRegExp("\r"), "");

}

/* MSG_FILE_REQ_RECV */
icqMessage::icqMessage(unsigned long uin, TimeStr time, const char *body, 
                       const char *file, unsigned long size, 
                       unsigned long seq, icqMsgType type)
{
  msgType = type;
  msgUin = uin;
  msgTime = time;
  msgBody = body;
  msgSeq = seq;
  msgFile = file;
  msgFileSize = size;
  msgNew = TRUE;
}

/* URL */
icqMessage::icqMessage(unsigned long uin, TimeStr time, const char *url, const char *desc, icqMsgType type)
{
  msgType = type;
  msgUin = uin;
  msgTime = time;
  msgBody = desc;
  msgURL = url;
  msgNew = TRUE;

  /* Ensure that there is no '\n' in URL */
  msgURL.replace(QRegExp("\n"), "");
/*  int c = 0;
    while((c = msgURL.find('\n',0)) != -1) 
    msgURL.remove(c,1);*/
}

icqMessage::icqMessage(unsigned long uin, TimeStr time, const char *nick, const char *first,
                       const char *last, const char *email, const char *reason, icqMsgType type)
{
  msgType = type;
  msgUin = uin;
  msgTime = time;
  msgBody = reason;
  msgNick = nick;
  msgFirst = first;
  msgLast = last;
  msgEmail = email;
  msgNew = TRUE;
}

icqMessage::icqMessage(unsigned long uin, TimeStr time, const char *nick, const char *first,
                       const char *last, const char *email, icqMsgType type)
{
  msgType = type;
  msgUin = uin;
  msgTime = time;
  msgNick = nick;
  msgFirst = first;
  msgLast = last;
  msgEmail = email;  
  msgNew = TRUE;
}

/* Pager/Express */
icqMessage::icqMessage(unsigned long uin, TimeStr time, const char *nick, const char *email,
                       const char *body, icqMsgType type)
{
  msgType = type;
  msgUin = uin;
  msgTime = time;
  msgNick = nick;
  msgEmail = email;
  msgBody = body;
  msgNew = TRUE;
}

icqMessage::icqMessage(icqMessage *msg)
{
  msgType = msg->type();
  msgUin = msg->uin();
  msgTime = msg->time();
  msgBody = msg->body();
  msgURL = msg->url();
  msgNew = msg->isNew();
}

unsigned long icqMessage::uin() const
{
  return msgUin;
}

const char *icqMessage::body() const
{
  return msgBody;
}

const char *icqMessage::url() const
{
  return msgURL;
}

const char *icqMessage::nick() const
{
  return msgNick;
}

const char *icqMessage::first() const
{
  return msgFirst;
}

const char *icqMessage::last() const
{
  return msgLast;
}

const char *icqMessage::email() const
{
  return msgEmail;
}

TimeStr icqMessage::time() const
{
  return msgTime;
}

int icqMessage::isNew() const
{
  return msgNew;
}

void icqMessage::expire()
{
  msgNew = 0;
}

icqMsgType icqMessage::type()
{
  return msgType;
}

const unsigned long icqMessage::seq() const
{
  return msgSeq;
}

void icqMessage::setType(icqMsgType type)
{
  msgType = type;
}

const char *icqMessage::typeString() const
{
  return icqMsgTypeStrings[msgType];
}

const char *icqMessage::typeTitleString() const
{
  return icqMsgTypeTitles[msgType];
}

const char *icqMessage::typeString(icqMsgType t)
{
  return icqMsgTypeStrings[t];
}

const char *icqMessage::file() const
{
  return msgFile;
}

unsigned long icqMessage::fileSize() const
{
  return msgFileSize;
}

int icqMessage::save(QString path)
{

  QString strUin;
  strUin.setNum(msgUin);
  QString fname = path + "/" + strUin + ".history";

  QFile file(fname);
  if(!file.exists())
    file.open(IO_Raw | IO_WriteOnly);
  else
    file.open(IO_Raw | IO_Append | IO_WriteOnly);

  QString datetime;
  datetime.sprintf("[%s %02i/%02i/%04i %02i:%02i]\n", typeString(),
                   msgTime.day, msgTime.month, msgTime.year,
                   msgTime.hour, msgTime.minute);

  QString separator = "\n";
  file.writeBlock((const char*)datetime, datetime.length());
  QString temp;
  switch(msgType)
  {
    case MSG_URL_RECV:
    case MSG_URL_SENT:
      file.writeBlock((const char*)msgURL, msgURL.length());
      file.writeBlock((const char*)separator, separator.length());
      break;
    case MSG_AUTH_RECV:
    case MSG_ADDED:
      file.writeBlock((const char*)msgNick, msgNick.length());
      file.writeBlock((const char*)separator, separator.length());
      file.writeBlock((const char*)msgFirst, msgFirst.length());
      file.writeBlock((const char*)separator, separator.length());
      file.writeBlock((const char*)msgLast, msgLast.length());
      file.writeBlock((const char*)separator, separator.length());
      file.writeBlock((const char*)msgEmail, msgEmail.length());
      file.writeBlock((const char*)separator, separator.length());
      break;
    case MSG_PAGER_RECV:
    case MSG_EXPRESS_RECV:
      file.writeBlock((const char*)msgNick, msgNick.length());
      file.writeBlock((const char*)separator, separator.length());
      file.writeBlock((const char*)msgEmail, msgEmail.length());
      file.writeBlock((const char*)separator, separator.length());
    case MSG_FILE_REQ_RECV:
      file.writeBlock((const char*)msgFile, msgFile.length());
      file.writeBlock((const char*)separator, separator.length());
      temp.sprintf("%lu", msgFileSize);
      file.writeBlock((const char*)temp, temp.length());
      file.writeBlock((const char*)separator, separator.length());
      break;
    default:
      break;
  }
  file.writeBlock((const char*)msgBody, msgBody.length());
  if(msgBody.right(1) != "\n")
    file.writeBlock((const char*)separator, separator.length());
  file.close();
  return TRUE;
}
