/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
#ifndef _ICQMESSAGE_
#define _ICQMESSAGE_
 
#include <qstring.h>
#include <qfile.h>

enum icqMsgType { MSG_UNDEF, MSG_SENT, MSG_RECV, MSG_URL_SENT, MSG_URL_RECV,
                  MSG_AUTH_RECV, MSG_AUTH_GRANT, MSG_AUTH_REFUSE, MSG_CHAT_REQ_RECV,
                  MSG_ADDED, MSG_PAGER_RECV, MSG_EXPRESS_RECV, MSG_FILE_REQ_RECV };

typedef struct TmStr
{
  unsigned char hour;
  unsigned char minute;
  unsigned char day;
  unsigned char month;
  unsigned short year;
  TmStr()
  {
    hour = 0;
    minute = 0;
    day = 0;
    month = 0;
    year = 0;
  }
  TmStr(unsigned char phour, unsigned char pmin, unsigned char pday, unsigned char pmonth,
        unsigned short pyear)
  {
    hour = phour;
    minute = pmin;
    day = pday;
    month = pmonth;
    year = pyear;
  }
} TimeStr;

class icqMessage
{
public:
  /* Message */
  icqMessage(unsigned long, TimeStr, const char*, icqMsgType);
  icqMessage(unsigned long, TimeStr, const char*, unsigned long, icqMsgType);
  /* URL */
  icqMessage(unsigned long, TimeStr, const char*, const char*, icqMsgType);

  /* AuthRecv */
  icqMessage(unsigned long, TimeStr, const char*, const char*, const char*,
             const char*, const char*, icqMsgType);
  /* AuthGrant/AuthRefuse/Added */
  icqMessage(unsigned long, TimeStr, const char*, const char*, const char*,
             const char*, icqMsgType);
  /* Pager/Express */
  icqMessage(unsigned long, TimeStr, const char*, const char*, const char*,
             icqMsgType);
  /* File Request */
  icqMessage(unsigned long, TimeStr, const char*, const char*, unsigned long,
             unsigned long, icqMsgType);

  icqMessage(icqMessage*);
  const char *body() const;
  unsigned long uin() const;
  const char *url() const;
  const char *nick() const ;
  const char *first() const ;
  const char *last() const ;
  const char *email() const ;
  const unsigned long seq() const ;
  const char *typeString() const ;
  const char *typeTitleString() const ;
  const char *file() const ;
  unsigned long fileSize() const ;

  static const char *typeString( icqMsgType t );
  TimeStr time() const;
  int isNew() const;
  void expire();
  icqMsgType type() ;
  void setType(icqMsgType) ;
  int save(QString) ;
protected:
  icqMsgType msgType ;
  int msgNew;
  unsigned long msgUin;
  TimeStr msgTime;
  QString msgBody;
  QString msgURL;
  QString msgNick ;
  QString msgFirst ;
  QString msgLast ;
  QString msgEmail ;
  QString msgFile;
  unsigned long msgFileSize;
  unsigned long msgSeq;

  static const char *icqMsgTypeStrings[];
  static const char *icqMsgTypeTitles[];

};

#endif /* _ICQMESSAGE_ */
