/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
// $Id: icqsound.cpp,v 1.4 2000/07/21 19:14:02 denis Exp $
// $Log: icqsound.cpp,v $
// Revision 1.4  2000/07/21 19:14:02  denis
// Sound events were modified to be strict.
//
// Revision 1.3  2000/07/11 18:07:22  denis
// clear() function for setting all events to "none" was added.
// Some cleanups.
//
// Revision 1.2  2000/07/09 18:48:33  denis
// Sound events were renamed.
//
// Revision 1.1  2000/05/24 16:59:50  denis
// Sources have been splitted to the whole bunch of appropriate
// directories. Every directory is end up with static library. All
// of them are linked together to the final kicq binary.
//
// Revision 1.6  2000/01/10 16:08:23  denis
// Headers reorganized for simplify dependencies and speedup compilation.
//
// Revision 1.5  1999/12/02 14:22:54  denis
// Cleanups.
//
// Revision 1.4  1999/09/29 17:28:47  denis
// Cleanups.
//
// Revision 1.3  1999/07/16 15:48:03  denis
// Cleaned up.
//

#include <stdio.h>
#include <kapp.h>

#include "icqsound.h"
#include "mainwindow.h"

#include "icqsound.moc"

IcqSound::IcqSound()
{
  audio = new KAudio();

  if(!audio || audio->serverStatus())
  {
    // couldn't connect to audio server!
    delete audio;
    audio = 0;
  }

  eventList.append("Authorization Message");
  eventList.append("Chat Request");
  eventList.append("File Request");
  eventList.append("Startup");
  eventList.append("Incoming Message");
  eventList.append("Incoming URL");
  eventList.append("Online Alert");
  eventList.append("System Message");
  eventList.append("Message Sent");
  eventList.append("File Transfer Done");

  sounds.setAutoDelete(TRUE);
}

IcqSound::~IcqSound()
{
  delete audio;
}

void IcqSound::clear()
{
  sounds.clear();
  QStrListIterator it(eventList);
  while(it.current())
  {
    setEvent(it.current(), "(none)");
    ++it;
  }
}

void IcqSound::play(const char *soundfile)
{
  if(!strcmp(soundfile,"(beep)"))
    kapp->beep();
  else if(strcmp(soundfile,"(none)"))
  {
    QString fullsoundfile;

    if(soundfile[0]!='/')
    {
      fullsoundfile=kapp->localkdedir() + "/share/apps/" + kapp->appName() + "/sounds/" + soundfile;

      QFile checkfile(fullsoundfile);
      if(!checkfile.exists())
        fullsoundfile=kapp->kde_datadir()+"/"+kapp->appName()+"/sounds/" + soundfile;
    }
    else
      fullsoundfile = soundfile;

    if(audio)
      audio->play(fullsoundfile);
  }
}

void IcqSound::soundEvent(const char *event)
{
  if(MW->playWav)
    if(sounds[event])
      play(sounds[event]);
}

void IcqSound::operator>>(KSimpleConfig &pconfig)
{
  pconfig.setGroup("SoundEvents");

  QDictIterator<char> it(sounds);

  while(it.current())
  {
    pconfig.writeEntry(it.currentKey(), it.current());
    ++it;
  }
}

void IcqSound::operator<<(KSimpleConfig &pconfig)
{
  pconfig.setGroup("SoundEvents");

  QStrListIterator it(eventList);

  while(it.current())
  {
    QString str = pconfig.readEntry(it.current(),"(none)");
    setEvent(it.current(), str);
    ++it;
  }
}

void IcqSound::setEvent(const char *event, const char *file)
{
  sounds.replace(event, strdup(file));
}

const char *IcqSound::getEvent(const char *event)
{
  return sounds[event];
}
