/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
// $Id: wrapedit.cpp,v 1.2 2000/07/28 18:40:01 denis Exp $
// $Log: wrapedit.cpp,v $
// Revision 1.2  2000/07/28 18:40:01  denis
// Bug #100613 with hanging on wrapping long single word was fixed.
//
// Revision 1.1  2000/05/24 16:59:50  denis
// Sources have been splitted to the whole bunch of appropriate
// directories. Every directory is end up with static library. All
// of them are linked together to the final kicq binary.
//
// Revision 1.15  2000/03/30 15:06:50  denis
// ANSI fixes.
//
// Revision 1.14  2000/01/10 16:09:31  denis
// Headers reorganized for simplify dependencies and speedup compilation.
//
// Revision 1.13  1999/12/02 13:59:46  denis
// Cleanups.
//
// Revision 1.12  1999/10/15 16:56:36  guruz
// Various
//
// Revision 1.11  1999/10/14 13:01:18  denis
// Cleanups.
//
// Revision 1.10  1999/10/09 16:13:50  guruz
// Enhanced the Quick Phrase Feature. It now has an own Preferences Tab
//
// Revision 1.9  1999/10/06 19:12:26  denis
// Cleanups.
//
// Revision 1.8  1999/10/05 19:00:38  guruz
// -Added Message Timeout Patch by kyle@13th-floor.org
// -Fixed bug in wrapedit.cpp
//
// Revision 1.7  1999/10/05 17:42:40  guruz
// Changed some visual things
//
// Revision 1.6  1999/10/05 14:10:55  guruz
// -Base for a send window that allows to insert predefined phrases
// -Fixed Copy IP To Clipboard
//
// Revision 1.5  1999/07/16 15:49:08  denis
// Cleaned up.
//
// Revision 1.4  1999/04/04 21:51:51  bills
// modified keyPressEvent to allow for unlimited input
//
// Revision 1.3  1998/11/23 13:33:53  denis
// needUpdate added to avoid flickering
//
// Revision 1.2  1998/11/23 10:13:05  denis
// Buffer limit added
// Wrapping support enhanced
//

#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include <fstream.h>
#include <qkeycode.h>
#include <qpopupmenu.h>
#include <qstrlist.h>
#include <kconfig.h>
#include <kapp.h>

#include "wrapedit.h"

#include "wrapedit.moc"

WrapEdit::WrapEdit(QWidget *p, const char *n) : QMultiLineEdit(p,n) 
{
  popupmenu = new QPopupMenu;
  connect(popupmenu, SIGNAL(activated(int)), this, SLOT(activatedItem(int)));

  quickphrases_string_list = new QStrList;

  ifstream instream(kapp->localkdedir() + "/share/apps/kicq/quickphrases");
  static char buf[1024];
  QString StringToAdd;
  int i = 0;
  
  while(!instream.eof())
  {
    instream.getline(buf, sizeof buf - 1);
    if(instream.fail())
      break;
    else
    {
      i++;
      if(i > 10)
        break;
      StringToAdd = QString(buf);
      if(StringToAdd.stripWhiteSpace() != "")
      {
        popupmenu->insertItem(StringToAdd, 100+i);
        quickphrases_string_list->append(StringToAdd);
      }
    }
  }
}

WrapEdit::~WrapEdit() 
{
  delete popupmenu;
  popupmenu = 0L;

  delete quickphrases_string_list;
}

void WrapEdit::keyPressEvent(QKeyEvent *ev)
{
  if((ev->key() < 0x1000 || ev->key() == Key_Enter || ev->key() == Key_Return)
     && text().length()>=limit && limit != 0) // Do not accept more symbols!
  {
    kapp->beep();
    return;
  }

  QMultiLineEdit::keyPressEvent(ev);

  if(ev->key() != Key_Delete && ev->key() != Key_Backspace &&
     ev->key() != Key_Right && ev->key() != Key_Left &&
     ev->key() != Key_Down && ev->key() != Key_Up && ev->key() != Key_Next &&
     ev->key() != Key_Prior && ev->key() != Key_Home && ev->key() != Key_End &&
     wrap)
    wrapText();
}

void WrapEdit::resizeEvent(QResizeEvent *ev)
{
  QMultiLineEdit::resizeEvent(ev);
  wrapText(0);
}

void WrapEdit::quote()
{
}

void WrapEdit::setWraping(bool w)
{
  wrap = w;
}

bool WrapEdit::isWraping()
{
  return wrap;
}

void WrapEdit::setLimit(unsigned int l)
{
  limit = l;
}

unsigned int WrapEdit::getLimit()
{
  return limit;
}

void WrapEdit::wrapText(int r)
{
  int row, col, crow;
  int moveCursor;

  setAutoUpdate(FALSE);
  cursorPosition(&crow,&col);
  needUpdate = FALSE;
  for(row = r!=-1?r:crow; row < numLines(); row++)
    while(textWidth(row) + 5 > width())
    {
      /* Let's find last word on this line */
      char buffer[512];
      strncpy(buffer,textLine(row),512);

      char *p, *lastWord;
      p = &buffer[0] + strlen(buffer);
      while(*p == ' ' || *p == '\t')
        p--;
      while(*p != ' ' && *p != '\t' && p != &buffer[0])
        p--;

      if(p == &buffer[0])
        break;

      lastWord = p + 1;
      *p = '\0';

      /* Is the cursor somewhere around lastWord ? */
      if(strlen(buffer) + strlen(lastWord) + 1 == (unsigned int)col)
        moveCursor = 1;
      else
        moveCursor = 0;

      /* Replace current line */
      removeLine(row);
      insertLine(buffer,row);

      /* add lastWord to the start of next line */
      QString next = (QString)lastWord;
      QString prev;

/*      fprintf(stderr,"row = %d, col = %d, buffer = %d, lastWord = %d, lines = %d\n",
              row,col,strlen(buffer),strlen(lastWord),numLines());*/

      if(numLines() > row + 1)
      {
        if(numLines() > row)
          prev = textLine(row + 1);
        removeLine(row + 1);
        if(prev.length() > 0)
          next += " ";
        next += prev;
      }
      insertLine((const char *)next,row + 1);
      if(moveCursor)
        setCursorPosition(crow + 1, strlen(lastWord));
      else
        setCursorPosition(crow,col);
      needUpdate = TRUE;
    }
  setAutoUpdate(TRUE);
  if(needUpdate)
    update();
}

void WrapEdit::mousePressEvent(QMouseEvent *e) 
{ 
  QMultiLineEdit::mousePressEvent(e);
  if((e->button() == RightButton) && (popupmenu != NULL)) 
  {
    if ((!isReadOnly()) && (popupmenu->count() > 0))
      popupmenu->popup(QCursor::pos()); 
  }
}

void WrapEdit::activatedItem(int itemId)
{
  append(quickphrases_string_list->at(itemId-101));
}
