/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
// $Id: docking.cpp,v 1.2 2000/07/21 18:59:07 denis Exp $
// $Log: docking.cpp,v $
// Revision 1.2  2000/07/21 18:59:07  denis
// Move to active desktop on Restore menu command.
//
// Revision 1.1  2000/05/24 16:59:50  denis
// Sources have been splitted to the whole bunch of appropriate
// directories. Every directory is end up with static library. All
// of them are linked together to the final kicq binary.
//
// Revision 1.12  2000/05/05 03:37:49  bills
// kicq will now both dock and undock when double clicking on dock icon.  in
// addition, it will be automatically brought to the current desktop, if
// necessary
//
// Revision 1.11  2000/01/10 16:08:11  denis
// Headers reorganized for simplify dependencies and speedup compilation.
//
// Revision 1.10  1999/12/13 04:58:32  kyle
//
// docking.cpp - double-click on dock icon will now raise kicq window
//
// Revision 1.9  1999/12/12 18:08:06  denis
// Existing kapp->topWidget()'s chnaged to MW macros.
//
// Revision 1.8  1999/12/02 13:59:06  denis
// Cleanups.
//
// Revision 1.7  1999/07/16 15:47:49  denis
// Cleaned up.
//
// Revision 1.6  1999/07/16 13:01:46  denis
// Internalization support added.
//
// Revision 1.5  1998/12/08 16:15:03  denis
// setToolTip added
// Cleaned up a little before releasing
//

#include <qtooltip.h>
#include <kwm.h>
#include <kapp.h>

#include "mainwindow.h"
#include "docking.h"

#include "docking.moc"

DockWidget::DockWidget() : QWidget()
{
  docked = FALSE;
  doRemove = FALSE;
  dockPopup = new QPopupMenu();
  toggleID = dockPopup->insertItem(i18n("Restore"), this, SLOT(showWindow()));
  dockPopup->insertItem(i18n("Status"), MW->stateMenu());
  dockPopup->insertItem(i18n("Shutdown"), this, SLOT(emitQuit()));
  QToolTip::add(this, kapp->appName());
}

DockWidget::~DockWidget()
{
}

void DockWidget::dock()
{
  if(!docked)
  {
    KWM::setDockWindow(this->winId());
    this->setFixedSize(24, 24);
    show();
    paintIcon();
    update();
    docked = TRUE;
  }
}

void DockWidget::undock()
{
  if(docked)
  {
    // the widget's window has to be destroyed in order 
    // to undock from the panel. Simply using hide() is
    // not enough.
    this->destroy(TRUE, TRUE);
    // recreate window for further dockings
    this->create(0, TRUE, FALSE);
    docked = FALSE;
  }
}

const int DockWidget::isDocked()
{
  return docked;
}

void DockWidget::paintEvent(QPaintEvent*)
{
  paintIcon();
}

void DockWidget::paintIcon()
{
  bitBlt(this, (24-dockIcon.width())/2, (24-dockIcon.height())/2, &dockIcon);
}

void DockWidget::mouseDoubleClickEvent(QMouseEvent *e)
{
  if(e->button() == LeftButton)
  {
    if(MW->isUnreadMsgs())
      MW->readFirstRequest();
    else 
    {
      if(MW->isVisible())
        hideWindow();
      else
        showWindow();
      KWM::moveToDesktop(MW->winId(), KWM::currentDesktop());
    }
  }
}

void DockWidget::mousePressEvent(QMouseEvent *e)
{
  if(e->button() == RightButton || e->button() == MidButton)
  {
    int x = e->x();
    int y = e->y();
    QString text;
    if(MW->isUnreadMsgs())
    {
      if(!doRemove)
      {
        int readID = dockPopup->insertItem(i18n("Read Message"), -1, 0);
        dockPopup->connectItem(readID, MW, SLOT(readFirstRequest()));
        dockPopup->insertSeparator(1);
        doRemove = TRUE;
      }
    }
    else
      if(doRemove)
      {
        dockPopup->removeItemAt(1);
        dockPopup->removeItemAt(0);
        doRemove = FALSE;
      }
    if(MW->isVisible())
    {
      text = i18n("Dock");
      dockPopup->disconnectItem(toggleID, this, SLOT(showWindow()));
      dockPopup->connectItem(toggleID, this, SLOT(hideWindow()));
    }
    else
    {
      text = i18n("Restore");
      dockPopup->disconnectItem(toggleID, this, SLOT(hideWindow()));
      dockPopup->connectItem(toggleID, this, SLOT(showWindow()));
    }
    dockPopup->changeItem(text, toggleID);
    dockPopup->popup(mapToGlobal(QPoint(x, y)));
    dockPopup->exec();
  }
}

void DockWidget::hideWindow()
{
  if(MW->isVisible())
    MW->hide();
}

void DockWidget::showWindow()
{
  if(!MW->isVisible())
    MW->show();
  MW->raise();
  KWM::moveToDesktop(MW->winId(), KWM::currentDesktop());
}

void DockWidget::emitQuit()
{
  emit quitClicked();
}

void DockWidget::setIcon(QPixmap icon)
{
  dockIcon = icon;
  update();
}

void DockWidget::setToolTip(QString text)
{
  QToolTip::remove(this);
  QToolTip::add(this, (const char*)text);
}
